// A Bison parser, made by GNU Bison 3.8.2.

// Skeleton implementation for Bison LALR(1) parsers in C++

// Copyright (C) 2002-2015, 2018-2021 Free Software Foundation, Inc.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// As a special exception, you may create a larger work that contains
// part or all of the Bison parser skeleton and distribute that work
// under terms of your choice, so long as that work isn't itself a
// parser generator using the skeleton or a modified version thereof
// as a parser skeleton.  Alternatively, if you modify or redistribute
// the parser skeleton itself, you may (at your option) remove this
// special exception, which will cause the skeleton and the resulting
// Bison output files to be licensed under the GNU General Public
// License without this special exception.

// This special exception was added by the Free Software Foundation in
// version 2.2 of Bison.

// DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
// especially those whose name start with YY_ or yy_.  They are
// private implementation details that can be changed or removed.





#include "parser.hpp"


// Unqualified %code blocks.
#line 37 "asm/parser.y"

	#include <algorithm>
	#include <ctype.h>
	#include <inttypes.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <string_view>

	#include "extern/utf8decoder.hpp"
	#include "helpers.hpp"

	#include "asm/charmap.hpp"
	#include "asm/fixpoint.hpp"
	#include "asm/format.hpp"
	#include "asm/fstack.hpp"
	#include "asm/main.hpp"
	#include "asm/opt.hpp"
	#include "asm/output.hpp"
	#include "asm/section.hpp"
	#include "asm/symbol.hpp"
	#include "asm/warning.hpp"

	using namespace std::literals;

	yy::parser::symbol_type yylex(); // Provided by lexer.cpp

	static uint32_t strToNum(std::vector<int32_t> const &s);
	static void errorInvalidUTF8Byte(uint8_t byte, char const *functionName);
	static size_t strlenUTF8(std::string const &str, bool printErrors);
	static std::string strsliceUTF8(std::string const &str, uint32_t start, uint32_t stop);
	static std::string strsubUTF8(std::string const &str, uint32_t pos, uint32_t len);
	static size_t charlenUTF8(std::string const &str);
	static std::string strcharUTF8(std::string const &str, uint32_t idx);
	static std::string charsubUTF8(std::string const &str, uint32_t pos);
	static int32_t charcmp(std::string_view str1, std::string_view str2);
	static uint32_t adjustNegativeIndex(int32_t idx, size_t len, char const *functionName);
	static uint32_t adjustNegativePos(int32_t pos, size_t len, char const *functionName);
	static std::string strrpl(std::string_view str, std::string const &old, std::string const &rep);
	static std::string strfmt(
	    std::string const &spec, std::vector<Either<uint32_t, std::string>> const &args
	);
	static void compoundAssignment(std::string const &symName, RPNCommand op, int32_t constValue);
	static void failAssert(AssertionType type);
	static void failAssertMsg(AssertionType type, std::string const &message);

	template <typename N, typename S>
	static auto handleSymbolByType(std::string const &symName, N numCallback, S strCallback) {
		if (Symbol *sym = sym_FindScopedSymbol(symName); sym && sym->type == SYM_EQUS) {
			return strCallback(*sym->getEqus());
		} else {
			Expression expr;
			expr.makeSymbol(symName);
			return numCallback(expr);
		}
	}

	// The CPU encodes instructions in a logical way, so most instructions actually follow patterns.
	// These enums thus help with bit twiddling to compute opcodes.
	enum { REG_B, REG_C, REG_D, REG_E, REG_H, REG_L, REG_HL_IND, REG_A };

	enum { REG_BC_IND, REG_DE_IND, REG_HL_INDINC, REG_HL_INDDEC };

	// REG_AF == REG_SP since LD/INC/ADD/DEC allow SP, while PUSH/POP allow AF
	enum { REG_BC, REG_DE, REG_HL, REG_SP, REG_AF = REG_SP };
	// Names are not needed for AF or SP
	static char const *reg_tt_names[] = { "BC", "DE", "HL" };
	static char const *reg_tt_high_names[] = { "B", "D", "H" };
	static char const *reg_tt_low_names[] = { "C", "E", "L" };

	// CC_NZ == CC_Z ^ 1, and CC_NC == CC_C ^ 1, so `!` can toggle them
	enum { CC_NZ, CC_Z, CC_NC, CC_C };

#line 120 "/build/rgbds/src/rgbds/src/asm/parser.cpp"


#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> // FIXME: INFRINGES ON USER NAME SPACE.
#   define YY_(msgid) dgettext ("bison-runtime", msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(msgid) msgid
# endif
#endif


// Whether we are compiled with exception support.
#ifndef YY_EXCEPTIONS
# if defined __GNUC__ && !defined __EXCEPTIONS
#  define YY_EXCEPTIONS 0
# else
#  define YY_EXCEPTIONS 1
# endif
#endif



// Enable debugging if requested.
#if YYDEBUG

// A pseudo ostream that takes yydebug_ into account.
# define YYCDEBUG if (yydebug_) (*yycdebug_)

# define YY_SYMBOL_PRINT(Title, Symbol)         \
  do {                                          \
    if (yydebug_)                               \
    {                                           \
      *yycdebug_ << Title << ' ';               \
      yy_print_ (*yycdebug_, Symbol);           \
      *yycdebug_ << '\n';                       \
    }                                           \
  } while (false)

# define YY_REDUCE_PRINT(Rule)          \
  do {                                  \
    if (yydebug_)                       \
      yy_reduce_print_ (Rule);          \
  } while (false)

# define YY_STACK_PRINT()               \
  do {                                  \
    if (yydebug_)                       \
      yy_stack_print_ ();                \
  } while (false)

#else // !YYDEBUG

# define YYCDEBUG if (false) std::cerr
# define YY_SYMBOL_PRINT(Title, Symbol)  YY_USE (Symbol)
# define YY_REDUCE_PRINT(Rule)           static_cast<void> (0)
# define YY_STACK_PRINT()                static_cast<void> (0)

#endif // !YYDEBUG

#define yyerrok         (yyerrstatus_ = 0)
#define yyclearin       (yyla.clear ())

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab
#define YYRECOVERING()  (!!yyerrstatus_)

namespace yy {
#line 193 "/build/rgbds/src/rgbds/src/asm/parser.cpp"

  /// Build a parser object.
  parser::parser ()
#if YYDEBUG
    : yydebug_ (false),
      yycdebug_ (&std::cerr),
#else
    :
#endif
      yy_lac_established_ (false)
  {}

  parser::~parser ()
  {}

  parser::syntax_error::~syntax_error () YY_NOEXCEPT YY_NOTHROW
  {}

  /*---------.
  | symbol.  |
  `---------*/



  // by_state.
  parser::by_state::by_state () YY_NOEXCEPT
    : state (empty_state)
  {}

  parser::by_state::by_state (const by_state& that) YY_NOEXCEPT
    : state (that.state)
  {}

  void
  parser::by_state::clear () YY_NOEXCEPT
  {
    state = empty_state;
  }

  void
  parser::by_state::move (by_state& that)
  {
    state = that.state;
    that.clear ();
  }

  parser::by_state::by_state (state_type s) YY_NOEXCEPT
    : state (s)
  {}

  parser::symbol_kind_type
  parser::by_state::kind () const YY_NOEXCEPT
  {
    if (state == empty_state)
      return symbol_kind::S_YYEMPTY;
    else
      return YY_CAST (symbol_kind_type, yystos_[+state]);
  }

  parser::stack_symbol_type::stack_symbol_type ()
  {}

  parser::stack_symbol_type::stack_symbol_type (YY_RVREF (stack_symbol_type) that)
    : super_type (YY_MOVE (that.state))
  {
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.YY_MOVE_OR_COPY< AlignmentSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.YY_MOVE_OR_COPY< AssertionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.YY_MOVE_OR_COPY< Capture > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_reloc_3bit: // reloc_3bit
      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.YY_MOVE_OR_COPY< Expression > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_for_args: // for_args
        value.YY_MOVE_OR_COPY< ForArgs > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.YY_MOVE_OR_COPY< RPNCommand > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.YY_MOVE_OR_COPY< SectionModifier > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.YY_MOVE_OR_COPY< SectionSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.YY_MOVE_OR_COPY< SectionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.YY_MOVE_OR_COPY< StrFmtArgList > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_tt_no_af: // reg_tt_no_af
      case symbol_kind::S_reg_bc_or_de: // reg_bc_or_de
      case symbol_kind::S_reg_rr: // reg_rr
        value.YY_MOVE_OR_COPY< int32_t > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.YY_MOVE_OR_COPY< std::shared_ptr<MacroArgs> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_SYMBOL: // "symbol"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_LOCAL: // "local label"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_sym_no_anon: // scoped_sym_no_anon
      case symbol_kind::S_scoped_sym: // scoped_sym
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string_literal: // string_literal
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.YY_MOVE_OR_COPY< std::string > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.YY_MOVE_OR_COPY< std::vector<Expression> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.YY_MOVE_OR_COPY< std::vector<int32_t> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.YY_MOVE_OR_COPY< std::vector<std::string> > (YY_MOVE (that.value));
        break;

      default:
        break;
    }

#if 201103L <= YY_CPLUSPLUS
    // that is emptied.
    that.state = empty_state;
#endif
  }

  parser::stack_symbol_type::stack_symbol_type (state_type s, YY_MOVE_REF (symbol_type) that)
    : super_type (s)
  {
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.move< AlignmentSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.move< AssertionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.move< Capture > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_reloc_3bit: // reloc_3bit
      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.move< Expression > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_for_args: // for_args
        value.move< ForArgs > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.move< RPNCommand > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.move< SectionModifier > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.move< SectionSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.move< SectionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.move< StrFmtArgList > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_tt_no_af: // reg_tt_no_af
      case symbol_kind::S_reg_bc_or_de: // reg_bc_or_de
      case symbol_kind::S_reg_rr: // reg_rr
        value.move< int32_t > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.move< std::shared_ptr<MacroArgs> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_SYMBOL: // "symbol"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_LOCAL: // "local label"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_sym_no_anon: // scoped_sym_no_anon
      case symbol_kind::S_scoped_sym: // scoped_sym
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string_literal: // string_literal
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.move< std::string > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.move< std::vector<Expression> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.move< std::vector<int32_t> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.move< std::vector<std::string> > (YY_MOVE (that.value));
        break;

      default:
        break;
    }

    // that is emptied.
    that.kind_ = symbol_kind::S_YYEMPTY;
  }

#if YY_CPLUSPLUS < 201103L
  parser::stack_symbol_type&
  parser::stack_symbol_type::operator= (const stack_symbol_type& that)
  {
    state = that.state;
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.copy< AlignmentSpec > (that.value);
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.copy< AssertionType > (that.value);
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.copy< Capture > (that.value);
        break;

      case symbol_kind::S_reloc_3bit: // reloc_3bit
      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.copy< Expression > (that.value);
        break;

      case symbol_kind::S_for_args: // for_args
        value.copy< ForArgs > (that.value);
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.copy< RPNCommand > (that.value);
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.copy< SectionModifier > (that.value);
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.copy< SectionSpec > (that.value);
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.copy< SectionType > (that.value);
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.copy< StrFmtArgList > (that.value);
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_tt_no_af: // reg_tt_no_af
      case symbol_kind::S_reg_bc_or_de: // reg_bc_or_de
      case symbol_kind::S_reg_rr: // reg_rr
        value.copy< int32_t > (that.value);
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.copy< std::shared_ptr<MacroArgs> > (that.value);
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_SYMBOL: // "symbol"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_LOCAL: // "local label"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_sym_no_anon: // scoped_sym_no_anon
      case symbol_kind::S_scoped_sym: // scoped_sym
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string_literal: // string_literal
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.copy< std::string > (that.value);
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.copy< std::vector<Expression> > (that.value);
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.copy< std::vector<int32_t> > (that.value);
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.copy< std::vector<std::string> > (that.value);
        break;

      default:
        break;
    }

    return *this;
  }

  parser::stack_symbol_type&
  parser::stack_symbol_type::operator= (stack_symbol_type& that)
  {
    state = that.state;
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.move< AlignmentSpec > (that.value);
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.move< AssertionType > (that.value);
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.move< Capture > (that.value);
        break;

      case symbol_kind::S_reloc_3bit: // reloc_3bit
      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.move< Expression > (that.value);
        break;

      case symbol_kind::S_for_args: // for_args
        value.move< ForArgs > (that.value);
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.move< RPNCommand > (that.value);
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.move< SectionModifier > (that.value);
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.move< SectionSpec > (that.value);
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.move< SectionType > (that.value);
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.move< StrFmtArgList > (that.value);
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_tt_no_af: // reg_tt_no_af
      case symbol_kind::S_reg_bc_or_de: // reg_bc_or_de
      case symbol_kind::S_reg_rr: // reg_rr
        value.move< int32_t > (that.value);
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.move< std::shared_ptr<MacroArgs> > (that.value);
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_SYMBOL: // "symbol"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_LOCAL: // "local label"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_sym_no_anon: // scoped_sym_no_anon
      case symbol_kind::S_scoped_sym: // scoped_sym
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string_literal: // string_literal
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.move< std::string > (that.value);
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.move< std::vector<Expression> > (that.value);
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.move< std::vector<int32_t> > (that.value);
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.move< std::vector<std::string> > (that.value);
        break;

      default:
        break;
    }

    // that is emptied.
    that.state = empty_state;
    return *this;
  }
#endif

  template <typename Base>
  void
  parser::yy_destroy_ (const char* yymsg, basic_symbol<Base>& yysym) const
  {
    if (yymsg)
      YY_SYMBOL_PRINT (yymsg, yysym);
  }

#if YYDEBUG
  template <typename Base>
  void
  parser::yy_print_ (std::ostream& yyo, const basic_symbol<Base>& yysym) const
  {
    std::ostream& yyoutput = yyo;
    YY_USE (yyoutput);
    if (yysym.empty ())
      yyo << "empty symbol";
    else
      {
        symbol_kind_type yykind = yysym.kind ();
        yyo << (yykind < YYNTOKENS ? "token" : "nterm")
            << ' ' << yysym.name () << " (";
        YY_USE (yykind);
        yyo << ')';
      }
  }
#endif

  void
  parser::yypush_ (const char* m, YY_MOVE_REF (stack_symbol_type) sym)
  {
    if (m)
      YY_SYMBOL_PRINT (m, sym);
    yystack_.push (YY_MOVE (sym));
  }

  void
  parser::yypush_ (const char* m, state_type s, YY_MOVE_REF (symbol_type) sym)
  {
#if 201103L <= YY_CPLUSPLUS
    yypush_ (m, stack_symbol_type (s, std::move (sym)));
#else
    stack_symbol_type ss (s, sym);
    yypush_ (m, ss);
#endif
  }

  void
  parser::yypop_ (int n) YY_NOEXCEPT
  {
    yystack_.pop (n);
  }

#if YYDEBUG
  std::ostream&
  parser::debug_stream () const
  {
    return *yycdebug_;
  }

  void
  parser::set_debug_stream (std::ostream& o)
  {
    yycdebug_ = &o;
  }


  parser::debug_level_type
  parser::debug_level () const
  {
    return yydebug_;
  }

  void
  parser::set_debug_level (debug_level_type l)
  {
    yydebug_ = l;
  }
#endif // YYDEBUG

  parser::state_type
  parser::yy_lr_goto_state_ (state_type yystate, int yysym)
  {
    int yyr = yypgoto_[yysym - YYNTOKENS] + yystate;
    if (0 <= yyr && yyr <= yylast_ && yycheck_[yyr] == yystate)
      return yytable_[yyr];
    else
      return yydefgoto_[yysym - YYNTOKENS];
  }

  bool
  parser::yy_pact_value_is_default_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yypact_ninf_;
  }

  bool
  parser::yy_table_value_is_error_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yytable_ninf_;
  }

  int
  parser::operator() ()
  {
    return parse ();
  }

  int
  parser::parse ()
  {
    int yyn;
    /// Length of the RHS of the rule being reduced.
    int yylen = 0;

    // Error handling.
    int yynerrs_ = 0;
    int yyerrstatus_ = 0;

    /// The lookahead symbol.
    symbol_type yyla;

    /// The return value of parse ().
    int yyresult;

    // Discard the LAC context in case there still is one left from a
    // previous invocation.
    yy_lac_discard_ ("init");

#if YY_EXCEPTIONS
    try
#endif // YY_EXCEPTIONS
      {
    YYCDEBUG << "Starting parse\n";


    /* Initialize the stack.  The initial state will be set in
       yynewstate, since the latter expects the semantical and the
       location values to have been already stored, initialize these
       stacks with a primary value.  */
    yystack_.clear ();
    yypush_ (YY_NULLPTR, 0, YY_MOVE (yyla));

  /*-----------------------------------------------.
  | yynewstate -- push a new symbol on the stack.  |
  `-----------------------------------------------*/
  yynewstate:
    YYCDEBUG << "Entering state " << int (yystack_[0].state) << '\n';
    YY_STACK_PRINT ();

    // Accept?
    if (yystack_[0].state == yyfinal_)
      YYACCEPT;

    goto yybackup;


  /*-----------.
  | yybackup.  |
  `-----------*/
  yybackup:
    // Try to take a decision without lookahead.
    yyn = yypact_[+yystack_[0].state];
    if (yy_pact_value_is_default_ (yyn))
      goto yydefault;

    // Read a lookahead token.
    if (yyla.empty ())
      {
        YYCDEBUG << "Reading a token\n";
#if YY_EXCEPTIONS
        try
#endif // YY_EXCEPTIONS
          {
            symbol_type yylookahead (yylex ());
            yyla.move (yylookahead);
          }
#if YY_EXCEPTIONS
        catch (const syntax_error& yyexc)
          {
            YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
            error (yyexc);
            goto yyerrlab1;
          }
#endif // YY_EXCEPTIONS
      }
    YY_SYMBOL_PRINT ("Next token is", yyla);

    if (yyla.kind () == symbol_kind::S_YYerror)
    {
      // The scanner already issued an error message, process directly
      // to error recovery.  But do not keep the error token as
      // lookahead, it is too special and may lead us to an endless
      // loop in error recovery. */
      yyla.kind_ = symbol_kind::S_YYUNDEF;
      goto yyerrlab1;
    }

    /* If the proper action on seeing token YYLA.TYPE is to reduce or
       to detect an error, take that action.  */
    yyn += yyla.kind ();
    if (yyn < 0 || yylast_ < yyn || yycheck_[yyn] != yyla.kind ())
      {
        if (!yy_lac_establish_ (yyla.kind ()))
          goto yyerrlab;
        goto yydefault;
      }

    // Reduce or error.
    yyn = yytable_[yyn];
    if (yyn <= 0)
      {
        if (yy_table_value_is_error_ (yyn))
          goto yyerrlab;
        if (!yy_lac_establish_ (yyla.kind ()))
          goto yyerrlab;

        yyn = -yyn;
        goto yyreduce;
      }

    // Count tokens shifted since error; after three, turn off error status.
    if (yyerrstatus_)
      --yyerrstatus_;

    // Shift the lookahead token.
    yypush_ ("Shifting", state_type (yyn), YY_MOVE (yyla));
    yy_lac_discard_ ("shift");
    goto yynewstate;


  /*-----------------------------------------------------------.
  | yydefault -- do the default action for the current state.  |
  `-----------------------------------------------------------*/
  yydefault:
    yyn = yydefact_[+yystack_[0].state];
    if (yyn == 0)
      goto yyerrlab;
    goto yyreduce;


  /*-----------------------------.
  | yyreduce -- do a reduction.  |
  `-----------------------------*/
  yyreduce:
    yylen = yyr2_[yyn];
    {
      stack_symbol_type yylhs;
      yylhs.state = yy_lr_goto_state_ (yystack_[yylen].state, yyr1_[yyn]);
      /* Variants are always initialized to an empty instance of the
         correct type. The default '$$ = $1' action is NOT applied
         when using variants.  */
      switch (yyr1_[yyn])
    {
      case symbol_kind::S_align_spec: // align_spec
        yylhs.value.emplace< AlignmentSpec > ();
        break;

      case symbol_kind::S_assert_type: // assert_type
        yylhs.value.emplace< AssertionType > ();
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        yylhs.value.emplace< Capture > ();
        break;

      case symbol_kind::S_reloc_3bit: // reloc_3bit
      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        yylhs.value.emplace< Expression > ();
        break;

      case symbol_kind::S_for_args: // for_args
        yylhs.value.emplace< ForArgs > ();
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        yylhs.value.emplace< RPNCommand > ();
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        yylhs.value.emplace< SectionModifier > ();
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        yylhs.value.emplace< SectionSpec > ();
        break;

      case symbol_kind::S_sect_type: // sect_type
        yylhs.value.emplace< SectionType > ();
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        yylhs.value.emplace< StrFmtArgList > ();
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_tt_no_af: // reg_tt_no_af
      case symbol_kind::S_reg_bc_or_de: // reg_bc_or_de
      case symbol_kind::S_reg_rr: // reg_rr
        yylhs.value.emplace< int32_t > ();
        break;

      case symbol_kind::S_macro_args: // macro_args
        yylhs.value.emplace< std::shared_ptr<MacroArgs> > ();
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_SYMBOL: // "symbol"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_LOCAL: // "local label"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_sym_no_anon: // scoped_sym_no_anon
      case symbol_kind::S_scoped_sym: // scoped_sym
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string_literal: // string_literal
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        yylhs.value.emplace< std::string > ();
        break;

      case symbol_kind::S_ds_args: // ds_args
        yylhs.value.emplace< std::vector<Expression> > ();
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        yylhs.value.emplace< std::vector<int32_t> > ();
        break;

      case symbol_kind::S_purge_args: // purge_args
        yylhs.value.emplace< std::vector<std::string> > ();
        break;

      default:
        break;
    }



      // Perform the reduction.
      YY_REDUCE_PRINT (yyn);
#if YY_EXCEPTIONS
      try
#endif // YY_EXCEPTIONS
        {
          switch (yyn)
            {
  case 5: // $@1: %empty
#line 429 "asm/parser.y"
                {
		lexer_SetMode(LEXER_NORMAL);
		lexer_ToggleStringExpansion(true);
	}
#line 1138 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 6: // lines: error $@1 endofline
#line 432 "asm/parser.y"
                    {
		yyerrok;
	}
#line 1146 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 8: // diff_mark: "+"
#line 439 "asm/parser.y"
                 {
		::error(
		    "syntax error, unexpected + at the beginning of the line (is it a leftover diff mark?)\n"
		);
	}
#line 1156 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 9: // diff_mark: "-"
#line 444 "asm/parser.y"
                 {
		::error(
		    "syntax error, unexpected - at the beginning of the line (is it a leftover diff mark?)\n"
		);
	}
#line 1166 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 22: // if: "IF" iconst "end of line"
#line 477 "asm/parser.y"
                              {
		lexer_IncIFDepth();

		if (yystack_[1].value.as < int32_t > ()) {
			lexer_RunIFBlock();
		} else {
			lexer_SetMode(LEXER_SKIP_TO_ELIF);
		}
	}
#line 1180 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 23: // elif: "ELIF" iconst "end of line"
#line 489 "asm/parser.y"
                                {
		if (lexer_GetIFDepth() == 0) {
			fatalerror("Found ELIF outside of an IF construct\n");
		}
		if (lexer_RanIFBlock()) {
			if (lexer_ReachedELSEBlock()) {
				fatalerror("Found ELIF after an ELSE block\n");
			}
			lexer_SetMode(LEXER_SKIP_TO_ENDC);
		} else if (yystack_[1].value.as < int32_t > ()) {
			lexer_RunIFBlock();
		} else {
			lexer_SetMode(LEXER_SKIP_TO_ELIF);
		}
	}
#line 1200 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 24: // else: "ELSE" "end of line"
#line 507 "asm/parser.y"
                         {
		if (lexer_GetIFDepth() == 0) {
			fatalerror("Found ELSE outside of an IF construct\n");
		}
		if (lexer_RanIFBlock()) {
			if (lexer_ReachedELSEBlock()) {
				fatalerror("Found ELSE after an ELSE block\n");
			}
			lexer_SetMode(LEXER_SKIP_TO_ENDC);
		} else {
			lexer_RunIFBlock();
			lexer_ReachELSEBlock();
		}
	}
#line 1219 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 29: // endc: "ENDC"
#line 533 "asm/parser.y"
                 {
		lexer_DecIFDepth();
	}
#line 1227 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 30: // $@2: %empty
#line 539 "asm/parser.y"
               {
		lexer_ToggleStringExpansion(false);
	}
#line 1235 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 31: // def_id: "DEF" $@2 "symbol"
#line 541 "asm/parser.y"
                 {
		lexer_ToggleStringExpansion(true);
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1244 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 32: // $@3: %empty
#line 548 "asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1252 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 33: // redef_id: "REDEF" $@3 "symbol"
#line 550 "asm/parser.y"
                 {
		lexer_ToggleStringExpansion(true);
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1261 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 34: // scoped_sym_no_anon: "symbol"
#line 556 "asm/parser.y"
                    { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1267 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 35: // scoped_sym_no_anon: "label"
#line 556 "asm/parser.y"
                             { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1273 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 36: // scoped_sym_no_anon: "local label"
#line 556 "asm/parser.y"
                                     { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1279 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 37: // scoped_sym: scoped_sym_no_anon
#line 558 "asm/parser.y"
            { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1285 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 38: // scoped_sym: "anonymous label"
#line 558 "asm/parser.y"
                                 { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1291 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 40: // label: "label" ":"
#line 562 "asm/parser.y"
                      {
		sym_AddLabel(yystack_[1].value.as < std::string > ());
	}
#line 1299 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 41: // label: "label" "::"
#line 565 "asm/parser.y"
                             {
		sym_AddLabel(yystack_[1].value.as < std::string > ());
		sym_Export(yystack_[1].value.as < std::string > ());
	}
#line 1308 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 42: // label: "local label"
#line 569 "asm/parser.y"
                {
		sym_AddLocalLabel(yystack_[0].value.as < std::string > ());
	}
#line 1316 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 43: // label: "local label" ":"
#line 572 "asm/parser.y"
                      {
		sym_AddLocalLabel(yystack_[1].value.as < std::string > ());
	}
#line 1324 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 44: // label: "local label" "::"
#line 575 "asm/parser.y"
                             {
		sym_AddLocalLabel(yystack_[1].value.as < std::string > ());
		sym_Export(yystack_[1].value.as < std::string > ());
	}
#line 1333 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 45: // label: ":"
#line 579 "asm/parser.y"
                {
		sym_AddAnonLabel();
	}
#line 1341 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 46: // $@4: %empty
#line 585 "asm/parser.y"
               {
		// Parsing 'macro_args' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1350 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 47: // macro: "symbol" $@4 macro_args
#line 588 "asm/parser.y"
                     {
		fstk_RunMacro(yystack_[2].value.as < std::string > (), yystack_[0].value.as < std::shared_ptr<MacroArgs> > ());
	}
#line 1358 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 48: // macro_args: %empty
#line 594 "asm/parser.y"
               {
		yylhs.value.as < std::shared_ptr<MacroArgs> > () = std::make_shared<MacroArgs>();
	}
#line 1366 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 49: // macro_args: macro_args "string"
#line 597 "asm/parser.y"
                            {
		yylhs.value.as < std::shared_ptr<MacroArgs> > () = std::move(yystack_[1].value.as < std::shared_ptr<MacroArgs> > ());
		yylhs.value.as < std::shared_ptr<MacroArgs> > ()->appendArg(std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 1375 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 89: // def_numeric: def_equ
#line 646 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1381 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 90: // def_numeric: redef_equ
#line 647 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1387 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 91: // def_numeric: def_set
#line 648 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1393 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 92: // def_numeric: def_rb
#line 649 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1399 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 93: // def_numeric: def_rw
#line 650 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1405 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 94: // def_numeric: def_rl
#line 651 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1411 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 97: // compound_eq: "+="
#line 657 "asm/parser.y"
                  {
		yylhs.value.as < RPNCommand > () = RPN_ADD;
	}
#line 1419 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 98: // compound_eq: "-="
#line 660 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SUB;
	}
#line 1427 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 99: // compound_eq: "*="
#line 663 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_MUL;
	}
#line 1435 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 100: // compound_eq: "/="
#line 666 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_DIV;
	}
#line 1443 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 101: // compound_eq: "%="
#line 669 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_MOD;
	}
#line 1451 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 102: // compound_eq: "^="
#line 672 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_XOR;
	}
#line 1459 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 103: // compound_eq: "|="
#line 675 "asm/parser.y"
                   {
		yylhs.value.as < RPNCommand > () = RPN_OR;
	}
#line 1467 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 104: // compound_eq: "&="
#line 678 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_AND;
	}
#line 1475 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 105: // compound_eq: "<<="
#line 681 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SHL;
	}
#line 1483 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 106: // compound_eq: ">>="
#line 684 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SHR;
	}
#line 1491 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 107: // align: "ALIGN" align_spec
#line 690 "asm/parser.y"
                             {
		sect_AlignPC(yystack_[0].value.as < AlignmentSpec > ().alignment, yystack_[0].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1499 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 108: // align_spec: uconst
#line 696 "asm/parser.y"
               {
		if (yystack_[0].value.as < int32_t > () > 16) {
			::error("Alignment must be between 0 and 16, not %u\n", yystack_[0].value.as < int32_t > ());
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else {
			yylhs.value.as < AlignmentSpec > ().alignment = yystack_[0].value.as < int32_t > ();
			yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		}
	}
#line 1513 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 109: // align_spec: uconst "," iconst
#line 705 "asm/parser.y"
                              {
		if (yystack_[2].value.as < int32_t > () > 16) {
			::error("Alignment must be between 0 and 16, not %u\n", yystack_[2].value.as < int32_t > ());
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else if (yystack_[0].value.as < int32_t > () <= -(1 << yystack_[2].value.as < int32_t > ()) || yystack_[0].value.as < int32_t > () >= 1 << yystack_[2].value.as < int32_t > ()) {
			::error(
			    "The absolute alignment offset (%" PRIu32 ") must be less than alignment size (%d)\n",
			    static_cast<uint32_t>(yystack_[0].value.as < int32_t > () < 0 ? -yystack_[0].value.as < int32_t > () : yystack_[0].value.as < int32_t > ()),
			    1 << yystack_[2].value.as < int32_t > ()
			);
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else {
			yylhs.value.as < AlignmentSpec > ().alignment = yystack_[2].value.as < int32_t > ();
			yylhs.value.as < AlignmentSpec > ().alignOfs = yystack_[0].value.as < int32_t > () < 0 ? (1 << yystack_[2].value.as < int32_t > ()) + yystack_[0].value.as < int32_t > () : yystack_[0].value.as < int32_t > ();
		}
	}
#line 1534 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 110: // $@5: %empty
#line 724 "asm/parser.y"
                {
		// Parsing 'opt_list' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1543 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 114: // opt_list_entry: "string"
#line 736 "asm/parser.y"
               {
		opt_Parse(yystack_[0].value.as < std::string > ().c_str());
	}
#line 1551 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 115: // popo: "POPO"
#line 742 "asm/parser.y"
                 {
		opt_Pop();
	}
#line 1559 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 116: // $@6: %empty
#line 748 "asm/parser.y"
                  {
		opt_Push();
		// Parsing 'pusho_opt_list' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1569 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 118: // pusho_opt_list: %empty
#line 756 "asm/parser.y"
               {
		lexer_SetMode(LEXER_NORMAL);
	}
#line 1577 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 120: // pops: "POPS"
#line 763 "asm/parser.y"
                 {
		sect_PopSection();
	}
#line 1585 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 121: // pushs: "PUSHS"
#line 769 "asm/parser.y"
                  {
		sect_PushSection();
	}
#line 1593 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 122: // endsection: "ENDSECTION"
#line 775 "asm/parser.y"
                       {
		sect_EndSection();
	}
#line 1601 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 123: // fail: "FAIL" string
#line 781 "asm/parser.y"
                        {
		fatalerror("%s\n", yystack_[0].value.as < std::string > ().c_str());
	}
#line 1609 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 124: // warn: "WARN" string
#line 787 "asm/parser.y"
                        {
		warning(WARNING_USER, "%s\n", yystack_[0].value.as < std::string > ().c_str());
	}
#line 1617 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 125: // assert_type: %empty
#line 793 "asm/parser.y"
               {
		yylhs.value.as < AssertionType > () = ASSERT_ERROR;
	}
#line 1625 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 126: // assert_type: "WARN" ","
#line 796 "asm/parser.y"
                         {
		yylhs.value.as < AssertionType > () = ASSERT_WARN;
	}
#line 1633 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 127: // assert_type: "FAIL" ","
#line 799 "asm/parser.y"
                         {
		yylhs.value.as < AssertionType > () = ASSERT_ERROR;
	}
#line 1641 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 128: // assert_type: "FATAL" ","
#line 802 "asm/parser.y"
                          {
		yylhs.value.as < AssertionType > () = ASSERT_FATAL;
	}
#line 1649 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 129: // assert: "ASSERT" assert_type relocexpr
#line 808 "asm/parser.y"
                                         {
		if (!yystack_[0].value.as < Expression > ().isKnown()) {
			out_CreateAssert(yystack_[1].value.as < AssertionType > (), yystack_[0].value.as < Expression > (), "", sect_GetOutputOffset());
		} else if (yystack_[0].value.as < Expression > ().value() == 0) {
			failAssert(yystack_[1].value.as < AssertionType > ());
		}
	}
#line 1661 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 130: // assert: "ASSERT" assert_type relocexpr "," string
#line 815 "asm/parser.y"
                                                        {
		if (!yystack_[2].value.as < Expression > ().isKnown()) {
			out_CreateAssert(yystack_[3].value.as < AssertionType > (), yystack_[2].value.as < Expression > (), yystack_[0].value.as < std::string > (), sect_GetOutputOffset());
		} else if (yystack_[2].value.as < Expression > ().value() == 0) {
			failAssertMsg(yystack_[3].value.as < AssertionType > (), yystack_[0].value.as < std::string > ());
		}
	}
#line 1673 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 131: // assert: "STATIC_ASSERT" assert_type iconst
#line 822 "asm/parser.y"
                                               {
		if (yystack_[0].value.as < int32_t > () == 0) {
			failAssert(yystack_[1].value.as < AssertionType > ());
		}
	}
#line 1683 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 132: // assert: "STATIC_ASSERT" assert_type iconst "," string
#line 827 "asm/parser.y"
                                                            {
		if (yystack_[2].value.as < int32_t > () == 0) {
			failAssertMsg(yystack_[3].value.as < AssertionType > (), yystack_[0].value.as < std::string > ());
		}
	}
#line 1693 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 133: // shift: "SHIFT" shift_const
#line 835 "asm/parser.y"
                              {
		if (MacroArgs *macroArgs = fstk_GetCurrentMacroArgs(); macroArgs) {
			macroArgs->shiftArgs(yystack_[0].value.as < int32_t > ());
		} else {
			::error("Cannot shift macro arguments outside of a macro\n");
		}
	}
#line 1705 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 134: // shift_const: %empty
#line 845 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = 1;
	}
#line 1713 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 135: // shift_const: iconst
#line 848 "asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 1719 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 136: // load: "LOAD" sect_mod string "," sect_type sect_org sect_attrs
#line 852 "asm/parser.y"
                                                                     {
		sect_SetLoadSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 1727 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 137: // load: "ENDL"
#line 855 "asm/parser.y"
                   {
		sect_EndLoadSection(nullptr);
	}
#line 1735 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 138: // rept: "REPT" uconst "end of line" capture_rept endofline
#line 861 "asm/parser.y"
                                                       {
		if (yystack_[1].value.as < Capture > ().span.ptr) {
			fstk_RunRept(yystack_[3].value.as < int32_t > (), yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
		}
	}
#line 1745 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 139: // $@7: %empty
#line 869 "asm/parser.y"
                {
		lexer_ToggleStringExpansion(false);
	}
#line 1753 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 140: // $@8: %empty
#line 871 "asm/parser.y"
                 {
		lexer_ToggleStringExpansion(true);
	}
#line 1761 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 141: // for: "FOR" $@7 "symbol" $@8 "," for_args "end of line" capture_rept endofline
#line 873 "asm/parser.y"
                                                        {
		if (yystack_[1].value.as < Capture > ().span.ptr) {
			fstk_RunFor(yystack_[6].value.as < std::string > (), yystack_[3].value.as < ForArgs > ().start, yystack_[3].value.as < ForArgs > ().stop, yystack_[3].value.as < ForArgs > ().step, yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
		}
	}
#line 1771 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 142: // capture_rept: %empty
#line 881 "asm/parser.y"
               {
		yylhs.value.as < Capture > () = lexer_CaptureRept();
	}
#line 1779 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 143: // for_args: iconst
#line 887 "asm/parser.y"
               {
		yylhs.value.as < ForArgs > ().start = 0;
		yylhs.value.as < ForArgs > ().stop = yystack_[0].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = 1;
	}
#line 1789 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 144: // for_args: iconst "," iconst
#line 892 "asm/parser.y"
                              {
		yylhs.value.as < ForArgs > ().start = yystack_[2].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().stop = yystack_[0].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = 1;
	}
#line 1799 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 145: // for_args: iconst "," iconst "," iconst
#line 897 "asm/parser.y"
                                           {
		yylhs.value.as < ForArgs > ().start = yystack_[4].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().stop = yystack_[2].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = yystack_[0].value.as < int32_t > ();
	}
#line 1809 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 146: // break: label "BREAK" endofline
#line 905 "asm/parser.y"
                                  {
		if (fstk_Break()) {
			lexer_SetMode(LEXER_SKIP_TO_ENDR);
		}
	}
#line 1819 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 147: // $@9: %empty
#line 913 "asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1827 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 148: // $@10: %empty
#line 915 "asm/parser.y"
                 {
		lexer_ToggleStringExpansion(true);
	}
#line 1835 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 149: // def_macro: "MACRO" $@9 "symbol" $@10 "end of line" capture_macro endofline
#line 917 "asm/parser.y"
                                          {
		if (yystack_[1].value.as < Capture > ().span.ptr) {
			sym_AddMacro(yystack_[4].value.as < std::string > (), yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
		}
	}
#line 1845 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 150: // capture_macro: %empty
#line 925 "asm/parser.y"
               {
		yylhs.value.as < Capture > () = lexer_CaptureMacro();
	}
#line 1853 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 151: // rsset: "RSSET" uconst
#line 931 "asm/parser.y"
                         {
		sym_SetRSValue(yystack_[0].value.as < int32_t > ());
	}
#line 1861 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 152: // rsreset: "RSRESET"
#line 937 "asm/parser.y"
                    {
		sym_SetRSValue(0);
	}
#line 1869 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 153: // rs_uconst: %empty
#line 943 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = 1;
	}
#line 1877 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 154: // rs_uconst: uconst
#line 946 "asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 1883 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 155: // union: "UNION"
#line 950 "asm/parser.y"
                  {
		sect_StartUnion();
	}
#line 1891 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 156: // nextu: "NEXTU"
#line 956 "asm/parser.y"
                  {
		sect_NextUnionMember();
	}
#line 1899 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 157: // endu: "ENDU"
#line 962 "asm/parser.y"
                 {
		sect_EndUnion();
	}
#line 1907 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 158: // ds: "DS" uconst
#line 968 "asm/parser.y"
                      {
		sect_Skip(yystack_[0].value.as < int32_t > (), true);
	}
#line 1915 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 159: // ds: "DS" uconst "," ds_args trailing_comma
#line 971 "asm/parser.y"
                                                     {
		sect_RelBytes(yystack_[3].value.as < int32_t > (), yystack_[1].value.as < std::vector<Expression> > ());
	}
#line 1923 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 160: // ds: "DS" "ALIGN" "[" align_spec "]" trailing_comma
#line 974 "asm/parser.y"
                                                                   {
		uint32_t n = sect_GetAlignBytes(yystack_[2].value.as < AlignmentSpec > ().alignment, yystack_[2].value.as < AlignmentSpec > ().alignOfs);

		sect_Skip(n, true);
		sect_AlignPC(yystack_[2].value.as < AlignmentSpec > ().alignment, yystack_[2].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1934 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 161: // ds: "DS" "ALIGN" "[" align_spec "]" "," ds_args trailing_comma
#line 980 "asm/parser.y"
                                                                                 {
		uint32_t n = sect_GetAlignBytes(yystack_[4].value.as < AlignmentSpec > ().alignment, yystack_[4].value.as < AlignmentSpec > ().alignOfs);

		sect_RelBytes(n, yystack_[1].value.as < std::vector<Expression> > ());
		sect_AlignPC(yystack_[4].value.as < AlignmentSpec > ().alignment, yystack_[4].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1945 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 162: // ds_args: reloc_8bit
#line 989 "asm/parser.y"
                   {
		yylhs.value.as < std::vector<Expression> > ().push_back(std::move(yystack_[0].value.as < Expression > ()));
	}
#line 1953 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 163: // ds_args: ds_args "," reloc_8bit
#line 992 "asm/parser.y"
                                   {
		yylhs.value.as < std::vector<Expression> > () = std::move(yystack_[2].value.as < std::vector<Expression> > ());
		yylhs.value.as < std::vector<Expression> > ().push_back(std::move(yystack_[0].value.as < Expression > ()));
	}
#line 1962 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 164: // db: "DB"
#line 999 "asm/parser.y"
               {
		sect_Skip(1, false);
	}
#line 1970 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 166: // dw: "DW"
#line 1006 "asm/parser.y"
               {
		sect_Skip(2, false);
	}
#line 1978 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 168: // dl: "DL"
#line 1013 "asm/parser.y"
               {
		sect_Skip(4, false);
	}
#line 1986 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 170: // def_equ: def_id "EQU" iconst
#line 1020 "asm/parser.y"
                              {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddEqu(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1995 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 171: // redef_equ: redef_id "EQU" iconst
#line 1027 "asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_RedefEqu(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 2004 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 172: // def_set: def_id "=" iconst
#line 1034 "asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddVar(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 2013 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 173: // def_set: redef_id "=" iconst
#line 1038 "asm/parser.y"
                                    {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddVar(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 2022 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 174: // def_set: def_id compound_eq iconst
#line 1042 "asm/parser.y"
                                    {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		compoundAssignment(yylhs.value.as < std::string > (), yystack_[1].value.as < RPNCommand > (), yystack_[0].value.as < int32_t > ());
	}
#line 2031 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 175: // def_set: redef_id compound_eq iconst
#line 1046 "asm/parser.y"
                                      {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		compoundAssignment(yylhs.value.as < std::string > (), yystack_[1].value.as < RPNCommand > (), yystack_[0].value.as < int32_t > ());
	}
#line 2040 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 176: // def_rb: def_id "RB" rs_uconst
#line 1053 "asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + yystack_[0].value.as < int32_t > ());
	}
#line 2051 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 177: // def_rw: def_id "RW" rs_uconst
#line 1062 "asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + 2 * yystack_[0].value.as < int32_t > ());
	}
#line 2062 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 178: // def_rl: def_id "rl" rs_uconst
#line 1071 "asm/parser.y"
                                 {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + 4 * yystack_[0].value.as < int32_t > ());
	}
#line 2073 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 179: // def_equs: def_id "EQUS" string
#line 1080 "asm/parser.y"
                               {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddString(yylhs.value.as < std::string > (), std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 2082 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 180: // redef_equs: redef_id "EQUS" string
#line 1087 "asm/parser.y"
                                 {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_RedefString(yylhs.value.as < std::string > (), std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 2091 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 181: // $@11: %empty
#line 1094 "asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 2099 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 182: // purge: "PURGE" $@11 purge_args trailing_comma
#line 1096 "asm/parser.y"
                                    {
		for (std::string &arg : yystack_[1].value.as < std::vector<std::string> > ()) {
			sym_Purge(arg);
		}
		lexer_ToggleStringExpansion(true);
	}
#line 2110 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 183: // purge_args: scoped_sym_no_anon
#line 1105 "asm/parser.y"
                           {
		yylhs.value.as < std::vector<std::string> > ().push_back(yystack_[0].value.as < std::string > ());
	}
#line 2118 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 184: // purge_args: purge_args "," scoped_sym_no_anon
#line 1108 "asm/parser.y"
                                              {
		yylhs.value.as < std::vector<std::string> > () = std::move(yystack_[2].value.as < std::vector<std::string> > ());
		yylhs.value.as < std::vector<std::string> > ().push_back(yystack_[0].value.as < std::string > ());
	}
#line 2127 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 188: // export_list_entry: scoped_sym_no_anon
#line 1122 "asm/parser.y"
                           {
		sym_Export(yystack_[0].value.as < std::string > ());
	}
#line 2135 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 189: // export_def: "EXPORT" def_numeric
#line 1128 "asm/parser.y"
                               {
		sym_Export(yystack_[0].value.as < std::string > ());
	}
#line 2143 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 190: // include: label "INCLUDE" string endofline
#line 1134 "asm/parser.y"
                                           {
		fstk_RunInclude(yystack_[1].value.as < std::string > (), false);
		if (failedOnMissingInclude) {
			YYACCEPT;
		}
	}
#line 2154 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 191: // incbin: "INCBIN" string
#line 1143 "asm/parser.y"
                          {
		sect_BinaryFile(yystack_[0].value.as < std::string > (), 0);
		if (failedOnMissingInclude) {
			YYACCEPT;
		}
	}
#line 2165 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 192: // incbin: "INCBIN" string "," iconst
#line 1149 "asm/parser.y"
                                         {
		sect_BinaryFile(yystack_[2].value.as < std::string > (), yystack_[0].value.as < int32_t > ());
		if (failedOnMissingInclude) {
			YYACCEPT;
		}
	}
#line 2176 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 193: // incbin: "INCBIN" string "," iconst "," iconst
#line 1155 "asm/parser.y"
                                                      {
		sect_BinaryFileSlice(yystack_[4].value.as < std::string > (), yystack_[2].value.as < int32_t > (), yystack_[0].value.as < int32_t > ());
		if (failedOnMissingInclude) {
			YYACCEPT;
		}
	}
#line 2187 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 194: // charmap: "CHARMAP" string "," charmap_args trailing_comma
#line 1164 "asm/parser.y"
                                                             {
		charmap_Add(yystack_[3].value.as < std::string > (), std::move(yystack_[1].value.as < std::vector<int32_t> > ()));
	}
#line 2195 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 195: // charmap_args: iconst
#line 1170 "asm/parser.y"
               {
		yylhs.value.as < std::vector<int32_t> > ().push_back(std::move(yystack_[0].value.as < int32_t > ()));
	}
#line 2203 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 196: // charmap_args: charmap_args "," iconst
#line 1173 "asm/parser.y"
                                    {
		yylhs.value.as < std::vector<int32_t> > () = std::move(yystack_[2].value.as < std::vector<int32_t> > ());
		yylhs.value.as < std::vector<int32_t> > ().push_back(std::move(yystack_[0].value.as < int32_t > ()));
	}
#line 2212 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 197: // newcharmap: "NEWCHARMAP" "symbol"
#line 1180 "asm/parser.y"
                              {
		charmap_New(yystack_[0].value.as < std::string > (), nullptr);
	}
#line 2220 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 198: // newcharmap: "NEWCHARMAP" "symbol" "," "symbol"
#line 1183 "asm/parser.y"
                                             {
		charmap_New(yystack_[2].value.as < std::string > (), &yystack_[0].value.as < std::string > ());
	}
#line 2228 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 199: // setcharmap: "SETCHARMAP" "symbol"
#line 1189 "asm/parser.y"
                              {
		charmap_Set(yystack_[0].value.as < std::string > ());
	}
#line 2236 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 200: // pushc: "PUSHC"
#line 1195 "asm/parser.y"
                  {
		charmap_Push();
	}
#line 2244 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 201: // pushc_setcharmap: "PUSHC" "symbol"
#line 1201 "asm/parser.y"
                         {
		charmap_Push();
		charmap_Set(yystack_[0].value.as < std::string > ());
	}
#line 2253 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 202: // popc: "POPC"
#line 1208 "asm/parser.y"
                 {
		charmap_Pop();
	}
#line 2261 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 204: // println: "PRINTLN"
#line 1216 "asm/parser.y"
                    {
		putchar('\n');
		fflush(stdout);
	}
#line 2270 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 205: // println: "PRINTLN" print_exprs trailing_comma
#line 1220 "asm/parser.y"
                                                 {
		putchar('\n');
		fflush(stdout);
	}
#line 2279 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 208: // print_expr: relocexpr_no_str
#line 1232 "asm/parser.y"
                         {
		printf("$%" PRIX32, yystack_[0].value.as < Expression > ().getConstVal());
	}
#line 2287 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 209: // print_expr: string_literal
#line 1235 "asm/parser.y"
                         {
		// Allow printing NUL characters
		fwrite(yystack_[0].value.as < std::string > ().data(), 1, yystack_[0].value.as < std::string > ().length(), stdout);
	}
#line 2296 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 210: // print_expr: scoped_sym
#line 1239 "asm/parser.y"
                     {
		handleSymbolByType(
		    yystack_[0].value.as < std::string > (),
		    [](Expression const &expr) { printf("$%" PRIX32, expr.getConstVal()); },
		    [](std::string const &str) { fwrite(str.data(), 1, str.length(), stdout); }
		);
	}
#line 2308 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 211: // reloc_3bit: relocexpr
#line 1249 "asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(3);
	}
#line 2317 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 214: // constlist_8bit_entry: relocexpr_no_str
#line 1261 "asm/parser.y"
                         {
		yystack_[0].value.as < Expression > ().checkNBit(8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 0);
	}
#line 2326 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 215: // constlist_8bit_entry: string_literal
#line 1265 "asm/parser.y"
                         {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_ByteString(output);
	}
#line 2335 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 216: // constlist_8bit_entry: scoped_sym
#line 1269 "asm/parser.y"
                     {
		handleSymbolByType(
		    yystack_[0].value.as < std::string > (),
		    [](Expression const &expr) {
			    expr.checkNBit(8);
			    sect_RelByte(expr, 0);
		    },
		    [](std::string const &str) {
			    std::vector<int32_t> output = charmap_Convert(str);
			    sect_ByteString(output);
		    }
		);
	}
#line 2353 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 219: // constlist_16bit_entry: relocexpr_no_str
#line 1290 "asm/parser.y"
                         {
		yystack_[0].value.as < Expression > ().checkNBit(16);
		sect_RelWord(yystack_[0].value.as < Expression > (), 0);
	}
#line 2362 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 220: // constlist_16bit_entry: string_literal
#line 1294 "asm/parser.y"
                         {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_WordString(output);
	}
#line 2371 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 221: // constlist_16bit_entry: scoped_sym
#line 1298 "asm/parser.y"
                     {
		handleSymbolByType(
		    yystack_[0].value.as < std::string > (),
		    [](Expression const &expr) {
			    expr.checkNBit(16);
			    sect_RelWord(expr, 0);
		    },
		    [](std::string const &str) {
			    std::vector<int32_t> output = charmap_Convert(str);
			    sect_WordString(output);
		    }
		);
	}
#line 2389 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 224: // constlist_32bit_entry: relocexpr_no_str
#line 1319 "asm/parser.y"
                         {
		sect_RelLong(yystack_[0].value.as < Expression > (), 0);
	}
#line 2397 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 225: // constlist_32bit_entry: string_literal
#line 1322 "asm/parser.y"
                         {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_LongString(output);
	}
#line 2406 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 226: // constlist_32bit_entry: scoped_sym
#line 1326 "asm/parser.y"
                     {
		handleSymbolByType(
		    yystack_[0].value.as < std::string > (),
		    [](Expression const &expr) { sect_RelLong(expr, 0); },
		    [](std::string const &str) {
			    std::vector<int32_t> output = charmap_Convert(str);
			    sect_LongString(output);
		    }
		);
	}
#line 2421 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 227: // reloc_8bit: relocexpr
#line 1339 "asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2430 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 228: // reloc_8bit_offset: "+" relocexpr
#line 1346 "asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2439 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 229: // reloc_8bit_offset: "-" relocexpr
#line 1350 "asm/parser.y"
                           {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NEG, std::move(yystack_[0].value.as < Expression > ()));
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2448 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 230: // reloc_16bit: relocexpr
#line 1357 "asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(16);
	}
#line 2457 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 231: // relocexpr: relocexpr_no_str
#line 1364 "asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 2465 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 232: // relocexpr: string_literal
#line 1367 "asm/parser.y"
                         {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		yylhs.value.as < Expression > ().makeNumber(strToNum(output));
	}
#line 2474 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 233: // relocexpr: scoped_sym
#line 1371 "asm/parser.y"
                     {
		yylhs.value.as < Expression > () = handleSymbolByType(
		    yystack_[0].value.as < std::string > (),
		    [](Expression const &expr) { return expr; },
		    [](std::string const &str) {
			    std::vector<int32_t> output = charmap_Convert(str);
			    Expression expr;
			    expr.makeNumber(strToNum(output));
			    return expr;
		    }
		);
	}
#line 2491 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 234: // relocexpr_no_str: "number"
#line 1386 "asm/parser.y"
               {
		yylhs.value.as < Expression > ().makeNumber(yystack_[0].value.as < int32_t > ());
	}
#line 2499 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 235: // relocexpr_no_str: "!" relocexpr
#line 1389 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_LOGNOT, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2507 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 236: // relocexpr_no_str: relocexpr "||" relocexpr
#line 1392 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGOR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2515 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 237: // relocexpr_no_str: relocexpr "&&" relocexpr
#line 1395 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGAND, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2523 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 238: // relocexpr_no_str: relocexpr "==" relocexpr
#line 1398 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGEQ, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2531 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 239: // relocexpr_no_str: relocexpr ">" relocexpr
#line 1401 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGGT, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2539 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 240: // relocexpr_no_str: relocexpr "<" relocexpr
#line 1404 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGLT, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2547 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 241: // relocexpr_no_str: relocexpr ">=" relocexpr
#line 1407 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGGE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2555 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 242: // relocexpr_no_str: relocexpr "<=" relocexpr
#line 1410 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGLE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2563 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 243: // relocexpr_no_str: relocexpr "!=" relocexpr
#line 1413 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGNE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2571 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 244: // relocexpr_no_str: relocexpr "+" relocexpr
#line 1416 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_ADD, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2579 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 245: // relocexpr_no_str: relocexpr "-" relocexpr
#line 1419 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SUB, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2587 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 246: // relocexpr_no_str: relocexpr "^" relocexpr
#line 1422 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_XOR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2595 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 247: // relocexpr_no_str: relocexpr "|" relocexpr
#line 1425 "asm/parser.y"
                                    {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_OR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2603 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 248: // relocexpr_no_str: relocexpr "&" relocexpr
#line 1428 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_AND, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2611 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 249: // relocexpr_no_str: relocexpr "<<" relocexpr
#line 1431 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SHL, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2619 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 250: // relocexpr_no_str: relocexpr ">>" relocexpr
#line 1434 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SHR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2627 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 251: // relocexpr_no_str: relocexpr ">>>" relocexpr
#line 1437 "asm/parser.y"
                                      {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_USHR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2635 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 252: // relocexpr_no_str: relocexpr "*" relocexpr
#line 1440 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_MUL, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2643 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 253: // relocexpr_no_str: relocexpr "/" relocexpr
#line 1443 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_DIV, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2651 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 254: // relocexpr_no_str: relocexpr "%" relocexpr
#line 1446 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_MOD, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2659 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 255: // relocexpr_no_str: relocexpr "**" relocexpr
#line 1449 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_EXP, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2667 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 256: // relocexpr_no_str: "+" relocexpr
#line 1452 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 2675 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 257: // relocexpr_no_str: "-" relocexpr
#line 1455 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NEG, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2683 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 258: // relocexpr_no_str: "~" relocexpr
#line 1458 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NOT, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2691 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 259: // relocexpr_no_str: "HIGH" "(" relocexpr ")"
#line 1461 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_HIGH, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2699 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 260: // relocexpr_no_str: "LOW" "(" relocexpr ")"
#line 1464 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_LOW, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2707 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 261: // relocexpr_no_str: "BITWIDTH" "(" relocexpr ")"
#line 1467 "asm/parser.y"
                                              {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_BITWIDTH, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2715 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 262: // relocexpr_no_str: "TZCOUNT" "(" relocexpr ")"
#line 1470 "asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_TZCOUNT, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2723 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 263: // relocexpr_no_str: "ISCONST" "(" relocexpr ")"
#line 1473 "asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeNumber(yystack_[1].value.as < Expression > ().isKnown());
	}
#line 2731 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 264: // relocexpr_no_str: "BANK" "(" scoped_sym ")"
#line 1476 "asm/parser.y"
                                           {
		// '@' is also a SYMBOL; it is handled here
		yylhs.value.as < Expression > ().makeBankSymbol(yystack_[1].value.as < std::string > ());
	}
#line 2740 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 265: // relocexpr_no_str: "BANK" "(" string_literal ")"
#line 1480 "asm/parser.y"
                                               {
		yylhs.value.as < Expression > ().makeBankSection(yystack_[1].value.as < std::string > ());
	}
#line 2748 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 266: // relocexpr_no_str: "SIZEOF" "(" string ")"
#line 1483 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeSizeOfSection(yystack_[1].value.as < std::string > ());
	}
#line 2756 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 267: // relocexpr_no_str: "STARTOF" "(" string ")"
#line 1486 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeStartOfSection(yystack_[1].value.as < std::string > ());
	}
#line 2764 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 268: // relocexpr_no_str: "SIZEOF" "(" sect_type ")"
#line 1489 "asm/parser.y"
                                            {
		yylhs.value.as < Expression > ().makeSizeOfSectionType(yystack_[1].value.as < SectionType > ());
	}
#line 2772 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 269: // relocexpr_no_str: "STARTOF" "(" sect_type ")"
#line 1492 "asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeStartOfSectionType(yystack_[1].value.as < SectionType > ());
	}
#line 2780 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 270: // $@12: %empty
#line 1495 "asm/parser.y"
                 {
		lexer_ToggleStringExpansion(false);
	}
#line 2788 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 271: // relocexpr_no_str: "DEF" $@12 "(" scoped_sym ")"
#line 1497 "asm/parser.y"
                                   {
		yylhs.value.as < Expression > ().makeNumber(sym_FindScopedValidSymbol(yystack_[1].value.as < std::string > ()) != nullptr);
		lexer_ToggleStringExpansion(true);
	}
#line 2797 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 272: // relocexpr_no_str: "ROUND" "(" iconst precision_arg ")"
#line 1501 "asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(fix_Round(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2805 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 273: // relocexpr_no_str: "CEIL" "(" iconst precision_arg ")"
#line 1504 "asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_Ceil(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2813 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 274: // relocexpr_no_str: "FLOOR" "(" iconst precision_arg ")"
#line 1507 "asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(fix_Floor(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2821 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 275: // relocexpr_no_str: "FDIV" "(" iconst "," iconst precision_arg ")"
#line 1510 "asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Div(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2829 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 276: // relocexpr_no_str: "FMUL" "(" iconst "," iconst precision_arg ")"
#line 1513 "asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Mul(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2837 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 277: // relocexpr_no_str: "FMOD" "(" iconst "," iconst precision_arg ")"
#line 1516 "asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Mod(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2845 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 278: // relocexpr_no_str: "POW" "(" iconst "," iconst precision_arg ")"
#line 1519 "asm/parser.y"
                                                                 {
		yylhs.value.as < Expression > ().makeNumber(fix_Pow(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2853 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 279: // relocexpr_no_str: "LOG" "(" iconst "," iconst precision_arg ")"
#line 1522 "asm/parser.y"
                                                                 {
		yylhs.value.as < Expression > ().makeNumber(fix_Log(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2861 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 280: // relocexpr_no_str: "SIN" "(" iconst precision_arg ")"
#line 1525 "asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Sin(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2869 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 281: // relocexpr_no_str: "COS" "(" iconst precision_arg ")"
#line 1528 "asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Cos(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2877 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 282: // relocexpr_no_str: "TAN" "(" iconst precision_arg ")"
#line 1531 "asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Tan(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2885 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 283: // relocexpr_no_str: "ASIN" "(" iconst precision_arg ")"
#line 1534 "asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ASin(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2893 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 284: // relocexpr_no_str: "ACOS" "(" iconst precision_arg ")"
#line 1537 "asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ACos(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2901 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 285: // relocexpr_no_str: "ATAN" "(" iconst precision_arg ")"
#line 1540 "asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ATan(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2909 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 286: // relocexpr_no_str: "ATAN2" "(" iconst "," iconst precision_arg ")"
#line 1543 "asm/parser.y"
                                                                   {
		yylhs.value.as < Expression > ().makeNumber(fix_ATan2(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2917 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 287: // relocexpr_no_str: "STRCMP" "(" string "," string ")"
#line 1546 "asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(yystack_[3].value.as < std::string > ().compare(yystack_[1].value.as < std::string > ()));
	}
#line 2925 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 288: // relocexpr_no_str: "STRFIND" "(" string "," string ")"
#line 1549 "asm/parser.y"
                                                       {
		size_t pos = yystack_[3].value.as < std::string > ().find(yystack_[1].value.as < std::string > ());
		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos : -1);
	}
#line 2934 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 289: // relocexpr_no_str: "STRRFIND" "(" string "," string ")"
#line 1553 "asm/parser.y"
                                                        {
		size_t pos = yystack_[3].value.as < std::string > ().rfind(yystack_[1].value.as < std::string > ());
		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos : -1);
	}
#line 2943 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 290: // relocexpr_no_str: "STRIN" "(" string "," string ")"
#line 1557 "asm/parser.y"
                                                     {
		size_t pos = yystack_[3].value.as < std::string > ().find(yystack_[1].value.as < std::string > ());
		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos + 1 : 0);
	}
#line 2952 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 291: // relocexpr_no_str: "STRRIN" "(" string "," string ")"
#line 1561 "asm/parser.y"
                                                      {
		size_t pos = yystack_[3].value.as < std::string > ().rfind(yystack_[1].value.as < std::string > ());
		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos + 1 : 0);
	}
#line 2961 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 292: // relocexpr_no_str: "STRLEN" "(" string ")"
#line 1565 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeNumber(strlenUTF8(yystack_[1].value.as < std::string > (), true));
	}
#line 2969 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 293: // relocexpr_no_str: "CHARLEN" "(" string ")"
#line 1568 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeNumber(charlenUTF8(yystack_[1].value.as < std::string > ()));
	}
#line 2977 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 294: // relocexpr_no_str: "INCHARMAP" "(" string ")"
#line 1571 "asm/parser.y"
                                            {
		yylhs.value.as < Expression > ().makeNumber(charmap_HasChar(yystack_[1].value.as < std::string > ()));
	}
#line 2985 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 295: // relocexpr_no_str: "CHARCMP" "(" string "," string ")"
#line 1574 "asm/parser.y"
                                                       {
		yylhs.value.as < Expression > ().makeNumber(charcmp(yystack_[3].value.as < std::string > (), yystack_[1].value.as < std::string > ()));
	}
#line 2993 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 296: // relocexpr_no_str: "CHARSIZE" "(" string ")"
#line 1577 "asm/parser.y"
                                           {
		size_t charSize = charmap_CharSize(yystack_[1].value.as < std::string > ());
		if (charSize == 0) {
			::error("CHARSIZE: No character mapping for \"%s\"\n", yystack_[1].value.as < std::string > ().c_str());
		}
		yylhs.value.as < Expression > ().makeNumber(charSize);
	}
#line 3005 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 297: // relocexpr_no_str: "CHARVAL" "(" string "," iconst ")"
#line 1584 "asm/parser.y"
                                                       {
		if (size_t len = charmap_CharSize(yystack_[3].value.as < std::string > ()); len != 0) {
			uint32_t idx = adjustNegativeIndex(yystack_[1].value.as < int32_t > (), len, "CHARVAL");
			if (std::optional<int32_t> val = charmap_CharValue(yystack_[3].value.as < std::string > (), idx); val.has_value()) {
				yylhs.value.as < Expression > ().makeNumber(*val);
			} else {
				warning(
				    WARNING_BUILTIN_ARG,
				    "CHARVAL: Index %" PRIu32 " is past the end of the character mapping\n",
				    idx
				);
				yylhs.value.as < Expression > ().makeNumber(0);
			}
		} else {
			::error("CHARVAL: No character mapping for \"%s\"\n", yystack_[3].value.as < std::string > ().c_str());
			yylhs.value.as < Expression > ().makeNumber(0);
		}
	}
#line 3028 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 298: // relocexpr_no_str: "(" relocexpr ")"
#line 1602 "asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
	}
#line 3036 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 299: // uconst: iconst
#line 1608 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0) {
			fatalerror("Constant must not be negative: %d\n", yylhs.value.as < int32_t > ());
		}
	}
#line 3047 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 300: // iconst: relocexpr
#line 1617 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < Expression > ().getConstVal();
	}
#line 3055 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 301: // precision_arg: %empty
#line 1623 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = fix_Precision();
	}
#line 3063 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 302: // precision_arg: "," iconst
#line 1626 "asm/parser.y"
                       {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 1 || yylhs.value.as < int32_t > () > 31) {
			::error("Fixed-point precision must be between 1 and 31, not %" PRId32 "\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = fix_Precision();
		}
	}
#line 3075 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 303: // string_literal: "string"
#line 1636 "asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 3083 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 304: // string_literal: string "++" string
#line 1639 "asm/parser.y"
                               {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		yylhs.value.as < std::string > ().append(yystack_[0].value.as < std::string > ());
	}
#line 3092 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 305: // string_literal: "STRSLICE" "(" string "," iconst "," iconst ")"
#line 1643 "asm/parser.y"
                                                                     {
		size_t len = strlenUTF8(yystack_[5].value.as < std::string > (), false);
		uint32_t start = adjustNegativeIndex(yystack_[3].value.as < int32_t > (), len, "STRSLICE");
		uint32_t stop = adjustNegativeIndex(yystack_[1].value.as < int32_t > (), len, "STRSLICE");
		yylhs.value.as < std::string > () = strsliceUTF8(yystack_[5].value.as < std::string > (), start, stop);
	}
#line 3103 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 306: // string_literal: "STRSLICE" "(" string "," iconst ")"
#line 1649 "asm/parser.y"
                                                        {
		size_t len = strlenUTF8(yystack_[3].value.as < std::string > (), false);
		uint32_t start = adjustNegativeIndex(yystack_[1].value.as < int32_t > (), len, "STRSLICE");
		yylhs.value.as < std::string > () = strsliceUTF8(yystack_[3].value.as < std::string > (), start, len);
	}
#line 3113 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 307: // string_literal: "STRSUB" "(" string "," iconst "," uconst ")"
#line 1654 "asm/parser.y"
                                                                   {
		size_t len = strlenUTF8(yystack_[5].value.as < std::string > (), false);
		uint32_t pos = adjustNegativePos(yystack_[3].value.as < int32_t > (), len, "STRSUB");
		yylhs.value.as < std::string > () = strsubUTF8(yystack_[5].value.as < std::string > (), pos, yystack_[1].value.as < int32_t > ());
	}
#line 3123 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 308: // string_literal: "STRSUB" "(" string "," iconst ")"
#line 1659 "asm/parser.y"
                                                      {
		size_t len = strlenUTF8(yystack_[3].value.as < std::string > (), false);
		uint32_t pos = adjustNegativePos(yystack_[1].value.as < int32_t > (), len, "STRSUB");
		yylhs.value.as < std::string > () = strsubUTF8(yystack_[3].value.as < std::string > (), pos, pos > len ? 0 : len + 1 - pos);
	}
#line 3133 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 309: // string_literal: "STRCHAR" "(" string "," iconst ")"
#line 1664 "asm/parser.y"
                                                       {
		size_t len = charlenUTF8(yystack_[3].value.as < std::string > ());
		uint32_t idx = adjustNegativeIndex(yystack_[1].value.as < int32_t > (), len, "STRCHAR");
		yylhs.value.as < std::string > () = strcharUTF8(yystack_[3].value.as < std::string > (), idx);
	}
#line 3143 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 310: // string_literal: "CHARSUB" "(" string "," iconst ")"
#line 1669 "asm/parser.y"
                                                       {
		size_t len = charlenUTF8(yystack_[3].value.as < std::string > ());
		uint32_t pos = adjustNegativePos(yystack_[1].value.as < int32_t > (), len, "CHARSUB");
		yylhs.value.as < std::string > () = charsubUTF8(yystack_[3].value.as < std::string > (), pos);
	}
#line 3153 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 311: // string_literal: "REVCHAR" "(" charmap_args ")"
#line 1674 "asm/parser.y"
                                                {
		bool unique;
		yylhs.value.as < std::string > () = charmap_Reverse(yystack_[1].value.as < std::vector<int32_t> > (), unique);
		if (!unique) {
			::error("REVCHAR: Multiple character mappings to values\n");
		} else if (yylhs.value.as < std::string > ().empty()) {
			::error("REVCHAR: No character mapping to values\n");
		}
	}
#line 3167 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 312: // string_literal: "STRCAT" "(" ")"
#line 1683 "asm/parser.y"
                                  {
		yylhs.value.as < std::string > ().clear();
	}
#line 3175 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 313: // string_literal: "STRCAT" "(" strcat_args ")"
#line 1686 "asm/parser.y"
                                              {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
	}
#line 3183 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 314: // string_literal: "STRUPR" "(" string ")"
#line 1689 "asm/parser.y"
                                         {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
		std::transform(RANGE(yylhs.value.as < std::string > ()), yylhs.value.as < std::string > ().begin(), [](char c) { return toupper(c); });
	}
#line 3192 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 315: // string_literal: "STRLWR" "(" string ")"
#line 1693 "asm/parser.y"
                                         {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
		std::transform(RANGE(yylhs.value.as < std::string > ()), yylhs.value.as < std::string > ().begin(), [](char c) { return tolower(c); });
	}
#line 3201 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 316: // string_literal: "STRRPL" "(" string "," string "," string ")"
#line 1697 "asm/parser.y"
                                                                   {
		yylhs.value.as < std::string > () = strrpl(yystack_[5].value.as < std::string > (), yystack_[3].value.as < std::string > (), yystack_[1].value.as < std::string > ());
	}
#line 3209 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 317: // string_literal: "STRFMT" "(" strfmt_args ")"
#line 1700 "asm/parser.y"
                                              {
		yylhs.value.as < std::string > () = strfmt(yystack_[1].value.as < StrFmtArgList > ().format, yystack_[1].value.as < StrFmtArgList > ().args);
	}
#line 3217 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 318: // string_literal: "SECTION" "(" scoped_sym ")"
#line 1703 "asm/parser.y"
                                               {
		Symbol *sym = sym_FindScopedValidSymbol(yystack_[1].value.as < std::string > ());

		if (!sym) {
			if (sym_IsPurgedScoped(yystack_[1].value.as < std::string > ())) {
				fatalerror("Unknown symbol \"%s\"; it was purged\n", yystack_[1].value.as < std::string > ().c_str());
			} else {
				fatalerror("Unknown symbol \"%s\"\n", yystack_[1].value.as < std::string > ().c_str());
			}
		}
		Section const *section = sym->getSection();

		if (!section) {
			fatalerror("\"%s\" does not belong to any section\n", sym->name.c_str());
		}
		// Section names are capped by rgbasm's maximum string length,
		// so this currently can't overflow.
		yylhs.value.as < std::string > () = section->name;
	}
#line 3241 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 319: // string: string_literal
#line 1725 "asm/parser.y"
                       {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 3249 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 320: // string: scoped_sym
#line 1728 "asm/parser.y"
                     {
		if (Symbol *sym = sym_FindScopedSymbol(yystack_[0].value.as < std::string > ()); sym && sym->type == SYM_EQUS) {
			yylhs.value.as < std::string > () = *sym->getEqus();
		} else {
			::error("'%s' is not a string symbol\n", yystack_[0].value.as < std::string > ().c_str());
		}
	}
#line 3261 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 321: // strcat_args: string
#line 1738 "asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 3269 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 322: // strcat_args: strcat_args "," string
#line 1741 "asm/parser.y"
                                   {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		yylhs.value.as < std::string > ().append(yystack_[0].value.as < std::string > ());
	}
#line 3278 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 323: // strfmt_args: %empty
#line 1748 "asm/parser.y"
                 {}
#line 3284 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 324: // strfmt_args: string strfmt_va_args
#line 1749 "asm/parser.y"
                                {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[0].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().format = std::move(yystack_[1].value.as < std::string > ());
	}
#line 3293 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 325: // strfmt_va_args: %empty
#line 1756 "asm/parser.y"
                 {}
#line 3299 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 326: // strfmt_va_args: strfmt_va_args "," relocexpr_no_str
#line 1757 "asm/parser.y"
                                                {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().args.push_back(static_cast<uint32_t>(yystack_[0].value.as < Expression > ().getConstVal()));
	}
#line 3308 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 327: // strfmt_va_args: strfmt_va_args "," string_literal
#line 1761 "asm/parser.y"
                                              {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().args.push_back(std::move(yystack_[0].value.as < std::string > ()));
	}
#line 3317 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 328: // strfmt_va_args: strfmt_va_args "," scoped_sym
#line 1765 "asm/parser.y"
                                          {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		handleSymbolByType(
		    yystack_[0].value.as < std::string > (),
		    [&](Expression const &expr) {
			    yylhs.value.as < StrFmtArgList > ().args.push_back(static_cast<uint32_t>(expr.getConstVal()));
		    },
		    [&](std::string const &str) { yylhs.value.as < StrFmtArgList > ().args.push_back(str); }
		);
	}
#line 3332 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 329: // section: "SECTION" sect_mod string "," sect_type sect_org sect_attrs
#line 1778 "asm/parser.y"
                                                                        {
		sect_NewSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 3340 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 330: // pushs_section: "PUSHS" sect_mod string "," sect_type sect_org sect_attrs
#line 1784 "asm/parser.y"
                                                                      {
		sect_PushSection();
		sect_NewSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 3349 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 331: // sect_mod: %empty
#line 1791 "asm/parser.y"
               {
		yylhs.value.as < SectionModifier > () = SECTION_NORMAL;
	}
#line 3357 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 332: // sect_mod: "UNION"
#line 1794 "asm/parser.y"
                    {
		yylhs.value.as < SectionModifier > () = SECTION_UNION;
	}
#line 3365 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 333: // sect_mod: "FRAGMENT"
#line 1797 "asm/parser.y"
                       {
		yylhs.value.as < SectionModifier > () = SECTION_FRAGMENT;
	}
#line 3373 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 334: // sect_type: "WRAM0"
#line 1803 "asm/parser.y"
                   {
		yylhs.value.as < SectionType > () = SECTTYPE_WRAM0;
	}
#line 3381 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 335: // sect_type: "VRAM"
#line 1806 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_VRAM;
	}
#line 3389 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 336: // sect_type: "ROMX"
#line 1809 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_ROMX;
	}
#line 3397 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 337: // sect_type: "ROM0"
#line 1812 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_ROM0;
	}
#line 3405 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 338: // sect_type: "HRAM"
#line 1815 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_HRAM;
	}
#line 3413 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 339: // sect_type: "WRAMX"
#line 1818 "asm/parser.y"
                     {
		yylhs.value.as < SectionType > () = SECTTYPE_WRAMX;
	}
#line 3421 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 340: // sect_type: "SRAM"
#line 1821 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_SRAM;
	}
#line 3429 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 341: // sect_type: "OAM"
#line 1824 "asm/parser.y"
                   {
		yylhs.value.as < SectionType > () = SECTTYPE_OAM;
	}
#line 3437 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 342: // sect_org: %empty
#line 1830 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = -1;
	}
#line 3445 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 343: // sect_org: "[" uconst "]"
#line 1833 "asm/parser.y"
                               {
		yylhs.value.as < int32_t > () = yystack_[1].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0 || yylhs.value.as < int32_t > () > 0xFFFF) {
			::error("Address $%x is not 16-bit\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = -1;
		}
	}
#line 3457 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 344: // sect_attrs: %empty
#line 1843 "asm/parser.y"
               {
		yylhs.value.as < SectionSpec > ().alignment = 0;
		yylhs.value.as < SectionSpec > ().alignOfs = 0;
		yylhs.value.as < SectionSpec > ().bank = -1;
	}
#line 3467 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 345: // sect_attrs: sect_attrs "," "ALIGN" "[" align_spec "]"
#line 1848 "asm/parser.y"
                                                              {
		yylhs.value.as < SectionSpec > () = yystack_[5].value.as < SectionSpec > ();
		yylhs.value.as < SectionSpec > ().alignment = yystack_[1].value.as < AlignmentSpec > ().alignment;
		yylhs.value.as < SectionSpec > ().alignOfs = yystack_[1].value.as < AlignmentSpec > ().alignOfs;
	}
#line 3477 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 346: // sect_attrs: sect_attrs "," "BANK" "[" uconst "]"
#line 1853 "asm/parser.y"
                                                        {
		yylhs.value.as < SectionSpec > () = yystack_[5].value.as < SectionSpec > ();
		yylhs.value.as < SectionSpec > ().bank = yystack_[1].value.as < int32_t > (); // We cannot check the validity of this yet
	}
#line 3486 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 395: // sm83_adc: "adc" op_a_n
#line 1916 "asm/parser.y"
                        {
		sect_ConstByte(0xCE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3495 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 396: // sm83_adc: "adc" op_a_r
#line 1920 "asm/parser.y"
                          {
		sect_ConstByte(0x88 | yystack_[0].value.as < int32_t > ());
	}
#line 3503 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 397: // sm83_add: "add" op_a_n
#line 1926 "asm/parser.y"
                        {
		sect_ConstByte(0xC6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3512 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 398: // sm83_add: "add" op_a_r
#line 1930 "asm/parser.y"
                          {
		sect_ConstByte(0x80 | yystack_[0].value.as < int32_t > ());
	}
#line 3520 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 399: // sm83_add: "add" "hl" "," reg_ss
#line 1933 "asm/parser.y"
                                        {
		sect_ConstByte(0x09 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3528 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 400: // sm83_add: "add" "sp" "," reloc_8bit
#line 1936 "asm/parser.y"
                                            {
		sect_ConstByte(0xE8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3537 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 401: // sm83_and: "and" op_a_n
#line 1943 "asm/parser.y"
                        {
		sect_ConstByte(0xE6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3546 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 402: // sm83_and: "and" op_a_r
#line 1947 "asm/parser.y"
                          {
		sect_ConstByte(0xA0 | yystack_[0].value.as < int32_t > ());
	}
#line 3554 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 403: // sm83_bit: "bit" reloc_3bit "," reg_r
#line 1953 "asm/parser.y"
                                        {
		uint8_t mask = static_cast<uint8_t>(0x40 | yystack_[0].value.as < int32_t > ());
		yystack_[2].value.as < Expression > ().makeCheckBitIndex(mask);
		sect_ConstByte(0xCB);
		if (!yystack_[2].value.as < Expression > ().isKnown()) {
			sect_RelByte(yystack_[2].value.as < Expression > (), 0);
		} else {
			sect_ConstByte(mask | (yystack_[2].value.as < Expression > ().value() << 3));
		}
	}
#line 3569 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 404: // sm83_call: "call" reloc_16bit
#line 1966 "asm/parser.y"
                              {
		sect_ConstByte(0xCD);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3578 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 405: // sm83_call: "call" ccode_expr "," reloc_16bit
#line 1970 "asm/parser.y"
                                                 {
		sect_ConstByte(0xC4 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3587 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 406: // sm83_ccf: "ccf"
#line 1977 "asm/parser.y"
                 {
		sect_ConstByte(0x3F);
	}
#line 3595 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 407: // sm83_cp: "cp" op_a_n
#line 1983 "asm/parser.y"
                       {
		sect_ConstByte(0xFE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3604 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 408: // sm83_cp: "cp" op_a_r
#line 1987 "asm/parser.y"
                         {
		sect_ConstByte(0xB8 | yystack_[0].value.as < int32_t > ());
	}
#line 3612 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 409: // sm83_cpl: "cpl"
#line 1993 "asm/parser.y"
                 {
		sect_ConstByte(0x2F);
	}
#line 3620 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 410: // sm83_cpl: "cpl" MODE_A
#line 1996 "asm/parser.y"
                          {
		sect_ConstByte(0x2F);
	}
#line 3628 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 411: // sm83_daa: "daa"
#line 2002 "asm/parser.y"
                 {
		sect_ConstByte(0x27);
	}
#line 3636 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 412: // sm83_dec: "dec" reg_r
#line 2008 "asm/parser.y"
                       {
		sect_ConstByte(0x05 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3644 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 413: // sm83_dec: "dec" reg_ss
#line 2011 "asm/parser.y"
                          {
		sect_ConstByte(0x0B | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3652 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 414: // sm83_di: "di"
#line 2017 "asm/parser.y"
                {
		sect_ConstByte(0xF3);
	}
#line 3660 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 415: // sm83_ei: "ei"
#line 2023 "asm/parser.y"
                {
		sect_ConstByte(0xFB);
	}
#line 3668 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 416: // sm83_halt: "halt"
#line 2029 "asm/parser.y"
                  {
		sect_ConstByte(0x76);
	}
#line 3676 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 417: // sm83_inc: "inc" reg_r
#line 2035 "asm/parser.y"
                       {
		sect_ConstByte(0x04 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3684 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 418: // sm83_inc: "inc" reg_ss
#line 2038 "asm/parser.y"
                          {
		sect_ConstByte(0x03 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3692 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 419: // sm83_jp: "jp" reloc_16bit
#line 2044 "asm/parser.y"
                            {
		sect_ConstByte(0xC3);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3701 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 420: // sm83_jp: "jp" ccode_expr "," reloc_16bit
#line 2048 "asm/parser.y"
                                               {
		sect_ConstByte(0xC2 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3710 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 421: // sm83_jp: "jp" "hl"
#line 2052 "asm/parser.y"
                          {
		sect_ConstByte(0xE9);
	}
#line 3718 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 422: // sm83_jr: "jr" reloc_16bit
#line 2058 "asm/parser.y"
                            {
		sect_ConstByte(0x18);
		sect_PCRelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3727 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 423: // sm83_jr: "jr" ccode_expr "," reloc_16bit
#line 2062 "asm/parser.y"
                                               {
		sect_ConstByte(0x20 | (yystack_[2].value.as < int32_t > () << 3));
		sect_PCRelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3736 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 424: // sm83_ldi: "ldi" "[" "hl" "]" "," MODE_A
#line 2069 "asm/parser.y"
                                                    {
		sect_ConstByte(0x02 | (2 << 4));
	}
#line 3744 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 425: // sm83_ldi: "ldi" MODE_A "," "[" "hl" "]"
#line 2072 "asm/parser.y"
                                                      {
		sect_ConstByte(0x0A | (2 << 4));
	}
#line 3752 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 426: // sm83_ldd: "ldd" "[" "hl" "]" "," MODE_A
#line 2078 "asm/parser.y"
                                                    {
		sect_ConstByte(0x02 | (3 << 4));
	}
#line 3760 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 427: // sm83_ldd: "ldd" MODE_A "," "[" "hl" "]"
#line 2081 "asm/parser.y"
                                                      {
		sect_ConstByte(0x0A | (3 << 4));
	}
#line 3768 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 428: // sm83_ldh: "ldh" MODE_A "," op_mem_ind
#line 2087 "asm/parser.y"
                                         {
		if (yystack_[0].value.as < Expression > ().makeCheckHRAM()) {
			warning(
			    WARNING_OBSOLETE,
			    "LDH is deprecated with values from $00 to $FF; use $FF00 to $FFFF\n"
			);
		}

		sect_ConstByte(0xF0);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3784 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 429: // sm83_ldh: "ldh" op_mem_ind "," MODE_A
#line 2098 "asm/parser.y"
                                           {
		if (yystack_[2].value.as < Expression > ().makeCheckHRAM()) {
			warning(
			    WARNING_OBSOLETE,
			    "LDH is deprecated with values from $00 to $FF; use $FF00 to $FFFF\n"
			);
		}

		sect_ConstByte(0xE0);
		sect_RelByte(yystack_[2].value.as < Expression > (), 1);
	}
#line 3800 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 430: // sm83_ldh: "ldh" MODE_A "," c_ind
#line 2109 "asm/parser.y"
                                      {
		sect_ConstByte(0xF2);
	}
#line 3808 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 431: // sm83_ldh: "ldh" MODE_A "," ff00_c_ind
#line 2112 "asm/parser.y"
                                           {
		sect_ConstByte(0xF2);
	}
#line 3816 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 432: // sm83_ldh: "ldh" c_ind "," MODE_A
#line 2115 "asm/parser.y"
                                      {
		sect_ConstByte(0xE2);
	}
#line 3824 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 433: // sm83_ldh: "ldh" ff00_c_ind "," MODE_A
#line 2118 "asm/parser.y"
                                           {
		sect_ConstByte(0xE2);
	}
#line 3832 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 435: // ff00_c_ind: "[" relocexpr "+" MODE_C "]"
#line 2126 "asm/parser.y"
                                              {
		// This has to use `relocexpr`, not `iconst`, to avoid a shift/reduce conflict
		if (yystack_[3].value.as < Expression > ().getConstVal() != 0xFF00) {
			::error("Base value must be equal to $FF00 for $FF00+C\n");
		}
	}
#line 3843 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 444: // sm83_ld_hl: "ld" "hl" "," "sp" reloc_8bit_offset
#line 2146 "asm/parser.y"
                                                        {
		sect_ConstByte(0xF8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3852 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 445: // sm83_ld_hl: "ld" "hl" "," "sp"
#line 2150 "asm/parser.y"
                                        {
		::error("LD HL, SP is not a valid instruction; use LD HL, SP + 0\n");
	}
#line 3860 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 446: // sm83_ld_hl: "ld" "hl" "," reloc_16bit
#line 2153 "asm/parser.y"
                                            {
		sect_ConstByte(0x01 | (REG_HL << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3869 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 447: // sm83_ld_hl: "ld" "hl" "," reg_tt_no_af
#line 2157 "asm/parser.y"
                                             {
		::error(
		    "LD HL, %s is not a valid instruction; use LD H, %s and LD L, %s\n",
		    reg_tt_names[yystack_[0].value.as < int32_t > ()],
		    reg_tt_high_names[yystack_[0].value.as < int32_t > ()],
		    reg_tt_low_names[yystack_[0].value.as < int32_t > ()]
		);
	}
#line 3882 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 448: // sm83_ld_sp: "ld" "sp" "," "hl"
#line 2168 "asm/parser.y"
                                      {
		sect_ConstByte(0xF9);
	}
#line 3890 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 449: // sm83_ld_sp: "ld" "sp" "," reg_bc_or_de
#line 2171 "asm/parser.y"
                                             {
		::error("LD SP, %s is not a valid instruction\n", reg_tt_names[yystack_[0].value.as < int32_t > ()]);
	}
#line 3898 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 450: // sm83_ld_sp: "ld" "sp" "," reloc_16bit
#line 2174 "asm/parser.y"
                                            {
		sect_ConstByte(0x01 | (REG_SP << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3907 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 451: // sm83_ld_mem: "ld" op_mem_ind "," "sp"
#line 2181 "asm/parser.y"
                                         {
		sect_ConstByte(0x08);
		sect_RelWord(yystack_[2].value.as < Expression > (), 1);
	}
#line 3916 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 452: // sm83_ld_mem: "ld" op_mem_ind "," MODE_A
#line 2185 "asm/parser.y"
                                          {
		sect_ConstByte(0xEA);
		sect_RelWord(yystack_[2].value.as < Expression > (), 1);
	}
#line 3925 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 453: // sm83_ld_c_ind: "ld" ff00_c_ind "," MODE_A
#line 2192 "asm/parser.y"
                                        {
		sect_ConstByte(0xE2);
	}
#line 3933 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 454: // sm83_ld_c_ind: "ld" c_ind "," MODE_A
#line 2195 "asm/parser.y"
                                     {
		warning(WARNING_OBSOLETE, "LD [C], A is deprecated; use LDH [C], A\n");
		sect_ConstByte(0xE2);
	}
#line 3942 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 455: // sm83_ld_rr: "ld" reg_rr "," MODE_A
#line 2202 "asm/parser.y"
                                    {
		sect_ConstByte(0x02 | (yystack_[2].value.as < int32_t > () << 4));
	}
#line 3950 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 456: // sm83_ld_r_no_a: "ld" reg_r_no_a "," reloc_8bit
#line 2208 "asm/parser.y"
                                            {
		sect_ConstByte(0x06 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3959 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 457: // sm83_ld_r_no_a: "ld" reg_r_no_a "," reg_r
#line 2212 "asm/parser.y"
                                         {
		if (yystack_[2].value.as < int32_t > () == REG_HL_IND && yystack_[0].value.as < int32_t > () == REG_HL_IND) {
			::error("LD [HL], [HL] is not a valid instruction\n");
		} else {
			sect_ConstByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
		}
	}
#line 3971 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 458: // sm83_ld_a: "ld" reg_a "," reloc_8bit
#line 2222 "asm/parser.y"
                                       {
		sect_ConstByte(0x06 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3980 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 459: // sm83_ld_a: "ld" reg_a "," reg_r
#line 2226 "asm/parser.y"
                                    {
		sect_ConstByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3988 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 460: // sm83_ld_a: "ld" reg_a "," ff00_c_ind
#line 2229 "asm/parser.y"
                                         {
		sect_ConstByte(0xF2);
	}
#line 3996 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 461: // sm83_ld_a: "ld" reg_a "," c_ind
#line 2232 "asm/parser.y"
                                    {
		warning(WARNING_OBSOLETE, "LD A, [C] is deprecated; use LDH A, [C]\n");
		sect_ConstByte(0xF2);
	}
#line 4005 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 462: // sm83_ld_a: "ld" reg_a "," reg_rr
#line 2236 "asm/parser.y"
                                     {
		sect_ConstByte(0x0A | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 4013 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 463: // sm83_ld_a: "ld" reg_a "," op_mem_ind
#line 2239 "asm/parser.y"
                                         {
		sect_ConstByte(0xFA);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 4022 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 464: // sm83_ld_ss: "ld" reg_bc_or_de "," reloc_16bit
#line 2246 "asm/parser.y"
                                               {
		sect_ConstByte(0x01 | (yystack_[2].value.as < int32_t > () << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 4031 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 465: // sm83_ld_ss: "ld" reg_bc_or_de "," reg_tt_no_af
#line 2250 "asm/parser.y"
                                                  {
		::error(
		    "LD %s, %s is not a valid instruction; use LD %s, %s and LD %s, %s\n",
		    reg_tt_names[yystack_[2].value.as < int32_t > ()],
		    reg_tt_names[yystack_[0].value.as < int32_t > ()],
		    reg_tt_high_names[yystack_[2].value.as < int32_t > ()],
		    reg_tt_high_names[yystack_[0].value.as < int32_t > ()],
		    reg_tt_low_names[yystack_[2].value.as < int32_t > ()],
		    reg_tt_low_names[yystack_[0].value.as < int32_t > ()]
		);
	}
#line 4047 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 466: // sm83_nop: "nop"
#line 2266 "asm/parser.y"
                 {
		sect_ConstByte(0x00);
	}
#line 4055 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 467: // sm83_or: "or" op_a_n
#line 2272 "asm/parser.y"
                       {
		sect_ConstByte(0xF6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4064 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 468: // sm83_or: "or" op_a_r
#line 2276 "asm/parser.y"
                         {
		sect_ConstByte(0xB0 | yystack_[0].value.as < int32_t > ());
	}
#line 4072 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 469: // sm83_pop: "pop" reg_tt
#line 2282 "asm/parser.y"
                        {
		sect_ConstByte(0xC1 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 4080 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 470: // sm83_push: "push" reg_tt
#line 2288 "asm/parser.y"
                         {
		sect_ConstByte(0xC5 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 4088 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 471: // sm83_res: "res" reloc_3bit "," reg_r
#line 2294 "asm/parser.y"
                                        {
		uint8_t mask = static_cast<uint8_t>(0x80 | yystack_[0].value.as < int32_t > ());
		yystack_[2].value.as < Expression > ().makeCheckBitIndex(mask);
		sect_ConstByte(0xCB);
		if (!yystack_[2].value.as < Expression > ().isKnown()) {
			sect_RelByte(yystack_[2].value.as < Expression > (), 0);
		} else {
			sect_ConstByte(mask | (yystack_[2].value.as < Expression > ().value() << 3));
		}
	}
#line 4103 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 472: // sm83_ret: "ret"
#line 2307 "asm/parser.y"
                 {
		sect_ConstByte(0xC9);
	}
#line 4111 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 473: // sm83_ret: "ret" ccode_expr
#line 2310 "asm/parser.y"
                              {
		sect_ConstByte(0xC0 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 4119 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 474: // sm83_reti: "reti"
#line 2316 "asm/parser.y"
                  {
		sect_ConstByte(0xD9);
	}
#line 4127 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 475: // sm83_rl: "rl" reg_r
#line 2322 "asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x10 | yystack_[0].value.as < int32_t > ());
	}
#line 4136 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 476: // sm83_rla: "rla"
#line 2329 "asm/parser.y"
                 {
		sect_ConstByte(0x17);
	}
#line 4144 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 477: // sm83_rlc: "rlc" reg_r
#line 2335 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x00 | yystack_[0].value.as < int32_t > ());
	}
#line 4153 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 478: // sm83_rlca: "rlca"
#line 2342 "asm/parser.y"
                  {
		sect_ConstByte(0x07);
	}
#line 4161 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 479: // sm83_rr: "rr" reg_r
#line 2348 "asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x18 | yystack_[0].value.as < int32_t > ());
	}
#line 4170 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 480: // sm83_rra: "rra"
#line 2355 "asm/parser.y"
                 {
		sect_ConstByte(0x1F);
	}
#line 4178 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 481: // sm83_rrc: "rrc" reg_r
#line 2361 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x08 | yystack_[0].value.as < int32_t > ());
	}
#line 4187 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 482: // sm83_rrca: "rrca"
#line 2368 "asm/parser.y"
                  {
		sect_ConstByte(0x0F);
	}
#line 4195 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 483: // sm83_rst: "rst" reloc_8bit
#line 2374 "asm/parser.y"
                            {
		yystack_[0].value.as < Expression > ().makeCheckRST();
		if (!yystack_[0].value.as < Expression > ().isKnown()) {
			sect_RelByte(yystack_[0].value.as < Expression > (), 0);
		} else {
			sect_ConstByte(0xC7 | yystack_[0].value.as < Expression > ().value());
		}
	}
#line 4208 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 484: // sm83_sbc: "sbc" op_a_n
#line 2385 "asm/parser.y"
                        {
		sect_ConstByte(0xDE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4217 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 485: // sm83_sbc: "sbc" op_a_r
#line 2389 "asm/parser.y"
                          {
		sect_ConstByte(0x98 | yystack_[0].value.as < int32_t > ());
	}
#line 4225 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 486: // sm83_scf: "scf"
#line 2395 "asm/parser.y"
                 {
		sect_ConstByte(0x37);
	}
#line 4233 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 487: // sm83_set: "set" reloc_3bit "," reg_r
#line 2401 "asm/parser.y"
                                        {
		uint8_t mask = static_cast<uint8_t>(0xC0 | yystack_[0].value.as < int32_t > ());
		yystack_[2].value.as < Expression > ().makeCheckBitIndex(mask);
		sect_ConstByte(0xCB);
		if (!yystack_[2].value.as < Expression > ().isKnown()) {
			sect_RelByte(yystack_[2].value.as < Expression > (), 0);
		} else {
			sect_ConstByte(mask | (yystack_[2].value.as < Expression > ().value() << 3));
		}
	}
#line 4248 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 488: // sm83_sla: "sla" reg_r
#line 2414 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x20 | yystack_[0].value.as < int32_t > ());
	}
#line 4257 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 489: // sm83_sra: "sra" reg_r
#line 2421 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x28 | yystack_[0].value.as < int32_t > ());
	}
#line 4266 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 490: // sm83_srl: "srl" reg_r
#line 2428 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x38 | yystack_[0].value.as < int32_t > ());
	}
#line 4275 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 491: // sm83_stop: "stop"
#line 2435 "asm/parser.y"
                  {
		sect_ConstByte(0x10);
		sect_ConstByte(0x00);
	}
#line 4284 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 492: // sm83_stop: "stop" reloc_8bit
#line 2439 "asm/parser.y"
                               {
		sect_ConstByte(0x10);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4293 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 493: // sm83_sub: "sub" op_a_n
#line 2446 "asm/parser.y"
                        {
		sect_ConstByte(0xD6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4302 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 494: // sm83_sub: "sub" op_a_r
#line 2450 "asm/parser.y"
                          {
		sect_ConstByte(0x90 | yystack_[0].value.as < int32_t > ());
	}
#line 4310 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 495: // sm83_swap: "swap" reg_r
#line 2456 "asm/parser.y"
                        {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x30 | yystack_[0].value.as < int32_t > ());
	}
#line 4319 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 496: // sm83_xor: "xor" op_a_n
#line 2463 "asm/parser.y"
                        {
		sect_ConstByte(0xEE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4328 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 497: // sm83_xor: "xor" op_a_r
#line 2467 "asm/parser.y"
                          {
		sect_ConstByte(0xA8 | yystack_[0].value.as < int32_t > ());
	}
#line 4336 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 498: // op_mem_ind: "[" reloc_16bit "]"
#line 2475 "asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
	}
#line 4344 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 499: // op_a_r: reg_r
#line 2481 "asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4350 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 500: // op_a_r: MODE_A "," reg_r
#line 2482 "asm/parser.y"
                             {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
	}
#line 4358 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 501: // op_a_n: reloc_8bit
#line 2488 "asm/parser.y"
                   {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 4366 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 502: // op_a_n: MODE_A "," reloc_8bit
#line 2491 "asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 4374 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 517: // ccode_expr: ccode
#line 2534 "asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4380 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 518: // ccode_expr: "!" ccode_expr
#line 2535 "asm/parser.y"
                                 {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > () ^ 1;
	}
#line 4388 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 519: // ccode: "nz"
#line 2541 "asm/parser.y"
              {
		yylhs.value.as < int32_t > () = CC_NZ;
	}
#line 4396 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 520: // ccode: "z"
#line 2544 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = CC_Z;
	}
#line 4404 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 521: // ccode: "nc"
#line 2547 "asm/parser.y"
                {
		yylhs.value.as < int32_t > () = CC_NC;
	}
#line 4412 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 522: // ccode: "c"
#line 2550 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = CC_C;
	}
#line 4420 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 523: // reg_r: reg_r_no_a
#line 2555 "asm/parser.y"
       { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4426 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 524: // reg_r: reg_a
#line 2555 "asm/parser.y"
                    { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4432 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 525: // reg_r_no_a: MODE_B
#line 2558 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = REG_B;
	}
#line 4440 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 526: // reg_r_no_a: MODE_C
#line 2561 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_C;
	}
#line 4448 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 527: // reg_r_no_a: MODE_D
#line 2564 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_D;
	}
#line 4456 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 528: // reg_r_no_a: MODE_E
#line 2567 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_E;
	}
#line 4464 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 529: // reg_r_no_a: MODE_H
#line 2570 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_H;
	}
#line 4472 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 530: // reg_r_no_a: MODE_L
#line 2573 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_L;
	}
#line 4480 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 531: // reg_r_no_a: "[" "hl" "]"
#line 2576 "asm/parser.y"
                                {
		yylhs.value.as < int32_t > () = REG_HL_IND;
	}
#line 4488 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 532: // reg_a: MODE_A
#line 2582 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = REG_A;
	}
#line 4496 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 533: // reg_tt: reg_tt_no_af
#line 2588 "asm/parser.y"
        { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4502 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 534: // reg_tt: "af"
#line 2589 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_AF;
	}
#line 4510 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 535: // reg_ss: reg_tt_no_af
#line 2595 "asm/parser.y"
        { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4516 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 536: // reg_ss: "sp"
#line 2596 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_SP;
	}
#line 4524 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 537: // reg_tt_no_af: reg_bc_or_de
#line 2602 "asm/parser.y"
        { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4530 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 538: // reg_tt_no_af: "hl"
#line 2603 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_HL;
	}
#line 4538 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 539: // reg_bc_or_de: "bc"
#line 2609 "asm/parser.y"
                {
		yylhs.value.as < int32_t > () = REG_BC;
	}
#line 4546 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 540: // reg_bc_or_de: "de"
#line 2612 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_DE;
	}
#line 4554 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 541: // reg_rr: "[" "bc" "]"
#line 2618 "asm/parser.y"
                              {
		yylhs.value.as < int32_t > () = REG_BC_IND;
	}
#line 4562 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 542: // reg_rr: "[" "de" "]"
#line 2621 "asm/parser.y"
                                {
		yylhs.value.as < int32_t > () = REG_DE_IND;
	}
#line 4570 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 543: // reg_rr: hl_ind_inc
#line 2624 "asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = REG_HL_INDINC;
	}
#line 4578 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 544: // reg_rr: hl_ind_dec
#line 2627 "asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = REG_HL_INDDEC;
	}
#line 4586 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;


#line 4590 "/build/rgbds/src/rgbds/src/asm/parser.cpp"

            default:
              break;
            }
        }
#if YY_EXCEPTIONS
      catch (const syntax_error& yyexc)
        {
          YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
          error (yyexc);
          YYERROR;
        }
#endif // YY_EXCEPTIONS
      YY_SYMBOL_PRINT ("-> $$ =", yylhs);
      yypop_ (yylen);
      yylen = 0;

      // Shift the result of the reduction.
      yypush_ (YY_NULLPTR, YY_MOVE (yylhs));
    }
    goto yynewstate;


  /*--------------------------------------.
  | yyerrlab -- here on detecting error.  |
  `--------------------------------------*/
  yyerrlab:
    // If not already recovering from an error, report this error.
    if (!yyerrstatus_)
      {
        ++yynerrs_;
        context yyctx (*this, yyla);
        std::string msg = yysyntax_error_ (yyctx);
        error (YY_MOVE (msg));
      }


    if (yyerrstatus_ == 3)
      {
        /* If just tried and failed to reuse lookahead token after an
           error, discard it.  */

        // Return failure if at end of input.
        if (yyla.kind () == symbol_kind::S_YYEOF)
          YYABORT;
        else if (!yyla.empty ())
          {
            yy_destroy_ ("Error: discarding", yyla);
            yyla.clear ();
          }
      }

    // Else will try to reuse lookahead token after shifting the error token.
    goto yyerrlab1;


  /*---------------------------------------------------.
  | yyerrorlab -- error raised explicitly by YYERROR.  |
  `---------------------------------------------------*/
  yyerrorlab:
    /* Pacify compilers when the user code never invokes YYERROR and
       the label yyerrorlab therefore never appears in user code.  */
    if (false)
      YYERROR;

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYERROR.  */
    yypop_ (yylen);
    yylen = 0;
    YY_STACK_PRINT ();
    goto yyerrlab1;


  /*-------------------------------------------------------------.
  | yyerrlab1 -- common code for both syntax error and YYERROR.  |
  `-------------------------------------------------------------*/
  yyerrlab1:
    yyerrstatus_ = 3;   // Each real token shifted decrements this.
    // Pop stack until we find a state that shifts the error token.
    for (;;)
      {
        yyn = yypact_[+yystack_[0].state];
        if (!yy_pact_value_is_default_ (yyn))
          {
            yyn += symbol_kind::S_YYerror;
            if (0 <= yyn && yyn <= yylast_
                && yycheck_[yyn] == symbol_kind::S_YYerror)
              {
                yyn = yytable_[yyn];
                if (0 < yyn)
                  break;
              }
          }

        // Pop the current state because it cannot handle the error token.
        if (yystack_.size () == 1)
          YYABORT;

        yy_destroy_ ("Error: popping", yystack_[0]);
        yypop_ ();
        YY_STACK_PRINT ();
      }
    {
      stack_symbol_type error_token;


      // Shift the error token.
      yy_lac_discard_ ("error recovery");
      error_token.state = state_type (yyn);
      yypush_ ("Shifting", YY_MOVE (error_token));
    }
    goto yynewstate;


  /*-------------------------------------.
  | yyacceptlab -- YYACCEPT comes here.  |
  `-------------------------------------*/
  yyacceptlab:
    yyresult = 0;
    goto yyreturn;


  /*-----------------------------------.
  | yyabortlab -- YYABORT comes here.  |
  `-----------------------------------*/
  yyabortlab:
    yyresult = 1;
    goto yyreturn;


  /*-----------------------------------------------------.
  | yyreturn -- parsing is finished, return the result.  |
  `-----------------------------------------------------*/
  yyreturn:
    if (!yyla.empty ())
      yy_destroy_ ("Cleanup: discarding lookahead", yyla);

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYABORT or YYACCEPT.  */
    yypop_ (yylen);
    YY_STACK_PRINT ();
    while (1 < yystack_.size ())
      {
        yy_destroy_ ("Cleanup: popping", yystack_[0]);
        yypop_ ();
      }

    return yyresult;
  }
#if YY_EXCEPTIONS
    catch (...)
      {
        YYCDEBUG << "Exception caught: cleaning lookahead and stack\n";
        // Do not try to display the values of the reclaimed symbols,
        // as their printers might throw an exception.
        if (!yyla.empty ())
          yy_destroy_ (YY_NULLPTR, yyla);

        while (1 < yystack_.size ())
          {
            yy_destroy_ (YY_NULLPTR, yystack_[0]);
            yypop_ ();
          }
        throw;
      }
#endif // YY_EXCEPTIONS
  }

  void
  parser::error (const syntax_error& yyexc)
  {
    error (yyexc.what ());
  }

  const char *
  parser::symbol_name (symbol_kind_type yysymbol)
  {
    static const char *const yy_sname[] =
    {
    "end of file", "error", "invalid token", "end of line", "end of buffer",
  ",", ":", "::", "[", "]", "(", ")", "+", "-", "*", "/", "%", "**", "++",
  "==", "!=", "<", ">", "<=", ">=", "&&", "||", "!", "&", "|", "^", "<<",
  ">>", ">>>", "~", "NEG", "=", "+=", "-=", "*=", "/=", "%=", "&=", "|=",
  "^=", "<<=", ">>=", "a", "b", "c", "d", "e", "h", "l", "af", "bc", "de",
  "hl", "sp", "hli/hl+", "hld/hl-", "z", "nz", "nc", "adc", "add", "and",
  "bit", "call", "ccf", "cp", "cpl", "daa", "dec", "di", "ei", "halt",
  "inc", "jp", "jr", "ldd", "ldh", "ldi", "ld", "nop", "or", "pop", "push",
  "res", "reti", "ret", "rla", "rlca", "rlc", "rl", "rra", "rrca", "rrc",
  "rr", "rst", "sbc", "scf", "set", "sla", "sra", "srl", "stop", "sub",
  "swap", "xor", "ALIGN", "ASSERT", "BREAK", "CHARMAP", "DB", "DL", "DS",
  "DW", "ELIF", "ELSE", "ENDC", "ENDL", "ENDM", "ENDR", "ENDSECTION",
  "ENDU", "EQU", "EQUS", "EXPORT", "FAIL", "FATAL", "FOR", "FRAGMENT",
  "IF", "INCBIN", "INCLUDE", "LOAD", "MACRO", "NEWCHARMAP", "NEXTU", "OPT",
  "POPC", "POPO", "POPS", "PRINTLN", "PRINT", "PURGE", "PUSHC", "PUSHO",
  "PUSHS", "RB", "REDEF", "REPT", "RSRESET", "RSSET", "RW", "SECTION",
  "SETCHARMAP", "SHIFT", "STATIC_ASSERT", "UNION", "WARN", "ACOS", "ASIN",
  "ATAN", "ATAN2", "BANK", "BITWIDTH", "CEIL", "CHARCMP", "CHARLEN",
  "CHARSIZE", "CHARSUB", "CHARVAL", "COS", "DEF", "FDIV", "FLOOR", "FMOD",
  "FMUL", "HIGH", "INCHARMAP", "ISCONST", "LOG", "LOW", "POW", "REVCHAR",
  "ROUND", "SIN", "SIZEOF", "STARTOF", "STRCAT", "STRCHAR", "STRCMP",
  "STRFIND", "STRFMT", "STRIN", "STRLEN", "STRLWR", "STRRFIND", "STRRIN",
  "STRRPL", "STRSLICE", "STRSUB", "STRUPR", "TAN", "TZCOUNT", "HRAM",
  "OAM", "ROM0", "ROMX", "SRAM", "VRAM", "WRAM0", "WRAMX", "number",
  "string", "symbol", "label", "local label", "anonymous label", "$accept",
  "asm_file", "lines", "$@1", "diff_mark", "line", "endofline",
  "line_directive", "if", "elif", "else", "plain_directive", "endc",
  "def_id", "$@2", "redef_id", "$@3", "scoped_sym_no_anon", "scoped_sym",
  "label", "macro", "$@4", "macro_args", "directive", "def_numeric",
  "trailing_comma", "compound_eq", "align", "align_spec", "opt", "$@5",
  "opt_list", "opt_list_entry", "popo", "pusho", "$@6", "pusho_opt_list",
  "pops", "pushs", "endsection", "fail", "warn", "assert_type", "assert",
  "shift", "shift_const", "load", "rept", "for", "$@7", "$@8",
  "capture_rept", "for_args", "break", "def_macro", "$@9", "$@10",
  "capture_macro", "rsset", "rsreset", "rs_uconst", "union", "nextu",
  "endu", "ds", "ds_args", "db", "dw", "dl", "def_equ", "redef_equ",
  "def_set", "def_rb", "def_rw", "def_rl", "def_equs", "redef_equs",
  "purge", "$@11", "purge_args", "export", "export_list",
  "export_list_entry", "export_def", "include", "incbin", "charmap",
  "charmap_args", "newcharmap", "setcharmap", "pushc", "pushc_setcharmap",
  "popc", "print", "println", "print_exprs", "print_expr", "reloc_3bit",
  "constlist_8bit", "constlist_8bit_entry", "constlist_16bit",
  "constlist_16bit_entry", "constlist_32bit", "constlist_32bit_entry",
  "reloc_8bit", "reloc_8bit_offset", "reloc_16bit", "relocexpr",
  "relocexpr_no_str", "$@12", "uconst", "iconst", "precision_arg",
  "string_literal", "string", "strcat_args", "strfmt_args",
  "strfmt_va_args", "section", "pushs_section", "sect_mod", "sect_type",
  "sect_org", "sect_attrs", "cpu_commands", "cpu_command", "sm83_adc",
  "sm83_add", "sm83_and", "sm83_bit", "sm83_call", "sm83_ccf", "sm83_cp",
  "sm83_cpl", "sm83_daa", "sm83_dec", "sm83_di", "sm83_ei", "sm83_halt",
  "sm83_inc", "sm83_jp", "sm83_jr", "sm83_ldi", "sm83_ldd", "sm83_ldh",
  "c_ind", "ff00_c_ind", "sm83_ld", "sm83_ld_hl", "sm83_ld_sp",
  "sm83_ld_mem", "sm83_ld_c_ind", "sm83_ld_rr", "sm83_ld_r_no_a",
  "sm83_ld_a", "sm83_ld_ss", "sm83_nop", "sm83_or", "sm83_pop",
  "sm83_push", "sm83_res", "sm83_ret", "sm83_reti", "sm83_rl", "sm83_rla",
  "sm83_rlc", "sm83_rlca", "sm83_rr", "sm83_rra", "sm83_rrc", "sm83_rrca",
  "sm83_rst", "sm83_sbc", "sm83_scf", "sm83_set", "sm83_sla", "sm83_sra",
  "sm83_srl", "sm83_stop", "sm83_sub", "sm83_swap", "sm83_xor",
  "op_mem_ind", "op_a_r", "op_a_n", "MODE_A", "MODE_B", "MODE_C", "MODE_D",
  "MODE_E", "MODE_H", "MODE_L", "ccode_expr", "ccode", "reg_r",
  "reg_r_no_a", "reg_a", "reg_tt", "reg_ss", "reg_tt_no_af",
  "reg_bc_or_de", "reg_rr", "hl_ind_inc", "hl_ind_dec", YY_NULLPTR
    };
    return yy_sname[yysymbol];
  }



  // parser::context.
  parser::context::context (const parser& yyparser, const symbol_type& yyla)
    : yyparser_ (yyparser)
    , yyla_ (yyla)
  {}

  int
  parser::context::expected_tokens (symbol_kind_type yyarg[], int yyargn) const
  {
    // Actual number of expected tokens
    int yycount = 0;

#if YYDEBUG
    // Execute LAC once. We don't care if it is successful, we
    // only do it for the sake of debugging output.
    if (!yyparser_.yy_lac_established_)
      yyparser_.yy_lac_check_ (yyla_.kind ());
#endif

    for (int yyx = 0; yyx < YYNTOKENS; ++yyx)
      {
        symbol_kind_type yysym = YY_CAST (symbol_kind_type, yyx);
        if (yysym != symbol_kind::S_YYerror
            && yysym != symbol_kind::S_YYUNDEF
            && yyparser_.yy_lac_check_ (yysym))
          {
            if (!yyarg)
              ++yycount;
            else if (yycount == yyargn)
              return 0;
            else
              yyarg[yycount++] = yysym;
          }
      }
    if (yyarg && yycount == 0 && 0 < yyargn)
      yyarg[0] = symbol_kind::S_YYEMPTY;
    return yycount;
  }




  bool
  parser::yy_lac_check_ (symbol_kind_type yytoken) const
  {
    // Logically, the yylac_stack's lifetime is confined to this function.
    // Clear it, to get rid of potential left-overs from previous call.
    yylac_stack_.clear ();
    // Reduce until we encounter a shift and thereby accept the token.
#if YYDEBUG
    YYCDEBUG << "LAC: checking lookahead " << symbol_name (yytoken) << ':';
#endif
    std::ptrdiff_t lac_top = 0;
    while (true)
      {
        state_type top_state = (yylac_stack_.empty ()
                                ? yystack_[lac_top].state
                                : yylac_stack_.back ());
        int yyrule = yypact_[+top_state];
        if (yy_pact_value_is_default_ (yyrule)
            || (yyrule += yytoken) < 0 || yylast_ < yyrule
            || yycheck_[yyrule] != yytoken)
          {
            // Use the default action.
            yyrule = yydefact_[+top_state];
            if (yyrule == 0)
              {
                YYCDEBUG << " Err\n";
                return false;
              }
          }
        else
          {
            // Use the action from yytable.
            yyrule = yytable_[yyrule];
            if (yy_table_value_is_error_ (yyrule))
              {
                YYCDEBUG << " Err\n";
                return false;
              }
            if (0 < yyrule)
              {
                YYCDEBUG << " S" << yyrule << '\n';
                return true;
              }
            yyrule = -yyrule;
          }
        // By now we know we have to simulate a reduce.
        YYCDEBUG << " R" << yyrule - 1;
        // Pop the corresponding number of values from the stack.
        {
          std::ptrdiff_t yylen = yyr2_[yyrule];
          // First pop from the LAC stack as many tokens as possible.
          std::ptrdiff_t lac_size = std::ptrdiff_t (yylac_stack_.size ());
          if (yylen < lac_size)
            {
              yylac_stack_.resize (std::size_t (lac_size - yylen));
              yylen = 0;
            }
          else if (lac_size)
            {
              yylac_stack_.clear ();
              yylen -= lac_size;
            }
          // Only afterwards look at the main stack.
          // We simulate popping elements by incrementing lac_top.
          lac_top += yylen;
        }
        // Keep top_state in sync with the updated stack.
        top_state = (yylac_stack_.empty ()
                     ? yystack_[lac_top].state
                     : yylac_stack_.back ());
        // Push the resulting state of the reduction.
        state_type state = yy_lr_goto_state_ (top_state, yyr1_[yyrule]);
        YYCDEBUG << " G" << int (state);
        yylac_stack_.push_back (state);
      }
  }

  // Establish the initial context if no initial context currently exists.
  bool
  parser::yy_lac_establish_ (symbol_kind_type yytoken)
  {
    /* Establish the initial context for the current lookahead if no initial
       context is currently established.

       We define a context as a snapshot of the parser stacks.  We define
       the initial context for a lookahead as the context in which the
       parser initially examines that lookahead in order to select a
       syntactic action.  Thus, if the lookahead eventually proves
       syntactically unacceptable (possibly in a later context reached via a
       series of reductions), the initial context can be used to determine
       the exact set of tokens that would be syntactically acceptable in the
       lookahead's place.  Moreover, it is the context after which any
       further semantic actions would be erroneous because they would be
       determined by a syntactically unacceptable token.

       yy_lac_establish_ should be invoked when a reduction is about to be
       performed in an inconsistent state (which, for the purposes of LAC,
       includes consistent states that don't know they're consistent because
       their default reductions have been disabled).

       For parse.lac=full, the implementation of yy_lac_establish_ is as
       follows.  If no initial context is currently established for the
       current lookahead, then check if that lookahead can eventually be
       shifted if syntactic actions continue from the current context.  */
    if (yy_lac_established_)
      return true;
    else
      {
#if YYDEBUG
        YYCDEBUG << "LAC: initial context established for "
                 << symbol_name (yytoken) << '\n';
#endif
        yy_lac_established_ = true;
        return yy_lac_check_ (yytoken);
      }
  }

  // Discard any previous initial lookahead context.
  void
  parser::yy_lac_discard_ (const char* event)
  {
   /* Discard any previous initial lookahead context because of Event,
      which may be a lookahead change or an invalidation of the currently
      established initial context for the current lookahead.

      The most common example of a lookahead change is a shift.  An example
      of both cases is syntax error recovery.  That is, a syntax error
      occurs when the lookahead is syntactically erroneous for the
      currently established initial context, so error recovery manipulates
      the parser stacks to try to find a new initial context in which the
      current lookahead is syntactically acceptable.  If it fails to find
      such a context, it discards the lookahead.  */
    if (yy_lac_established_)
      {
        YYCDEBUG << "LAC: initial context discarded due to "
                 << event << '\n';
        yy_lac_established_ = false;
      }
  }


  int
  parser::yy_syntax_error_arguments_ (const context& yyctx,
                                                 symbol_kind_type yyarg[], int yyargn) const
  {
    /* There are many possibilities here to consider:
       - If this state is a consistent state with a default action, then
         the only way this function was invoked is if the default action
         is an error action.  In that case, don't check for expected
         tokens because there are none.
       - The only way there can be no lookahead present (in yyla) is
         if this state is a consistent state with a default action.
         Thus, detecting the absence of a lookahead is sufficient to
         determine that there is no unexpected or expected token to
         report.  In that case, just report a simple "syntax error".
       - Don't assume there isn't a lookahead just because this state is
         a consistent state with a default action.  There might have
         been a previous inconsistent state, consistent state with a
         non-default action, or user semantic action that manipulated
         yyla.  (However, yyla is currently not documented for users.)
         In the first two cases, it might appear that the current syntax
         error should have been detected in the previous state when
         yy_lac_check was invoked.  However, at that time, there might
         have been a different syntax error that discarded a different
         initial context during error recovery, leaving behind the
         current lookahead.
    */

    if (!yyctx.lookahead ().empty ())
      {
        if (yyarg)
          yyarg[0] = yyctx.token ();
        int yyn = yyctx.expected_tokens (yyarg ? yyarg + 1 : yyarg, yyargn - 1);
        return yyn + 1;
      }
    return 0;
  }

  // Generate an error message.
  std::string
  parser::yysyntax_error_ (const context& yyctx) const
  {
    // Its maximum.
    enum { YYARGS_MAX = 5 };
    // Arguments of yyformat.
    symbol_kind_type yyarg[YYARGS_MAX];
    int yycount = yy_syntax_error_arguments_ (yyctx, yyarg, YYARGS_MAX);

    char const* yyformat = YY_NULLPTR;
    switch (yycount)
      {
#define YYCASE_(N, S)                         \
        case N:                               \
          yyformat = S;                       \
        break
      default: // Avoid compiler warnings.
        YYCASE_ (0, YY_("syntax error"));
        YYCASE_ (1, YY_("syntax error, unexpected %s"));
        YYCASE_ (2, YY_("syntax error, unexpected %s, expecting %s"));
        YYCASE_ (3, YY_("syntax error, unexpected %s, expecting %s or %s"));
        YYCASE_ (4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
        YYCASE_ (5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
#undef YYCASE_
      }

    std::string yyres;
    // Argument number.
    std::ptrdiff_t yyi = 0;
    for (char const* yyp = yyformat; *yyp; ++yyp)
      if (yyp[0] == '%' && yyp[1] == 's' && yyi < yycount)
        {
          yyres += symbol_name (yyarg[yyi++]);
          ++yyp;
        }
      else
        yyres += *yyp;
    return yyres;
  }


  const short parser::yypact_ninf_ = -695;

  const short parser::yytable_ninf_ = -329;

  const short
  parser::yypact_[] =
  {
    1026,  -695,    18,    86,   193,  -695,  -695,  -695,    38,  -695,
    -695,  -695,  -695,  3147,    53,  -695,  3147,  -695,  3147,   209,
     367,  -695,  -695,  -695,  -695,  -695,   193,  3304,  -695,  -695,
    -695,  -695,  -695,  3147,  3147,  3147,  3147,  3147,    63,    83,
      85,   119,   252,   273,   291,   305,   319,   321,   339,   368,
     381,   385,  -695,   400,   438,   446,   459,   461,   463,   483,
     491,   493,   495,   515,   521,   536,   537,   540,   541,   545,
     554,   555,   571,   577,   580,   585,   589,   595,   596,   599,
     600,   603,   605,   606,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,   174,  2115,  -695,   243,   194,   247,  -695,   227,   616,
     403,   619,  -695,  -695,  -695,  -695,  -695,  -695,  1510,  1309,
    1510,  3147,  2034,  -695,  1510,     5,  -695,   290,  -695,  -695,
    -695,   290,  1837,  2034,    33,    37,    58,   762,  -695,  1510,
     456,   456,  3147,  -695,    73,  -695,  -695,   336,   336,  -695,
    -695,   336,   336,  3147,  1510,  -695,  3147,   336,   336,   336,
    3147,  1510,   336,  1510,  3147,   -60,   193,  1063,  3147,  3147,
    2820,  3147,  -695,  -695,  -695,  -695,  -101,  1063,  1063,  1063,
     -72,   406,  -695,  -695,  -695,  -695,  -695,  3147,  3147,  -695,
     408,  -695,    59,  -695,  -695,  3147,   -72,   440,  3147,   -60,
    -695,  1063,  -695,  -695,  -695,   842,   831,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,   654,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  1405,
     609,   609,   609,   609,   310,  3147,  3147,  3147,  3147,  1063,
    3147,  3147,  1063,  1063,  1063,  1063,  1063,  3147,   653,  3147,
    3147,  3147,  3147,  3147,  1063,  3147,  3147,  3147,  3147,  3147,
    3147,  3147,   868,   868,   426,  1063,  1063,  1063,  1063,  1063,
    1063,  1063,  1063,  1063,  1063,  1063,  1063,  1063,  3147,  3147,
    3147,  3147,  3147,  3147,  3147,  3147,  3147,  3147,  3147,  3147,
    3147,  3147,  3147,  3147,  3147,  3147,  3147,  3147,  3147,  3147,
    -695,  1063,  -695,  -695,  -695,  -695,   607,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,   661,   663,  -695,  2115,  -695,  -695,
     669,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
     672,   673,  -695,  -695,  -695,  -695,   676,  2115,  2034,  -695,
    -695,  -695,  -695,  -695,  2115,   680,  -695,  -695,  -695,   670,
    -695,  -695,  -695,  -695,  -695,   677,   686,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,   685,  -695,   692,   641,
     694,  2885,   695,   696,   698,   699,   649,   703,  1772,   704,
     707,   708,   710,   711,   713,   714,   718,   720,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,   721,    73,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,   725,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,   726,   727,   728,
     729,  3147,  -695,  -695,  -695,   123,   103,   730,  -695,  2115,
     486,   145,   183,   732,  -695,   514,   240,   731,   733,   284,
     736,  -695,   518,   376,  1152,   441,  -695,  -695,   737,  -695,
     247,   156,    79,  -695,  -695,  1063,   738,   528,   395,   740,
    -695,   530,   414,   740,   437,  -695,   528,  1063,   529,  -695,
    1063,  -695,  -695,  -695,  3147,   247,   533,  -695,  3147,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  3147,
    3147,  1063,  3147,  3147,  3147,  3147,  3147,  1063,  3147,  3118,
    -695,   745,   742,   742,   742,   746,   747,   756,  1622,   742,
     159,    78,   154,   163,   202,   742,   310,   752,   742,   755,
     763,  1890,   182,  2146,   768,  2408,   769,    67,  -695,   742,
     742,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,   200,
     764,   274,   765,  -695,   247,   114,   203,   234,   236,   247,
     774,   237,   312,   316,   248,   279,   285,   286,   295,   364,
     742,  2670,   749,   749,   609,   609,   609,   609,   620,   620,
     620,   620,   620,   620,  1318,  1860,   407,   407,   407,   523,
     523,   523,  -695,   781,   784,   193,   780,  2099,  2361,  1510,
     546,  3147,   336,  -695,  3147,   739,   584,   611,  3147,  3147,
     782,   786,   785,   783,  2312,   787,     5,     5,     5,   791,
     792,   794,   796,   797,   251,   798,   799,  2296,  2558,     5,
       5,   -25,  1510,  1575,  2623,     5,   336,   336,  3147,  -695,
    -695,  -695,  1378,  3147,  3147,  -695,  3147,  -695,  3147,  3147,
    3147,  -695,   437,  -695,  3147,  -695,   307,   573,  -695,   528,
    -695,  3147,  -695,  -695,  -695,   795,   528,  -695,   315,  -695,
     317,   816,  -695,   587,  -695,  -695,  -695,  -695,   247,  -695,
    -695,  -695,  -695,  -695,   247,  -695,  -695,  -695,  3147,   812,
     814,   815,  3147,  -695,  -695,  -695,   817,  1063,  -695,  -695,
    3147,  3147,   818,   820,  3147,   824,  3147,  3147,  -695,  -695,
    -695,  3147,  -695,  3147,  3147,  -695,   826,   827,  -695,  -695,
    -695,  -695,  1063,  -695,  3147,  1063,  1063,   834,  -695,  1063,
    -695,  -695,  1063,  1063,  1063,  3147,  3147,  -695,   829,  -695,
    3147,  -695,  -695,  -695,   830,   835,   836,   837,   838,   839,
     840,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,   849,
     800,  3082,  -695,  2885,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,   851,   801,  -695,  -695,   850,   852,  -695,  -695,   393,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  1063,   855,  -695,  -695,   853,   858,  -695,
    -695,  -695,   859,   254,  -695,  -695,  -695,   437,  -695,   254,
     254,  1063,  -695,  -695,  -695,  -695,  -695,   742,  -695,   390,
     854,   878,  -695,  -695,   742,  -695,   742,   742,   742,   742,
    -695,  -695,  -695,   247,   879,   415,   417,  3147,   423,   432,
     434,   372,   126,   409,  -695,   888,   887,   193,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,     5,   885,   886,     5,   889,
    -695,  -695,  3147,  3147,  -695,   247,  3147,  -695,   891,  3147,
    -695,  3147,   893,  -695,   893,   893,   247,   892,  -695,  -695,
    -695,   894,   895,   896,   898,   899,  -695,  -695,  -695,   109,
     489,   135,  -695,  -695,  -695,  1063,  3147,  -695,  3147,  -695,
    -695,  3147,  -695,  -695,  -695,  -695,  -695,  -695,  2115,  2115,
    3147,  -695,  -695,  -695,  3147,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,   436,   900,   901,   193,   897,   858,
     890,   908,   908,   908,  -695,  -695,  -695,  -695,  3147,  -695,
    -695,   -81,  -695,   907,   910,  3147,  3147,   911,   912,  -695,
    -695
  };

  const short
  parser::yydefact_[] =
  {
       0,     5,     0,     7,     0,     1,     8,     9,    39,    12,
      13,     6,    45,     0,     0,   139,     0,   147,     0,     0,
      42,     4,    11,    19,    20,    21,     0,    25,    15,    16,
      17,    14,    18,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   270,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   234,   303,    34,    35,    36,    38,
      37,   233,   300,   231,     0,   232,     0,    24,     0,     0,
       0,     0,   299,    40,    41,    43,    44,    10,     0,     0,
       0,     0,     0,   406,     0,   409,   411,     0,   414,   415,
     416,     0,     0,     0,     0,     0,     0,     0,   466,     0,
       0,     0,     0,   474,   472,   476,   478,     0,     0,   480,
     482,     0,     0,     0,     0,   486,     0,     0,     0,     0,
     491,     0,     0,     0,     0,   125,     0,     0,   164,   168,
       0,   166,    29,   137,   122,   157,     0,     0,     0,     0,
     331,     0,   156,   110,   202,   115,   120,   204,     0,   181,
     200,   116,   331,    32,   152,     0,   331,     0,   134,   125,
     155,     0,    30,    46,    50,     0,     0,    27,    28,    77,
      88,    87,    85,    86,    81,    82,    84,    74,    75,    76,
      73,    72,    61,    60,    62,    63,    64,    58,    55,    56,
      57,    89,    90,    91,    92,    93,    94,    78,    79,    80,
      53,    54,    65,    66,    67,    68,    69,    71,    70,    51,
      52,    59,    83,    26,   347,   349,   350,   351,   352,   353,
     354,   355,   356,   357,   358,   359,   360,   361,   362,   363,
     364,   368,   366,   367,   365,   440,   441,   436,   437,   438,
     442,   443,   439,   369,   370,   371,   372,   373,   374,   375,
     376,   377,   378,   379,   380,   381,   382,   383,   384,   385,
     386,   387,   388,   389,   390,   391,   392,   393,   394,     0,
     256,   257,   235,   258,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   323,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      23,     0,   140,    22,   148,   142,     0,   503,   505,   507,
     509,   511,   513,   515,     0,     0,   501,   227,   396,   395,
     532,   525,   526,   527,   528,   529,   530,   499,   523,   524,
       0,     0,   398,   397,   402,   401,     0,   211,     0,   522,
     520,   519,   521,   404,   230,     0,   517,   408,   407,     0,
     410,   539,   540,   538,   536,     0,     0,   532,   412,   413,
     535,   537,   417,   418,   421,   419,     0,   422,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   543,   544,
     468,   467,   534,   469,   533,   470,     0,     0,   473,   477,
     475,   481,   479,   483,   485,   484,     0,   488,   489,   490,
     492,   494,   493,   495,   497,   496,   107,   108,     0,     0,
       0,     0,   146,   320,   319,     0,   233,    95,   212,     0,
     231,   232,   233,    95,   222,   231,   232,     0,   158,   233,
      95,   217,   231,   232,     0,     0,   188,   189,    95,   186,
     123,   191,     0,   333,   332,     0,   197,     0,   233,    95,
     206,   231,   232,    95,     0,   201,   118,     0,     0,   151,
       0,   199,   133,   135,     0,   124,     0,    48,     0,    97,
      98,    99,   100,   101,   104,   103,   102,   105,   106,   153,
       0,     0,   153,   153,     0,     0,     0,     0,     0,     0,
     298,     0,   301,   301,   301,     0,   320,   319,     0,   301,
       0,     0,     0,     0,     0,   301,     0,     0,   301,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   195,   301,
     301,   338,   341,   337,   336,   340,   335,   334,   339,     0,
       0,     0,     0,   312,   321,     0,     0,     0,     0,   325,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     301,     0,   244,   245,   252,   253,   254,   255,   238,   243,
     240,   239,   242,   241,   237,   236,   248,   247,   246,   249,
     250,   251,   304,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   518,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   230,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   127,
     128,   126,   129,     0,    96,   165,    96,   169,     0,     0,
      96,   167,    96,   185,     0,   190,     0,     0,   114,   111,
     112,    96,   205,   203,   183,    95,   119,   117,     0,    33,
       0,   131,    31,    47,   172,   178,   154,   170,   179,   176,
     177,   174,   173,   171,   180,   175,   348,   318,     0,     0,
       0,     0,     0,   264,   265,   261,     0,     0,   293,   296,
       0,     0,     0,     0,     0,     0,     0,     0,   259,   294,
     263,     0,   260,     0,     0,   311,     0,     0,   266,   268,
     267,   269,     0,   313,     0,     0,     0,   324,   317,     0,
     292,   315,     0,     0,     0,     0,     0,   314,     0,   262,
       0,   150,   138,   531,     0,     0,     0,     0,     0,     0,
       0,   502,   500,   399,   400,   403,   405,   420,   423,     0,
       0,     0,   498,     0,   434,   432,   433,   429,   430,   431,
     428,     0,     0,   541,   542,     0,     0,   545,   547,   445,
     446,   447,   448,   450,   449,   454,   453,   451,   452,   456,
     457,   458,   461,   460,   463,   459,   462,   464,   465,   455,
     471,   487,   109,     0,    95,   213,   223,     0,    95,   162,
     218,   187,   192,     0,   198,   113,   207,    96,   182,     0,
       0,     0,    49,   302,   284,   283,   285,   301,   273,     0,
       0,     0,   281,   271,   301,   274,   301,   301,   301,   301,
     196,   272,   280,   322,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   282,     0,   143,     0,   504,   506,
     510,   514,   508,   512,   516,     0,     0,     0,     0,     0,
     546,   548,     0,     0,   444,   130,    96,   194,    95,    96,
     159,     0,   342,   184,   342,   342,   132,     0,   295,   310,
     297,     0,     0,     0,     0,     0,   309,   287,   288,   233,
     231,   232,   290,   289,   291,     0,     0,   306,     0,   308,
     142,     0,   149,   426,   427,   435,   424,   425,   228,   229,
      96,   160,   163,   193,     0,   344,   344,   344,   286,   275,
     277,   276,   279,   278,     0,     0,     0,     0,   144,    95,
       0,   136,   330,   329,   316,   305,   307,   141,     0,   161,
     343,     0,   145,     0,     0,     0,     0,     0,     0,   345,
     346
  };

  const short
  parser::yypgoto_[] =
  {
    -695,  -695,  -695,  -695,  -695,  -695,   -24,  -695,  -695,  -695,
    -695,  -695,  -695,   750,  -695,   753,  -695,  -163,  -110,  -695,
    -695,  -695,  -695,  -695,   757,  -439,  -183,  -695,  -694,  -695,
    -695,   361,  -231,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,   715,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,   -48,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -104,  -695,  -695,  -695,  -695,   -54,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,   213,  -695,  -695,  -695,  -695,   224,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,   754,   207,   -55,  -695,   226,
    -695,   221,  -695,   229,  -135,  -695,   -80,     1,  -138,  -695,
     -18,   -12,  -390,   675,   -65,  -695,  -695,  -695,  -695,  -695,
     -16,  -303,  -430,  -444,   369,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -120,  -118,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,  -695,
    -695,  -695,  -108,    29,   302,   -98,  -695,  -387,  -695,  -695,
    -695,  -695,   -47,  -695,   255,   810,   822,   808,   -97,  -125,
    -102,   250,  -695,  -695
  };

  const short
  parser::yydefgoto_[] =
  {
       0,     2,     3,     4,     8,    21,    11,    22,    23,    24,
      25,    26,   194,   195,   546,   196,   538,    90,    91,    27,
     197,   547,   733,   198,   199,   705,   564,   200,   486,   201,
     527,   719,   720,   202,   203,   536,   727,   204,   205,   206,
     207,   208,   491,   209,   210,   542,   211,    28,    29,    98,
     653,   655,   915,    30,    31,   100,   654,   917,   212,   213,
     735,   214,   215,   216,   217,   868,   218,   219,   220,   221,
     222,   223,   224,   225,   226,   227,   228,   229,   534,   725,
     230,   518,   519,   231,    32,   232,   233,   597,   234,   235,
     236,   237,   238,   239,   240,   529,   530,   406,   497,   498,
     510,   511,   503,   504,   386,   934,   673,    92,    93,   318,
     487,   102,   749,    95,    96,   615,   620,   787,   241,   242,
     525,   610,   985,  1001,   243,   244,   245,   246,   247,   248,
     249,   250,   251,   252,   253,   254,   255,   256,   257,   258,
     259,   260,   261,   262,   263,   442,   443,   264,   265,   266,
     267,   268,   269,   270,   271,   272,   273,   274,   275,   276,
     277,   278,   279,   280,   281,   282,   283,   284,   285,   286,
     287,   288,   289,   290,   291,   292,   293,   294,   295,   296,
     297,   298,   444,   388,   389,   427,   391,   392,   393,   394,
     395,   396,   663,   416,   397,   398,   399,   463,   429,   430,
     431,   457,   458,   459
  };

  const short
  parser::yytable_[] =
  {
     101,    94,   107,   516,    99,   464,   464,   451,   473,   452,
     390,   390,   390,   568,   867,   480,   390,   420,     5,   453,
     500,   505,   377,   512,   433,   456,   440,   445,   447,  1013,
     612,   390,   413,   847,   299,   300,   301,   302,   303,   531,
     531,   439,   435,   437,    12,   441,   390,   493,   496,   502,
     183,   509,   377,   390,   675,   390,    97,   493,   493,   493,
     523,   675,  -121,  -121,   707,   415,   446,   528,   528,   488,
     489,   711,   774,   304,   192,   436,   438,   466,   775,   713,
     377,   493,     9,    10,   377,  1014,    -2,   468,   524,   758,
     722,   476,   495,   305,   723,   306,   371,   371,     6,     7,
     467,   490,   520,   521,   522,   377,  -216,  -216,  -216,   387,
     387,   387,   407,   414,  -328,   387,    86,    87,    88,   782,
    -328,  -320,   409,   414,   414,   783,   545,  -320,   703,   307,
     387,   966,   492,   407,   410,   411,   412,   967,   402,   404,
    -327,   371,   508,   417,   387,   387,  -327,   407,  -215,  -215,
    -215,   387,   387,  -319,   387,   419,    13,    14,   460,   499,
     499,   714,   499,  -319,   757,   759,   537,   539,   760,    15,
     540,    16,   371,   474,   371,    17,   543,   371,   499,   499,
     481,   371,   484,   750,   751,   419,  -226,  -226,  -226,   756,
      18,   523,  -320,   769,   571,   762,     9,    10,   765,   576,
     371,  -320,   493,   493,   493,   493,   493,   761,   784,   776,
     777,   778,  -319,   419,   493,   103,   104,   419,   371,   524,
     371,   371,   493,   493,   493,   493,   493,   493,   493,   493,
     493,   493,   493,   493,   493,   493,   493,   493,   419,   785,
     798,   786,   789,  -225,  -225,  -225,   370,   580,   581,   582,
     583,   584,   371,   792,   371,   371,    19,    20,  -319,   592,
     803,   493,   308,   835,   836,   371,   371,   609,   611,   614,
     616,   617,   618,   619,   621,   622,   623,   624,   625,   626,
     627,   628,   629,   309,   793,   780,   878,  -221,  -221,  -221,
     794,   795,   371,   572,   573,   574,   575,   371,   376,   579,
     796,   310,  -320,   371,   371,   585,   652,   587,   588,   589,
     590,   578,   873,   371,   594,   311,   596,   598,   599,   600,
     879,  1017,   880,   790,   591,   371,   593,   791,   595,   312,
     371,   313,   568,   371,   371,   371,   630,   377,   378,   379,
     380,   381,   382,   383,   376,   421,   422,   423,   424,   314,
     631,   632,   633,   634,   635,   636,   637,   638,   639,   640,
     641,   642,   643,   644,   645,   646,   647,   648,   649,   650,
     651,   724,   428,   105,   106,   797,   432,   965,   315,  -220,
    -220,  -220,   371,   377,   378,   379,   380,   381,   382,   383,
     371,   316,   469,   470,  -319,   317,   471,   472,  -210,  -210,
    -210,   948,   477,   478,   479,   932,   933,   483,   371,   302,
     319,   403,   405,  -320,   968,   493,   418,  -209,  -209,  -209,
     969,   352,   353,   354,   355,   937,   957,   493,   958,   940,
     493,   461,  -319,   371,   962,   371,   927,   613,   367,   368,
     369,   371,   674,   963,   372,   964,   475,  1004,   320,   674,
     371,   493,   371,   482,   371,   485,   321,   493,   739,   740,
     716,   601,   602,   603,   604,   605,   606,   607,   608,   322,
     425,   323,   728,   324,   426,   730,   763,   565,   549,   550,
     551,   552,   553,   554,   555,   556,   557,   558,   875,  -214,
    -214,  -214,   702,   325,  -326,   875,   738,   947,   715,   981,
    -326,   326,   744,   327,   951,   328,   952,   953,   954,   955,
     462,   421,   422,   423,   986,   987,   425,  -224,  -224,  -224,
     426,  -219,  -219,  -219,   811,   329,   814,    86,    87,    88,
      89,   330,   731,  -208,  -208,  -208,   734,   352,   353,   354,
     355,   736,  1002,  1003,   736,   736,   331,   332,   737,   516,
     333,   334,   741,   742,   743,   335,   745,   849,   851,   828,
    1009,   829,   841,   813,   336,   337,   500,   566,   505,   858,
     942,   830,   512,   852,   869,   853,   944,   945,   825,   826,
     827,   338,    38,   531,   816,   854,   844,   339,   817,   818,
     340,   845,   846,   848,   496,   341,   502,   859,    49,   342,
     509,   421,   422,   423,   424,   343,   344,   840,   843,   345,
     346,   528,    63,   347,   857,   348,   349,    68,    69,   373,
     374,    72,   375,   526,    75,   535,   355,    78,    79,    80,
      81,   802,   350,   351,   352,   353,   354,   355,   804,   805,
     806,   807,    85,    86,    87,    88,    89,   493,   364,   365,
     366,   367,   368,   369,    86,    87,    88,   541,   591,   595,
     387,   569,   387,   586,   656,   414,   808,   809,   810,   414,
     414,   657,   493,   658,   659,   493,   493,   660,   661,   493,
     665,   662,   493,   493,   493,   664,   862,   666,   414,   414,
     668,   598,   889,   387,   387,   414,   667,   669,   670,   671,
     676,   677,   872,   678,   679,   499,   680,   499,   681,   687,
     387,   499,   688,   689,   943,   690,   691,   903,   692,   693,
     905,   906,   499,   694,   908,   695,   696,   909,   910,   911,
     697,   698,   699,   700,   701,   704,   883,   706,   709,   708,
     887,   710,   712,   717,   718,   721,   729,   748,   890,   891,
     732,   752,   894,   493,   896,   897,   747,   764,   753,   898,
     766,   899,   900,   352,   353,   354,   355,   754,   767,   960,
     448,   493,   904,   771,   773,   779,   781,   364,   365,   366,
     367,   368,   369,   912,   913,   788,   800,   801,   916,   803,
     874,   819,   822,   804,   820,   821,   824,   959,   935,   441,
     877,   831,   832,   882,   982,   833,   834,   837,   838,   377,
     378,   379,   380,   381,   382,   383,   946,   421,   422,   449,
     450,   881,   595,   884,   632,   885,   886,   973,   888,   892,
     976,   893,   494,   501,   506,   895,   513,   901,   902,   907,
     914,   918,   494,   494,   494,   869,   919,   920,   921,   922,
     923,   924,   532,   532,   925,   493,   928,   926,   929,   930,
     936,   931,   938,   939,   941,   949,   494,   565,   549,   550,
     551,   552,   553,   554,   555,   556,   557,   558,   548,   549,
     550,   551,   552,   553,   554,   555,   556,   557,   558,   950,
     956,   970,   971,   972,   974,   975,   980,   726,   977,  1010,
     994,   984,  1008,   988,   544,   989,   990,   991,   499,   992,
     993,  1005,  1006,  1011,   812,  1015,   514,   815,  1016,   515,
    1019,  1020,   997,   517,   900,   871,   999,   864,   876,   983,
     865,   870,   533,   978,   979,   866,   559,   454,   746,   465,
     387,     0,   425,   856,     0,     0,   426,   850,   855,   455,
     996,   860,   861,     0,   995,     0,     0,   566,   567,   998,
       0,     0,     0,     0,     0,     0,  1000,     0,   560,   561,
       0,     0,     0,  1007,     0,     0,     0,     0,     0,     0,
       0,   387,     0,     0,   577,     0,     0,   494,   494,   494,
     494,   494,   562,     0,     0,     0,  1012,   563,  1018,   494,
       0,     0,     0,     0,     0,     0,     0,   494,   494,   494,
     494,   494,   494,   494,   494,   494,   494,   494,   494,   494,
     494,   494,   494,     0,    38,     0,    -3,     1,     0,    -3,
      -3,     0,    -3,     0,     0,     0,     0,     0,    -3,    -3,
      49,     0,     0,     0,     0,     0,   494,     0,     0,     0,
       0,     0,     0,     0,    63,     0,     0,     0,     0,    68,
      69,     0,     0,    72,     0,     0,    75,     0,     0,    78,
      79,    80,    81,     0,     0,   601,   602,   603,   604,   605,
     606,   607,   608,     0,    85,    86,    87,    88,    89,     0,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,     0,     0,
      -3,    -3,     0,     0,    -3,    -3,     0,    -3,     0,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,     0,    -3,    -3,    -3,
      -3,     0,    -3,    -3,    -3,    -3,    -3,    -3,   548,   549,
     550,   551,   552,   553,   554,   555,   556,   557,   558,     0,
     494,    -3,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   494,     0,     0,   494,     0,     0,     0,    38,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,    49,   494,     0,     0,     0,
       0,     0,   494,    -3,    -3,    -3,   559,     0,     0,    63,
       0,     0,     0,     0,    68,    69,     0,     0,    72,     0,
       0,    75,     0,     0,    78,    79,    80,    81,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   560,    85,
      86,    87,    88,    89,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   562,     0,     0,     0,     0,   563,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   376,     0,    33,
       0,    34,    35,     0,     0,     0,     0,     0,     0,     0,
     350,   351,   352,   353,   354,   355,    36,   356,   357,   358,
     359,   360,   361,    37,     0,     0,   364,   365,   366,   367,
     368,   369,     0,     0,     0,     0,   377,   378,   379,   380,
     381,   382,   383,     0,     0,     0,   400,   401,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   501,
       0,   506,     0,   863,     0,   513,     0,     0,     0,     0,
     350,   351,   352,   353,   354,   355,   532,   356,   357,   358,
     359,   360,   361,   362,   363,     0,   364,   365,   366,   367,
     368,   369,     0,     0,     0,     0,   570,   350,   351,   352,
     353,   354,   355,     0,   356,   357,   358,   359,   360,   361,
     362,   363,   494,   364,   365,   366,   367,   368,   369,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   494,     0,     0,
     494,   494,     0,     0,   494,    38,     0,   494,   494,   494,
       0,    39,    40,    41,    42,    43,    44,    45,    46,    47,
      48,    49,    50,    51,    52,    53,    54,    55,    56,   384,
      58,    59,    60,   385,    62,    63,    64,    65,    66,    67,
      68,    69,    70,    71,    72,    73,    74,    75,    76,    77,
      78,    79,    80,    81,    82,    83,     0,     0,   376,     0,
      33,     0,    34,    35,    84,    85,    86,    87,    88,    89,
       0,     0,     0,     0,     0,     0,     0,    36,   494,     0,
       0,     0,     0,     0,    37,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   494,   377,   378,   379,
     380,   381,   382,   383,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   961,   448,     0,    33,     0,    34,    35,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    36,     0,     0,     0,     0,     0,     0,    37,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   377,   378,   379,   380,   381,   382,   383,     0,
       0,     0,     0,   755,   350,   351,   352,   353,   354,   355,
     494,   356,   357,   358,   359,   360,   361,   362,   363,     0,
     364,   365,   366,   367,   368,   369,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,    38,     0,     0,     0,
       0,     0,    39,    40,    41,    42,    43,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
     384,    58,    59,    60,   385,    62,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      77,    78,    79,    80,    81,    82,    83,     0,     0,     0,
       0,     0,     0,     0,     0,    84,    85,    86,    87,    88,
      89,    38,     0,     0,     0,     0,     0,    39,    40,    41,
      42,    43,    44,    45,    46,    47,    48,    49,    50,    51,
      52,    53,    54,    55,    56,   384,    58,    59,    60,   385,
      62,    63,    64,    65,    66,    67,    68,    69,    70,    71,
      72,    73,    74,    75,    76,    77,    78,    79,    80,    81,
      82,    83,    33,     0,    34,    35,     0,     0,     0,     0,
      84,    85,    86,    87,    88,    89,     0,     0,     0,    36,
       0,     0,     0,     0,     0,     0,    37,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   379,     0,     0,     0,     0,     0,   682,   683,   684,
       0,   685,   686,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    33,     0,    34,
      35,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   408,     0,     0,     0,     0,     0,
       0,    37,   350,   351,   352,   353,   354,   355,     0,   356,
     357,   358,   359,   360,   361,   362,   409,     0,   364,   365,
     366,   367,   368,   369,   434,     0,     0,     0,   410,   411,
     412,   768,   350,   351,   352,   353,   354,   355,     0,   356,
     357,   358,   359,   360,   361,   362,   363,     0,   364,   365,
     366,   367,   368,   369,     0,     0,     0,     0,    38,     0,
       0,     0,     0,     0,    39,    40,    41,    42,    43,    44,
      45,    46,    47,    48,    49,    50,    51,    52,    53,    54,
      55,    56,    57,    58,    59,    60,   672,    62,    63,    64,
      65,    66,    67,    68,    69,    70,    71,    72,    73,    74,
      75,    76,    77,    78,    79,    80,    81,    82,    83,     0,
       0,     0,     0,     0,     0,     0,     0,    84,    85,    86,
      87,    88,    89,    38,     0,     0,     0,     0,     0,    39,
      40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
      50,    51,    52,    53,    54,    55,    56,    57,    58,    59,
      60,    61,    62,    63,    64,    65,    66,    67,    68,    69,
      70,    71,    72,    73,    74,    75,    76,    77,    78,    79,
      80,    81,    82,    83,    33,     0,    34,    35,     0,     0,
       0,     0,    84,    85,    86,    87,    88,    89,     0,     0,
       0,   408,     0,     0,     0,     0,     0,     0,    37,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   409,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   410,   411,   412,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,    33,
       0,    34,    35,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,    36,   350,   351,   352,
     353,   354,   355,    37,   356,   357,   358,   359,   360,   361,
     362,   363,     0,   364,   365,   366,   367,   368,   369,     0,
       0,     0,     0,   804,   805,   806,   807,   770,   350,   351,
     352,   353,   354,   355,     0,   356,   357,   358,   359,   360,
     361,   362,   363,     0,   364,   365,   366,   367,   368,   369,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      38,     0,     0,     0,     0,     0,    39,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,    77,    78,    79,    80,    81,    82,
      83,     0,     0,     0,     0,     0,     0,     0,     0,    84,
      85,    86,    87,    88,    89,    38,     0,     0,     0,     0,
       0,    39,    40,    41,    42,    43,    44,    45,    46,    47,
      48,    49,    50,    51,    52,    53,    54,    55,    56,    57,
      58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
      68,    69,    70,    71,    72,    73,    74,    75,    76,    77,
      78,    79,    80,    81,    82,    83,    33,     0,    34,    35,
       0,     0,     0,     0,    84,    85,    86,    87,    88,    89,
       0,     0,     0,    36,   823,   351,   352,   353,   354,   355,
      37,   356,   357,   358,   359,   360,   361,   362,   363,     0,
     364,   365,   366,   367,   368,   369,     0,     0,     0,     0,
       0,   421,   422,   423,   839,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,    33,     0,    34,    35,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    36,     0,
       0,     0,     0,     0,     0,    37,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   808,   809,   810,   772,
     350,   351,   352,   353,   354,   355,     0,   356,   357,   358,
     359,   360,   361,   362,   363,     0,   364,   365,   366,   367,
     368,   369,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    38,     0,     0,     0,     0,     0,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,    58,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,    76,    77,    78,    79,    80,
      81,    82,    83,     0,     0,     0,     0,     0,     0,     0,
       0,    84,    85,    86,    87,    88,    89,    38,     0,     0,
       0,     0,     0,    39,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
      76,    77,    78,    79,    80,    81,    82,    83,    33,     0,
      34,    35,     0,     0,     0,     0,    84,    85,    86,    87,
      88,    89,     0,     0,     0,    36,     0,     0,     0,     0,
       0,     0,    37,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   421,   422,   842,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    33,     0,    34,    35,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      36,     0,     0,     0,     0,     0,     0,    37,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   421,   422,
     423,   799,   350,   351,   352,   353,   354,   355,     0,   356,
     357,   358,   359,   360,   361,   362,   363,     0,   364,   365,
     366,   367,   368,   369,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    38,     0,     0,     0,     0,     0,
      39,    40,    41,    42,    43,    44,    45,    46,    47,    48,
      49,    50,    51,    52,    53,    54,    55,    56,    57,    58,
      59,    60,    61,    62,    63,    64,    65,    66,    67,    68,
      69,    70,    71,    72,    73,    74,    75,    76,    77,    78,
      79,    80,    81,    82,    83,     0,     0,     0,     0,     0,
       0,     0,     0,    84,    85,    86,    87,    88,    89,    38,
       0,     0,     0,     0,     0,    39,    40,    41,    42,    43,
      44,    45,    46,    47,    48,    49,    50,    51,    52,    53,
      54,    55,    56,    57,    58,    59,    60,    61,    62,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,    77,    78,    79,    80,    81,    82,    83,
      33,     0,    34,    35,     0,     0,     0,     0,    84,    85,
      86,    87,    88,    89,     0,     0,     0,    36,     0,     0,
       0,     0,     0,     0,    37,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,    33,     0,    34,    35,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    36,     0,     0,     0,     0,     0,     0,    37,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     507,     0,     0,     0,   379,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,    38,     0,     0,     0,
       0,     0,    39,    40,    41,    42,    43,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      77,    78,    79,    80,    81,    82,    83,     0,     0,     0,
       0,     0,     0,     0,     0,    84,    85,    86,    87,    88,
      89,    38,     0,     0,     0,     0,     0,    39,    40,    41,
      42,    43,    44,    45,    46,    47,    48,    49,    50,    51,
      52,    53,    54,    55,    56,    57,    58,    59,    60,   672,
      62,    63,    64,    65,    66,    67,    68,    69,    70,    71,
      72,    73,    74,    75,    76,    77,    78,    79,    80,    81,
      82,    83,    33,     0,    34,    35,     0,     0,     0,     0,
      84,    85,    86,    87,    88,    89,     0,     0,     0,    36,
       0,     0,     0,     0,     0,     0,    37,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   808,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    33,     0,    34,
      35,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    36,     0,     0,     0,     0,     0,
       0,    37,   108,   109,   110,   111,   112,   113,   114,   115,
     116,   117,   118,   119,   120,   121,   122,   123,   124,   125,
     126,   127,   128,   129,   130,   131,   132,   133,   134,   135,
     136,   137,   138,   139,   140,   141,   142,   143,   144,   145,
     146,   147,   148,   149,   150,   151,   152,   153,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    38,     0,
       0,     0,     0,     0,    39,    40,    41,    42,    43,    44,
      45,    46,    47,    48,    49,    50,    51,    52,    53,    54,
      55,    56,    57,    58,    59,    60,    61,    62,    63,    64,
      65,    66,    67,    68,    69,    70,    71,    72,    73,    74,
      75,    76,    77,    78,    79,    80,    81,    82,    83,     0,
       0,     0,     0,     0,     0,     0,     0,    84,    85,    86,
      87,    88,    89,    38,     0,     0,     0,     0,     0,    39,
      40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
      50,    51,    52,    53,    54,    55,    56,    57,    58,    59,
      60,    61,    62,    63,    64,    65,    66,    67,    68,    69,
      70,    71,    72,    73,    74,    75,    76,    77,    78,    79,
      80,    81,    82,    83,     0,     0,     0,     0,     0,     0,
       0,     0,    84,    85,    86,    87,    88,    89,   108,   109,
     110,   111,   112,   113,   114,   115,   116,   117,   118,   119,
     120,   121,   122,   123,   124,   125,   126,   127,   128,   129,
     130,   131,   132,   133,   134,   135,   136,   137,   138,   139,
     140,   141,   142,   143,   144,   145,   146,   147,   148,   149,
     150,   151,   152,   153,   154,   155,   156,   157,   158,   159,
     160,   161,     0,     0,   162,   163,     0,     0,   164,   165,
       0,     0,   166,   167,     0,     0,     0,     0,   168,   169,
     170,     0,   171,   172,   173,   174,   175,   176,   177,   178,
     179,   180,   181,   182,     0,   183,     0,   184,   185,     0,
     186,   187,   188,   189,   190,   191,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   192,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   193
  };

  const short
  parser::yycheck_[] =
  {
      18,    13,    26,   166,    16,   130,   131,   127,   143,   127,
     108,   109,   110,   196,   708,   150,   114,   115,     0,   127,
     158,   159,    47,   161,   121,   127,   124,   125,   126,   110,
     333,   129,   112,    58,    33,    34,    35,    36,    37,   177,
     178,     8,   122,   123,     6,     8,   144,   157,   158,   159,
     151,   161,    47,   151,   441,   153,     3,   167,   168,   169,
     132,   448,     3,     4,   503,   112,     8,   177,   178,   129,
     130,   510,     5,    10,   175,   122,   123,   132,    11,   518,
      47,   191,     3,     4,    47,   166,     0,   134,   160,    11,
     529,   146,   157,    10,   533,    10,    18,    18,    12,    13,
      27,   161,   167,   168,   169,    47,     3,     4,     5,   108,
     109,   110,   111,   112,     5,   114,   217,   218,   219,     5,
      11,    18,    49,   122,   123,    11,   191,    18,     5,    10,
     129,     5,   156,   132,    61,    62,    63,    11,   109,   110,
       5,    18,   160,   114,   143,   144,    11,   146,     3,     4,
       5,   150,   151,    18,   153,   180,   118,   119,   129,   158,
     159,     5,   161,    18,     5,    11,   182,   185,     5,   131,
     186,   133,    18,   144,    18,   137,   188,    18,   177,   178,
     151,    18,   153,   573,   574,   180,     3,     4,     5,   579,
     152,   132,    18,    11,   304,   585,     3,     4,   588,   309,
      18,    18,   312,   313,   314,   315,   316,     5,     5,   599,
     600,    11,    18,   180,   324,     6,     7,   180,    18,   160,
      18,    18,   332,   333,   334,   335,   336,   337,   338,   339,
     340,   341,   342,   343,   344,   345,   346,   347,   180,     5,
     630,     5,     5,     3,     4,     5,     3,   312,   313,   314,
     315,   316,    18,     5,    18,    18,   218,   219,    18,   324,
       9,   371,    10,    12,    13,    18,    18,   332,   333,   334,
     335,   336,   337,   338,   339,   340,   341,   342,   343,   344,
     345,   346,   347,    10,     5,    11,   725,     3,     4,     5,
       5,     5,    18,   305,   306,   307,   308,    18,     8,   311,
       5,    10,    18,    18,    18,   317,   371,   319,   320,   321,
     322,   310,     5,    18,   326,    10,   328,   329,   330,   331,
       5,  1015,     5,    11,   323,    18,   325,    11,   327,    10,
      18,    10,   515,    18,    18,    18,   348,    47,    48,    49,
      50,    51,    52,    53,     8,    55,    56,    57,    58,    10,
     349,   350,   351,   352,   353,   354,   355,   356,   357,   358,
     359,   360,   361,   362,   363,   364,   365,   366,   367,   368,
     369,   534,   117,     6,     7,    11,   121,     5,    10,     3,
       4,     5,    18,    47,    48,    49,    50,    51,    52,    53,
      18,    10,   137,   138,    18,    10,   141,   142,     3,     4,
       5,    11,   147,   148,   149,    12,    13,   152,    18,   408,
      10,   109,   110,    18,     5,   525,   114,     3,     4,     5,
      11,    14,    15,    16,    17,   864,    11,   537,    11,   868,
     540,   129,    18,    18,    11,    18,   823,    11,    31,    32,
      33,    18,   441,    11,   217,    11,   144,    11,    10,   448,
      18,   561,    18,   151,    18,   153,    10,   567,   562,   563,
     525,   207,   208,   209,   210,   211,   212,   213,   214,    10,
     180,    10,   537,    10,   184,   540,   586,    36,    37,    38,
      39,    40,    41,    42,    43,    44,    45,    46,   719,     3,
       4,     5,   491,    10,     5,   726,   561,   887,   522,   938,
      11,    10,   567,    10,   894,    10,   896,   897,   898,   899,
      54,    55,    56,    57,   944,   945,   180,     3,     4,     5,
     184,     3,     4,     5,   659,    10,   661,   217,   218,   219,
     220,    10,   544,     3,     4,     5,   548,    14,    15,    16,
      17,   559,   986,   987,   562,   563,    10,    10,   560,   712,
      10,    10,   564,   565,   566,    10,   568,   692,   693,   679,
     999,   679,   687,   660,    10,    10,   704,   126,   706,   694,
     873,   679,   710,   693,   709,   693,   879,   880,   676,   677,
     678,    10,   156,   721,   664,   693,   688,    10,   668,   669,
      10,   689,   690,   691,   704,    10,   706,   695,   172,    10,
     710,    55,    56,    57,    58,    10,    10,   687,   688,    10,
      10,   721,   186,    10,   694,    10,    10,   191,   192,     3,
     217,   195,     3,   217,   198,   217,    17,   201,   202,   203,
     204,   655,    12,    13,    14,    15,    16,    17,    54,    55,
      56,    57,   216,   217,   218,   219,   220,   757,    28,    29,
      30,    31,    32,    33,   217,   218,   219,   217,   657,   658,
     659,     7,   661,    10,    57,   664,    55,    56,    57,   668,
     669,    10,   782,    10,     5,   785,   786,     5,     5,   789,
      10,     5,   792,   793,   794,     5,   698,    10,   687,   688,
       5,   703,   757,   692,   693,   694,    10,     5,    57,     5,
       5,     5,   714,     5,     5,   704,    57,   706,     5,     5,
     709,   710,     5,     5,   877,     5,     5,   782,     5,     5,
     785,   786,   721,     5,   789,     5,     5,   792,   793,   794,
       5,     5,     5,     5,     5,     5,   748,     5,     5,     8,
     752,     5,     5,     5,   216,     5,   217,     5,   760,   761,
     217,     5,   764,   863,   766,   767,    11,     5,    11,   771,
       5,   773,   774,    14,    15,    16,    17,    11,     5,   907,
       8,   881,   784,     5,     5,    11,    11,    28,    29,    30,
      31,    32,    33,   795,   796,    11,     5,     3,   800,     9,
     217,     9,     9,    54,     8,    10,     9,   907,   863,     8,
       5,     9,     8,   216,   939,     9,     9,     9,     9,    47,
      48,    49,    50,    51,    52,    53,   881,    55,    56,    57,
      58,     5,   821,    11,   823,    11,    11,   925,    11,    11,
     928,    11,   157,   158,   159,    11,   161,    11,    11,     5,
      11,    11,   167,   168,   169,   980,    11,    11,    11,    11,
      11,    11,   177,   178,     5,   965,     5,    57,    57,     9,
       5,     9,     9,     5,     5,    11,   191,    36,    37,    38,
      39,    40,    41,    42,    43,    44,    45,    46,    36,    37,
      38,    39,    40,    41,    42,    43,    44,    45,    46,    11,
      11,     3,     5,   917,     9,     9,     5,   536,     9,     9,
     965,     8,     5,    11,   189,    11,    11,    11,   907,    11,
      11,    11,    11,     5,   659,     8,   166,   662,     8,   166,
       9,     9,   970,   166,   936,   712,   980,   703,   721,   941,
     704,   710,   178,   932,   933,   706,    94,   127,   569,   131,
     939,    -1,   180,   693,    -1,    -1,   184,   692,   693,   127,
     968,   696,   697,    -1,   966,    -1,    -1,   126,   127,   971,
      -1,    -1,    -1,    -1,    -1,    -1,   984,    -1,   126,   127,
      -1,    -1,    -1,   997,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   980,    -1,    -1,   309,    -1,    -1,   312,   313,   314,
     315,   316,   150,    -1,    -1,    -1,  1008,   155,  1016,   324,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   332,   333,   334,
     335,   336,   337,   338,   339,   340,   341,   342,   343,   344,
     345,   346,   347,    -1,   156,    -1,     0,     1,    -1,     3,
       4,    -1,     6,    -1,    -1,    -1,    -1,    -1,    12,    13,
     172,    -1,    -1,    -1,    -1,    -1,   371,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   186,    -1,    -1,    -1,    -1,   191,
     192,    -1,    -1,   195,    -1,    -1,   198,    -1,    -1,   201,
     202,   203,   204,    -1,    -1,   207,   208,   209,   210,   211,
     212,   213,   214,    -1,   216,   217,   218,   219,   220,    -1,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,    77,    78,    79,    80,    81,    82,    83,
      84,    85,    86,    87,    88,    89,    90,    91,    92,    93,
      94,    95,    96,    97,    98,    99,   100,   101,   102,   103,
     104,   105,   106,   107,   108,   109,   110,   111,   112,   113,
     114,   115,   116,   117,   118,   119,   120,   121,    -1,    -1,
     124,   125,    -1,    -1,   128,   129,    -1,   131,    -1,   133,
     134,   135,   136,   137,   138,   139,   140,   141,   142,   143,
     144,   145,   146,   147,   148,   149,    -1,   151,   152,   153,
     154,    -1,   156,   157,   158,   159,   160,   161,    36,    37,
      38,    39,    40,    41,    42,    43,    44,    45,    46,    -1,
     525,   175,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   537,    -1,    -1,   540,    -1,    -1,    -1,   156,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   172,   561,    -1,    -1,    -1,
      -1,    -1,   567,   217,   218,   219,    94,    -1,    -1,   186,
      -1,    -1,    -1,    -1,   191,   192,    -1,    -1,   195,    -1,
      -1,   198,    -1,    -1,   201,   202,   203,   204,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   126,   216,
     217,   218,   219,   220,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   150,    -1,    -1,    -1,    -1,   155,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,     8,    -1,    10,
      -1,    12,    13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      12,    13,    14,    15,    16,    17,    27,    19,    20,    21,
      22,    23,    24,    34,    -1,    -1,    28,    29,    30,    31,
      32,    33,    -1,    -1,    -1,    -1,    47,    48,    49,    50,
      51,    52,    53,    -1,    -1,    -1,    57,    58,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   704,
      -1,   706,    -1,     5,    -1,   710,    -1,    -1,    -1,    -1,
      12,    13,    14,    15,    16,    17,   721,    19,    20,    21,
      22,    23,    24,    25,    26,    -1,    28,    29,    30,    31,
      32,    33,    -1,    -1,    -1,    -1,    11,    12,    13,    14,
      15,    16,    17,    -1,    19,    20,    21,    22,    23,    24,
      25,    26,   757,    28,    29,    30,    31,    32,    33,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   782,    -1,    -1,
     785,   786,    -1,    -1,   789,   156,    -1,   792,   793,   794,
      -1,   162,   163,   164,   165,   166,   167,   168,   169,   170,
     171,   172,   173,   174,   175,   176,   177,   178,   179,   180,
     181,   182,   183,   184,   185,   186,   187,   188,   189,   190,
     191,   192,   193,   194,   195,   196,   197,   198,   199,   200,
     201,   202,   203,   204,   205,   206,    -1,    -1,     8,    -1,
      10,    -1,    12,    13,   215,   216,   217,   218,   219,   220,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    27,   863,    -1,
      -1,    -1,    -1,    -1,    34,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   881,    47,    48,    49,
      50,    51,    52,    53,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   907,     8,    -1,    10,    -1,    12,    13,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    27,    -1,    -1,    -1,    -1,    -1,    -1,    34,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    47,    48,    49,    50,    51,    52,    53,    -1,
      -1,    -1,    -1,    11,    12,    13,    14,    15,    16,    17,
     965,    19,    20,    21,    22,    23,    24,    25,    26,    -1,
      28,    29,    30,    31,    32,    33,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   156,    -1,    -1,    -1,
      -1,    -1,   162,   163,   164,   165,   166,   167,   168,   169,
     170,   171,   172,   173,   174,   175,   176,   177,   178,   179,
     180,   181,   182,   183,   184,   185,   186,   187,   188,   189,
     190,   191,   192,   193,   194,   195,   196,   197,   198,   199,
     200,   201,   202,   203,   204,   205,   206,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   215,   216,   217,   218,   219,
     220,   156,    -1,    -1,    -1,    -1,    -1,   162,   163,   164,
     165,   166,   167,   168,   169,   170,   171,   172,   173,   174,
     175,   176,   177,   178,   179,   180,   181,   182,   183,   184,
     185,   186,   187,   188,   189,   190,   191,   192,   193,   194,
     195,   196,   197,   198,   199,   200,   201,   202,   203,   204,
     205,   206,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,
     215,   216,   217,   218,   219,   220,    -1,    -1,    -1,    27,
      -1,    -1,    -1,    -1,    -1,    -1,    34,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    49,    -1,    -1,    -1,    -1,    -1,    55,    56,    57,
      -1,    59,    60,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    10,    -1,    12,
      13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    27,    -1,    -1,    -1,    -1,    -1,
      -1,    34,    12,    13,    14,    15,    16,    17,    -1,    19,
      20,    21,    22,    23,    24,    25,    49,    -1,    28,    29,
      30,    31,    32,    33,    57,    -1,    -1,    -1,    61,    62,
      63,    11,    12,    13,    14,    15,    16,    17,    -1,    19,
      20,    21,    22,    23,    24,    25,    26,    -1,    28,    29,
      30,    31,    32,    33,    -1,    -1,    -1,    -1,   156,    -1,
      -1,    -1,    -1,    -1,   162,   163,   164,   165,   166,   167,
     168,   169,   170,   171,   172,   173,   174,   175,   176,   177,
     178,   179,   180,   181,   182,   183,   184,   185,   186,   187,
     188,   189,   190,   191,   192,   193,   194,   195,   196,   197,
     198,   199,   200,   201,   202,   203,   204,   205,   206,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   215,   216,   217,
     218,   219,   220,   156,    -1,    -1,    -1,    -1,    -1,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,   198,   199,   200,   201,   202,
     203,   204,   205,   206,    10,    -1,    12,    13,    -1,    -1,
      -1,    -1,   215,   216,   217,   218,   219,   220,    -1,    -1,
      -1,    27,    -1,    -1,    -1,    -1,    -1,    -1,    34,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    49,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    61,    62,    63,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    10,
      -1,    12,    13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    27,    12,    13,    14,
      15,    16,    17,    34,    19,    20,    21,    22,    23,    24,
      25,    26,    -1,    28,    29,    30,    31,    32,    33,    -1,
      -1,    -1,    -1,    54,    55,    56,    57,    11,    12,    13,
      14,    15,    16,    17,    -1,    19,    20,    21,    22,    23,
      24,    25,    26,    -1,    28,    29,    30,    31,    32,    33,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     156,    -1,    -1,    -1,    -1,    -1,   162,   163,   164,   165,
     166,   167,   168,   169,   170,   171,   172,   173,   174,   175,
     176,   177,   178,   179,   180,   181,   182,   183,   184,   185,
     186,   187,   188,   189,   190,   191,   192,   193,   194,   195,
     196,   197,   198,   199,   200,   201,   202,   203,   204,   205,
     206,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   215,
     216,   217,   218,   219,   220,   156,    -1,    -1,    -1,    -1,
      -1,   162,   163,   164,   165,   166,   167,   168,   169,   170,
     171,   172,   173,   174,   175,   176,   177,   178,   179,   180,
     181,   182,   183,   184,   185,   186,   187,   188,   189,   190,
     191,   192,   193,   194,   195,   196,   197,   198,   199,   200,
     201,   202,   203,   204,   205,   206,    10,    -1,    12,    13,
      -1,    -1,    -1,    -1,   215,   216,   217,   218,   219,   220,
      -1,    -1,    -1,    27,    12,    13,    14,    15,    16,    17,
      34,    19,    20,    21,    22,    23,    24,    25,    26,    -1,
      28,    29,    30,    31,    32,    33,    -1,    -1,    -1,    -1,
      -1,    55,    56,    57,    58,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    27,    -1,
      -1,    -1,    -1,    -1,    -1,    34,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    55,    56,    57,    11,
      12,    13,    14,    15,    16,    17,    -1,    19,    20,    21,
      22,    23,    24,    25,    26,    -1,    28,    29,    30,    31,
      32,    33,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   156,    -1,    -1,    -1,    -1,    -1,   162,   163,
     164,   165,   166,   167,   168,   169,   170,   171,   172,   173,
     174,   175,   176,   177,   178,   179,   180,   181,   182,   183,
     184,   185,   186,   187,   188,   189,   190,   191,   192,   193,
     194,   195,   196,   197,   198,   199,   200,   201,   202,   203,
     204,   205,   206,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   215,   216,   217,   218,   219,   220,   156,    -1,    -1,
      -1,    -1,    -1,   162,   163,   164,   165,   166,   167,   168,
     169,   170,   171,   172,   173,   174,   175,   176,   177,   178,
     179,   180,   181,   182,   183,   184,   185,   186,   187,   188,
     189,   190,   191,   192,   193,   194,   195,   196,   197,   198,
     199,   200,   201,   202,   203,   204,   205,   206,    10,    -1,
      12,    13,    -1,    -1,    -1,    -1,   215,   216,   217,   218,
     219,   220,    -1,    -1,    -1,    27,    -1,    -1,    -1,    -1,
      -1,    -1,    34,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    55,    56,    57,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    10,    -1,    12,    13,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      27,    -1,    -1,    -1,    -1,    -1,    -1,    34,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    55,    56,
      57,    11,    12,    13,    14,    15,    16,    17,    -1,    19,
      20,    21,    22,    23,    24,    25,    26,    -1,    28,    29,
      30,    31,    32,    33,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   156,    -1,    -1,    -1,    -1,    -1,
     162,   163,   164,   165,   166,   167,   168,   169,   170,   171,
     172,   173,   174,   175,   176,   177,   178,   179,   180,   181,
     182,   183,   184,   185,   186,   187,   188,   189,   190,   191,
     192,   193,   194,   195,   196,   197,   198,   199,   200,   201,
     202,   203,   204,   205,   206,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   215,   216,   217,   218,   219,   220,   156,
      -1,    -1,    -1,    -1,    -1,   162,   163,   164,   165,   166,
     167,   168,   169,   170,   171,   172,   173,   174,   175,   176,
     177,   178,   179,   180,   181,   182,   183,   184,   185,   186,
     187,   188,   189,   190,   191,   192,   193,   194,   195,   196,
     197,   198,   199,   200,   201,   202,   203,   204,   205,   206,
      10,    -1,    12,    13,    -1,    -1,    -1,    -1,   215,   216,
     217,   218,   219,   220,    -1,    -1,    -1,    27,    -1,    -1,
      -1,    -1,    -1,    -1,    34,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    10,    -1,    12,    13,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    27,    -1,    -1,    -1,    -1,    -1,    -1,    34,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     110,    -1,    -1,    -1,    49,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   156,    -1,    -1,    -1,
      -1,    -1,   162,   163,   164,   165,   166,   167,   168,   169,
     170,   171,   172,   173,   174,   175,   176,   177,   178,   179,
     180,   181,   182,   183,   184,   185,   186,   187,   188,   189,
     190,   191,   192,   193,   194,   195,   196,   197,   198,   199,
     200,   201,   202,   203,   204,   205,   206,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   215,   216,   217,   218,   219,
     220,   156,    -1,    -1,    -1,    -1,    -1,   162,   163,   164,
     165,   166,   167,   168,   169,   170,   171,   172,   173,   174,
     175,   176,   177,   178,   179,   180,   181,   182,   183,   184,
     185,   186,   187,   188,   189,   190,   191,   192,   193,   194,
     195,   196,   197,   198,   199,   200,   201,   202,   203,   204,
     205,   206,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,
     215,   216,   217,   218,   219,   220,    -1,    -1,    -1,    27,
      -1,    -1,    -1,    -1,    -1,    -1,    34,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    55,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    10,    -1,    12,
      13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    27,    -1,    -1,    -1,    -1,    -1,
      -1,    34,    64,    65,    66,    67,    68,    69,    70,    71,
      72,    73,    74,    75,    76,    77,    78,    79,    80,    81,
      82,    83,    84,    85,    86,    87,    88,    89,    90,    91,
      92,    93,    94,    95,    96,    97,    98,    99,   100,   101,
     102,   103,   104,   105,   106,   107,   108,   109,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   156,    -1,
      -1,    -1,    -1,    -1,   162,   163,   164,   165,   166,   167,
     168,   169,   170,   171,   172,   173,   174,   175,   176,   177,
     178,   179,   180,   181,   182,   183,   184,   185,   186,   187,
     188,   189,   190,   191,   192,   193,   194,   195,   196,   197,
     198,   199,   200,   201,   202,   203,   204,   205,   206,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   215,   216,   217,
     218,   219,   220,   156,    -1,    -1,    -1,    -1,    -1,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,   198,   199,   200,   201,   202,
     203,   204,   205,   206,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   215,   216,   217,   218,   219,   220,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
      76,    77,    78,    79,    80,    81,    82,    83,    84,    85,
      86,    87,    88,    89,    90,    91,    92,    93,    94,    95,
      96,    97,    98,    99,   100,   101,   102,   103,   104,   105,
     106,   107,   108,   109,   110,   111,   112,   113,   114,   115,
     116,   117,    -1,    -1,   120,   121,    -1,    -1,   124,   125,
      -1,    -1,   128,   129,    -1,    -1,    -1,    -1,   134,   135,
     136,    -1,   138,   139,   140,   141,   142,   143,   144,   145,
     146,   147,   148,   149,    -1,   151,    -1,   153,   154,    -1,
     156,   157,   158,   159,   160,   161,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   175,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   217
  };

  const short
  parser::yystos_[] =
  {
       0,     1,   222,   223,   224,     0,    12,    13,   225,     3,
       4,   227,     6,   118,   119,   131,   133,   137,   152,   218,
     219,   226,   228,   229,   230,   231,   232,   240,   268,   269,
     274,   275,   305,    10,    12,    13,    27,    34,   156,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,   198,   199,   200,   201,   202,
     203,   204,   205,   206,   215,   216,   217,   218,   219,   220,
     238,   239,   328,   329,   332,   334,   335,     3,   270,   332,
     276,   331,   332,     6,     7,     6,     7,   227,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
      76,    77,    78,    79,    80,    81,    82,    83,    84,    85,
      86,    87,    88,    89,    90,    91,    92,    93,    94,    95,
      96,    97,    98,    99,   100,   101,   102,   103,   104,   105,
     106,   107,   108,   109,   110,   111,   112,   113,   114,   115,
     116,   117,   120,   121,   124,   125,   128,   129,   134,   135,
     136,   138,   139,   140,   141,   142,   143,   144,   145,   146,
     147,   148,   149,   151,   153,   154,   156,   157,   158,   159,
     160,   161,   175,   217,   233,   234,   236,   241,   244,   245,
     248,   250,   254,   255,   258,   259,   260,   261,   262,   264,
     265,   267,   279,   280,   282,   283,   284,   285,   287,   288,
     289,   290,   291,   292,   293,   294,   295,   296,   297,   298,
     301,   304,   306,   307,   309,   310,   311,   312,   313,   314,
     315,   339,   340,   345,   346,   347,   348,   349,   350,   351,
     352,   353,   354,   355,   356,   357,   358,   359,   360,   361,
     362,   363,   364,   365,   368,   369,   370,   371,   372,   373,
     374,   375,   376,   377,   378,   379,   380,   381,   382,   383,
     384,   385,   386,   387,   388,   389,   390,   391,   392,   393,
     394,   395,   396,   397,   398,   399,   400,   401,   402,   328,
     328,   328,   328,   328,    10,    10,    10,    10,    10,    10,
      10,    10,    10,    10,    10,    10,    10,    10,   330,    10,
      10,    10,    10,    10,    10,    10,    10,    10,    10,    10,
      10,    10,    10,    10,    10,    10,    10,    10,    10,    10,
      10,    10,    10,    10,    10,    10,    10,    10,    10,    10,
      12,    13,    14,    15,    16,    17,    19,    20,    21,    22,
      23,    24,    25,    26,    28,    29,    30,    31,    32,    33,
       3,    18,   217,     3,   217,     3,     8,    47,    48,    49,
      50,    51,    52,    53,   180,   184,   325,   328,   404,   405,
     406,   407,   408,   409,   410,   411,   412,   415,   416,   417,
      57,    58,   404,   405,   404,   405,   318,   328,    27,    49,
      61,    62,    63,   327,   328,   413,   414,   404,   405,   180,
     406,    55,    56,    57,    58,   180,   184,   406,   415,   419,
     420,   421,   415,   419,    57,   327,   413,   327,   413,     8,
     406,     8,   366,   367,   403,   406,     8,   406,     8,    57,
      58,   366,   367,   403,   416,   417,   421,   422,   423,   424,
     404,   405,    54,   418,   420,   418,   318,    27,   413,   415,
     415,   415,   415,   325,   404,   405,   318,   415,   415,   415,
     325,   404,   405,   415,   404,   405,   249,   331,   129,   130,
     161,   263,   227,   239,   334,   335,   239,   319,   320,   328,
     329,   334,   239,   323,   324,   329,   334,   110,   331,   239,
     321,   322,   329,   334,   234,   236,   238,   245,   302,   303,
     335,   335,   335,   132,   160,   341,   217,   251,   239,   316,
     317,   329,   334,   316,   299,   217,   256,   341,   237,   331,
     341,   217,   266,   332,   263,   335,   235,   242,    36,    37,
      38,    39,    40,    41,    42,    43,    44,    45,    46,    94,
     126,   127,   150,   155,   247,    36,   126,   127,   247,     7,
      11,   239,   332,   332,   332,   332,   239,   334,   328,   332,
     335,   335,   335,   335,   335,   332,    10,   332,   332,   332,
     332,   328,   335,   328,   332,   328,   332,   308,   332,   332,
     332,   207,   208,   209,   210,   211,   212,   213,   214,   335,
     342,   335,   342,    11,   335,   336,   335,   335,   335,   335,
     337,   335,   335,   335,   335,   335,   335,   335,   335,   335,
     332,   328,   328,   328,   328,   328,   328,   328,   328,   328,
     328,   328,   328,   328,   328,   328,   328,   328,   328,   328,
     328,   328,   335,   271,   277,   272,    57,    10,    10,     5,
       5,     5,     5,   413,     5,    10,    10,    10,     5,     5,
      57,     5,   184,   327,   328,   408,     5,     5,     5,     5,
      57,     5,    55,    56,    57,    59,    60,     5,     5,     5,
       5,     5,     5,     5,     5,     5,     5,     5,     5,     5,
       5,     5,   328,     5,     5,   246,     5,   246,     8,     5,
       5,   246,     5,   246,     5,   227,   335,     5,   216,   252,
     253,     5,   246,   246,   238,   300,   252,   257,   335,   217,
     335,   332,   217,   243,   332,   281,   331,   332,   335,   281,
     281,   332,   332,   332,   335,   332,   345,    11,     5,   333,
     333,   333,     5,    11,    11,    11,   333,     5,    11,    11,
       5,     5,   333,   239,     5,   333,     5,     5,    11,    11,
      11,     5,    11,     5,     5,    11,   333,   333,    11,    11,
      11,    11,     5,    11,     5,     5,     5,   338,    11,     5,
      11,    11,     5,     5,     5,     5,     5,    11,   333,    11,
       5,     3,   227,     9,    54,    55,    56,    57,    55,    56,
      57,   325,   415,   419,   325,   415,   327,   327,   327,     9,
       8,    10,     9,    12,     9,   406,   406,   406,   366,   367,
     403,     9,     8,     9,     9,    12,    13,     9,     9,    58,
     327,   420,    57,   327,   421,   406,   406,    58,   406,   325,
     415,   325,   366,   367,   403,   415,   422,   327,   420,   406,
     415,   415,   332,     5,   308,   320,   324,   249,   286,   325,
     322,   303,   332,     5,   217,   253,   317,     5,   246,     5,
       5,     5,   216,   332,    11,    11,    11,   332,    11,   335,
     332,   332,    11,    11,   332,    11,   332,   332,   332,   332,
     332,    11,    11,   335,   332,   335,   335,     5,   335,   335,
     335,   335,   332,   332,    11,   273,   332,   278,    11,    11,
      11,    11,    11,    11,    11,     5,    57,   408,     5,    57,
       9,     9,    12,    13,   326,   335,     5,   246,     9,     5,
     246,     5,   342,   238,   342,   342,   335,   333,    11,    11,
      11,   333,   333,   333,   333,   333,    11,    11,    11,   239,
     329,   334,    11,    11,    11,     5,     5,    11,     5,    11,
       3,     5,   227,   406,     9,     9,   406,     9,   328,   328,
       5,   246,   325,   332,     8,   343,   343,   343,    11,    11,
      11,    11,    11,    11,   335,   332,   331,   272,   332,   286,
     331,   344,   344,   344,    11,    11,    11,   227,     5,   246,
       9,     5,   332,   110,   166,     8,     8,   249,   331,     9,
       9
  };

  const short
  parser::yyr1_[] =
  {
       0,   221,   222,   223,   223,   224,   223,   225,   225,   225,
     226,   226,   227,   227,   228,   228,   228,   228,   228,   228,
     228,   228,   229,   230,   231,   232,   232,   232,   232,   233,
     235,   234,   237,   236,   238,   238,   238,   239,   239,   240,
     240,   240,   240,   240,   240,   240,   242,   241,   243,   243,
     244,   244,   244,   244,   244,   244,   244,   244,   244,   244,
     244,   244,   244,   244,   244,   244,   244,   244,   244,   244,
     244,   244,   244,   244,   244,   244,   244,   244,   244,   244,
     244,   244,   244,   244,   244,   244,   244,   244,   244,   245,
     245,   245,   245,   245,   245,   246,   246,   247,   247,   247,
     247,   247,   247,   247,   247,   247,   247,   248,   249,   249,
     251,   250,   252,   252,   253,   254,   256,   255,   257,   257,
     258,   259,   260,   261,   262,   263,   263,   263,   263,   264,
     264,   264,   264,   265,   266,   266,   267,   267,   268,   270,
     271,   269,   272,   273,   273,   273,   274,   276,   277,   275,
     278,   279,   280,   281,   281,   282,   283,   284,   285,   285,
     285,   285,   286,   286,   287,   287,   288,   288,   289,   289,
     290,   291,   292,   292,   292,   292,   293,   294,   295,   296,
     297,   299,   298,   300,   300,   301,   302,   302,   303,   304,
     305,   306,   306,   306,   307,   308,   308,   309,   309,   310,
     311,   312,   313,   314,   315,   315,   316,   316,   317,   317,
     317,   318,   319,   319,   320,   320,   320,   321,   321,   322,
     322,   322,   323,   323,   324,   324,   324,   325,   326,   326,
     327,   328,   328,   328,   329,   329,   329,   329,   329,   329,
     329,   329,   329,   329,   329,   329,   329,   329,   329,   329,
     329,   329,   329,   329,   329,   329,   329,   329,   329,   329,
     329,   329,   329,   329,   329,   329,   329,   329,   329,   329,
     330,   329,   329,   329,   329,   329,   329,   329,   329,   329,
     329,   329,   329,   329,   329,   329,   329,   329,   329,   329,
     329,   329,   329,   329,   329,   329,   329,   329,   329,   331,
     332,   333,   333,   334,   334,   334,   334,   334,   334,   334,
     334,   334,   334,   334,   334,   334,   334,   334,   334,   335,
     335,   336,   336,   337,   337,   338,   338,   338,   338,   339,
     340,   341,   341,   341,   342,   342,   342,   342,   342,   342,
     342,   342,   343,   343,   344,   344,   344,   345,   345,   346,
     346,   346,   346,   346,   346,   346,   346,   346,   346,   346,
     346,   346,   346,   346,   346,   346,   346,   346,   346,   346,
     346,   346,   346,   346,   346,   346,   346,   346,   346,   346,
     346,   346,   346,   346,   346,   346,   346,   346,   346,   346,
     346,   346,   346,   346,   346,   347,   347,   348,   348,   348,
     348,   349,   349,   350,   351,   351,   352,   353,   353,   354,
     354,   355,   356,   356,   357,   358,   359,   360,   360,   361,
     361,   361,   362,   362,   363,   363,   364,   364,   365,   365,
     365,   365,   365,   365,   366,   367,   368,   368,   368,   368,
     368,   368,   368,   368,   369,   369,   369,   369,   370,   370,
     370,   371,   371,   372,   372,   373,   374,   374,   375,   375,
     375,   375,   375,   375,   376,   376,   377,   378,   378,   379,
     380,   381,   382,   382,   383,   384,   385,   386,   387,   388,
     389,   390,   391,   392,   393,   393,   394,   395,   396,   397,
     398,   399,   399,   400,   400,   401,   402,   402,   403,   404,
     404,   405,   405,   406,   406,   407,   407,   408,   408,   409,
     409,   410,   410,   411,   411,   412,   412,   413,   413,   414,
     414,   414,   414,   415,   415,   416,   416,   416,   416,   416,
     416,   416,   417,   418,   418,   419,   419,   420,   420,   421,
     421,   422,   422,   422,   422,   423,   423,   424,   424
  };

  const signed char
  parser::yyr2_[] =
  {
       0,     2,     1,     0,     3,     0,     3,     0,     1,     1,
       2,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     3,     3,     2,     1,     2,     2,     2,     1,
       0,     3,     0,     3,     1,     1,     1,     1,     1,     0,
       2,     2,     1,     2,     2,     1,     0,     3,     0,     2,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     0,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     2,     1,     3,
       0,     3,     1,     2,     1,     1,     0,     3,     0,     1,
       1,     1,     1,     2,     2,     0,     2,     2,     2,     3,
       5,     3,     5,     2,     0,     1,     7,     1,     5,     0,
       0,     9,     0,     1,     3,     5,     3,     0,     0,     7,
       0,     2,     1,     0,     1,     1,     1,     1,     2,     5,
       6,     8,     1,     3,     1,     3,     1,     3,     1,     3,
       3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
       3,     0,     4,     1,     3,     3,     1,     3,     1,     2,
       4,     2,     4,     6,     5,     1,     3,     2,     4,     2,
       1,     2,     1,     3,     1,     3,     1,     3,     1,     1,
       1,     1,     1,     3,     1,     1,     1,     1,     3,     1,
       1,     1,     1,     3,     1,     1,     1,     1,     2,     2,
       1,     1,     1,     1,     1,     2,     3,     3,     3,     3,
       3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     3,     3,     2,     2,     2,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       0,     5,     5,     5,     5,     7,     7,     7,     7,     7,
       5,     5,     5,     5,     5,     5,     7,     6,     6,     6,
       6,     6,     4,     4,     4,     6,     4,     6,     3,     1,
       1,     0,     2,     1,     3,     8,     6,     8,     6,     6,
       6,     4,     3,     4,     4,     4,     8,     4,     4,     1,
       1,     1,     3,     0,     2,     0,     3,     3,     3,     7,
       7,     0,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     0,     3,     0,     6,     6,     1,     3,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     2,     2,     2,     2,     4,
       4,     2,     2,     4,     2,     4,     1,     2,     2,     1,
       2,     1,     2,     2,     1,     1,     1,     2,     2,     2,
       4,     2,     2,     4,     6,     6,     6,     6,     4,     4,
       4,     4,     4,     4,     3,     5,     1,     1,     1,     1,
       1,     1,     1,     1,     5,     4,     4,     4,     4,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     4,     4,     4,     4,     1,     2,     2,     2,
       2,     4,     1,     2,     1,     2,     1,     2,     1,     2,
       1,     2,     1,     2,     2,     2,     1,     4,     2,     2,
       2,     1,     2,     2,     2,     2,     2,     2,     3,     1,
       3,     1,     3,     1,     4,     1,     4,     1,     4,     1,
       4,     1,     4,     1,     4,     1,     4,     1,     2,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     3,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     3,     3,     1,     1,     3,     4,     3,     4
  };




#if YYDEBUG
  const short
  parser::yyrline_[] =
  {
       0,   423,   423,   426,   427,   429,   429,   438,   439,   444,
     454,   455,   458,   458,   465,   466,   467,   468,   469,   470,
     472,   473,   477,   489,   507,   526,   527,   528,   529,   533,
     539,   539,   548,   548,   556,   556,   556,   558,   558,   561,
     562,   565,   569,   572,   575,   579,   585,   585,   594,   597,
     604,   605,   606,   607,   608,   609,   610,   611,   612,   613,
     614,   615,   616,   617,   618,   619,   620,   621,   622,   623,
     624,   625,   626,   627,   628,   629,   630,   631,   632,   633,
     634,   635,   636,   637,   638,   639,   640,   641,   642,   646,
     647,   648,   649,   650,   651,   654,   654,   657,   660,   663,
     666,   669,   672,   675,   678,   681,   684,   690,   696,   705,
     724,   724,   731,   732,   736,   742,   748,   748,   756,   759,
     763,   769,   775,   781,   787,   793,   796,   799,   802,   808,
     815,   822,   827,   835,   845,   848,   852,   855,   861,   869,
     871,   869,   881,   887,   892,   897,   905,   913,   915,   913,
     925,   931,   937,   943,   946,   950,   956,   962,   968,   971,
     974,   980,   989,   992,   999,  1002,  1006,  1009,  1013,  1016,
    1020,  1027,  1034,  1038,  1042,  1046,  1053,  1062,  1071,  1080,
    1087,  1094,  1094,  1105,  1108,  1114,  1117,  1118,  1122,  1128,
    1134,  1143,  1149,  1155,  1164,  1170,  1173,  1180,  1183,  1189,
    1195,  1201,  1208,  1213,  1216,  1220,  1227,  1228,  1232,  1235,
    1239,  1249,  1256,  1257,  1261,  1265,  1269,  1285,  1286,  1290,
    1294,  1298,  1314,  1315,  1319,  1322,  1326,  1339,  1346,  1350,
    1357,  1364,  1367,  1371,  1386,  1389,  1392,  1395,  1398,  1401,
    1404,  1407,  1410,  1413,  1416,  1419,  1422,  1425,  1428,  1431,
    1434,  1437,  1440,  1443,  1446,  1449,  1452,  1455,  1458,  1461,
    1464,  1467,  1470,  1473,  1476,  1480,  1483,  1486,  1489,  1492,
    1495,  1495,  1501,  1504,  1507,  1510,  1513,  1516,  1519,  1522,
    1525,  1528,  1531,  1534,  1537,  1540,  1543,  1546,  1549,  1553,
    1557,  1561,  1565,  1568,  1571,  1574,  1577,  1584,  1602,  1608,
    1617,  1623,  1626,  1636,  1639,  1643,  1649,  1654,  1659,  1664,
    1669,  1674,  1683,  1686,  1689,  1693,  1697,  1700,  1703,  1725,
    1728,  1738,  1741,  1748,  1749,  1756,  1757,  1761,  1765,  1778,
    1784,  1791,  1794,  1797,  1803,  1806,  1809,  1812,  1815,  1818,
    1821,  1824,  1830,  1833,  1843,  1848,  1853,  1862,  1863,  1867,
    1868,  1869,  1870,  1871,  1872,  1873,  1874,  1875,  1876,  1877,
    1878,  1879,  1880,  1881,  1882,  1883,  1884,  1885,  1886,  1887,
    1888,  1889,  1890,  1891,  1892,  1893,  1894,  1895,  1896,  1897,
    1898,  1899,  1900,  1901,  1902,  1903,  1904,  1905,  1906,  1907,
    1908,  1909,  1910,  1911,  1912,  1916,  1920,  1926,  1930,  1933,
    1936,  1943,  1947,  1953,  1966,  1970,  1977,  1983,  1987,  1993,
    1996,  2002,  2008,  2011,  2017,  2023,  2029,  2035,  2038,  2044,
    2048,  2052,  2058,  2062,  2069,  2072,  2078,  2081,  2087,  2098,
    2109,  2112,  2115,  2118,  2123,  2126,  2135,  2136,  2137,  2138,
    2139,  2140,  2141,  2142,  2146,  2150,  2153,  2157,  2168,  2171,
    2174,  2181,  2185,  2192,  2195,  2202,  2208,  2212,  2222,  2226,
    2229,  2232,  2236,  2239,  2246,  2250,  2266,  2272,  2276,  2282,
    2288,  2294,  2307,  2310,  2316,  2322,  2329,  2335,  2342,  2348,
    2355,  2361,  2368,  2374,  2385,  2389,  2395,  2401,  2414,  2421,
    2428,  2435,  2439,  2446,  2450,  2456,  2463,  2467,  2475,  2481,
    2482,  2488,  2491,  2499,  2500,  2504,  2505,  2509,  2510,  2514,
    2515,  2519,  2520,  2524,  2525,  2529,  2530,  2534,  2535,  2541,
    2544,  2547,  2550,  2555,  2555,  2558,  2561,  2564,  2567,  2570,
    2573,  2576,  2582,  2588,  2589,  2595,  2596,  2602,  2603,  2609,
    2612,  2618,  2621,  2624,  2627,  2633,  2634,  2638,  2639
  };

  void
  parser::yy_stack_print_ () const
  {
    *yycdebug_ << "Stack now";
    for (stack_type::const_iterator
           i = yystack_.begin (),
           i_end = yystack_.end ();
         i != i_end; ++i)
      *yycdebug_ << ' ' << int (i->state);
    *yycdebug_ << '\n';
  }

  void
  parser::yy_reduce_print_ (int yyrule) const
  {
    int yylno = yyrline_[yyrule];
    int yynrhs = yyr2_[yyrule];
    // Print the symbols being reduced, and their result.
    *yycdebug_ << "Reducing stack by rule " << yyrule - 1
               << " (line " << yylno << "):\n";
    // The symbols being reduced.
    for (int yyi = 0; yyi < yynrhs; yyi++)
      YY_SYMBOL_PRINT ("   $" << yyi + 1 << " =",
                       yystack_[(yynrhs) - (yyi + 1)]);
  }
#endif // YYDEBUG


} // yy
#line 6409 "/build/rgbds/src/rgbds/src/asm/parser.cpp"

#line 2642 "asm/parser.y"


/******************** Semantic actions ********************/

void yy::parser::error(std::string const &str) {
	::error("%s\n", str.c_str());
}

static uint32_t strToNum(std::vector<int32_t> const &s) {
	uint32_t length = s.size();

	if (length == 1) {
		// The string is a single character with a single value,
		// which can be used directly as a number.
		return static_cast<uint32_t>(s[0]);
	}

	warning(WARNING_OBSOLETE, "Treating multi-unit strings as numbers is deprecated\n");

	for (int32_t v : s) {
		if (!checkNBit(v, 8, "All character units")) {
			break;
		}
	}

	uint32_t r = 0;

	for (uint32_t i = length < 4 ? 0 : length - 4; i < length; i++) {
		r <<= 8;
		r |= static_cast<uint8_t>(s[i]);
	}

	return r;
}

static void errorInvalidUTF8Byte(uint8_t byte, char const *functionName) {
	error("%s: Invalid UTF-8 byte 0x%02hhX\n", functionName, byte);
}

static size_t strlenUTF8(std::string const &str, bool printErrors) {
	char const *ptr = str.c_str();
	size_t len = 0;
	uint32_t state = 0;

	for (uint32_t codepoint = 0; *ptr; ptr++) {
		uint8_t byte = *ptr;

		switch (decode(&state, &codepoint, byte)) {
		case 1:
			if (printErrors) {
				errorInvalidUTF8Byte(byte, "STRLEN");
			}
			state = 0;
			// fallthrough
		case 0:
			len++;
			break;
		}
	}

	// Check for partial code point.
	if (state != 0) {
		if (printErrors) {
			error("STRLEN: Incomplete UTF-8 character\n");
		}
		len++;
	}

	return len;
}

static std::string strsliceUTF8(std::string const &str, uint32_t start, uint32_t stop) {
	char const *ptr = str.c_str();
	size_t index = 0;
	uint32_t state = 0;
	uint32_t codepoint = 0;
	uint32_t curIdx = 0;

	// Advance to starting index in source string.
	while (ptr[index] && curIdx < start) {
		switch (decode(&state, &codepoint, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSLICE");
			state = 0;
			// fallthrough
		case 0:
			curIdx++;
			break;
		}
		index++;
	}

	// An index 1 past the end of the string is allowed, but will trigger the
	// "Length too big" warning below if the length is nonzero.
	if (!ptr[index] && start > curIdx) {
		warning(
		    WARNING_BUILTIN_ARG,
		    "STRSLICE: Start index %" PRIu32 " is past the end of the string\n",
		    start
		);
	}

	size_t startIndex = index;

	// Advance to ending index in source string.
	while (ptr[index] && curIdx < stop) {
		switch (decode(&state, &codepoint, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSLICE");
			state = 0;
			// fallthrough
		case 0:
			curIdx++;
			break;
		}
		index++;
	}

	// Check for partial code point.
	if (state != 0) {
		error("STRSLICE: Incomplete UTF-8 character\n");
		curIdx++;
	}

	if (curIdx < stop) {
		warning(
		    WARNING_BUILTIN_ARG,
		    "STRSLICE: Stop index %" PRIu32 " is past the end of the string\n",
		    stop
		);
	}

	return std::string(ptr + startIndex, ptr + index);
}

static std::string strsubUTF8(std::string const &str, uint32_t pos, uint32_t len) {
	char const *ptr = str.c_str();
	size_t index = 0;
	uint32_t state = 0;
	uint32_t codepoint = 0;
	uint32_t curPos = 1;

	// Advance to starting position in source string.
	while (ptr[index] && curPos < pos) {
		switch (decode(&state, &codepoint, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSUB");
			state = 0;
			// fallthrough
		case 0:
			curPos++;
			break;
		}
		index++;
	}

	// A position 1 past the end of the string is allowed, but will trigger the
	// "Length too big" warning below if the length is nonzero.
	if (!ptr[index] && pos > curPos) {
		warning(
		    WARNING_BUILTIN_ARG, "STRSUB: Position %" PRIu32 " is past the end of the string\n", pos
		);
	}

	size_t startIndex = index;
	uint32_t curLen = 0;

	// Compute the result length in bytes.
	while (ptr[index] && curLen < len) {
		switch (decode(&state, &codepoint, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSUB");
			state = 0;
			// fallthrough
		case 0:
			curLen++;
			break;
		}
		index++;
	}

	// Check for partial code point.
	if (state != 0) {
		error("STRSUB: Incomplete UTF-8 character\n");
		curLen++;
	}

	if (curLen < len) {
		warning(WARNING_BUILTIN_ARG, "STRSUB: Length too big: %" PRIu32 "\n", len);
	}

	return std::string(ptr + startIndex, ptr + index);
}

static size_t charlenUTF8(std::string const &str) {
	std::string_view view = str;
	size_t len;

	for (len = 0; charmap_ConvertNext(view, nullptr); len++) {}

	return len;
}

static std::string strcharUTF8(std::string const &str, uint32_t idx) {
	std::string_view view = str;
	size_t charLen = 1;

	// Advance to starting index in source string.
	for (uint32_t curIdx = 0; charLen && curIdx < idx; curIdx++) {
		charLen = charmap_ConvertNext(view, nullptr);
	}

	std::string_view start = view;

	if (!charmap_ConvertNext(view, nullptr)) {
		warning(
		    WARNING_BUILTIN_ARG,
		    "STRCHAR: Index %" PRIu32 " is past the end of the string\n",
		    idx
		);
	}

	start = start.substr(0, start.length() - view.length());
	return std::string(start);
}

static std::string charsubUTF8(std::string const &str, uint32_t pos) {
	std::string_view view = str;
	size_t charLen = 1;

	// Advance to starting position in source string.
	for (uint32_t curPos = 1; charLen && curPos < pos; curPos++) {
		charLen = charmap_ConvertNext(view, nullptr);
	}

	std::string_view start = view;

	if (!charmap_ConvertNext(view, nullptr)) {
		warning(
		    WARNING_BUILTIN_ARG,
		    "CHARSUB: Position %" PRIu32 " is past the end of the string\n",
		    pos
		);
	}

	start = start.substr(0, start.length() - view.length());
	return std::string(start);
}

static int32_t charcmp(std::string_view str1, std::string_view str2) {
	std::vector<int32_t> seq1, seq2;
	size_t idx1 = 0, idx2 = 0;
	for (;;) {
		if (idx1 >= seq1.size()) {
			idx1 = 0;
			seq1.clear();
			charmap_ConvertNext(str1, &seq1);
		}
		if (idx2 >= seq2.size()) {
			idx2 = 0;
			seq2.clear();
			charmap_ConvertNext(str2, &seq2);
		}
		if (seq1.empty() != seq2.empty()) {
			return seq1.empty() ? -1 : 1;
		} else if (seq1.empty()) {
			return 0;
		} else {
			int32_t value1 = seq1[idx1++], value2 = seq2[idx2++];
			if (value1 != value2) {
				return (value1 > value2) - (value1 < value2);
			}
		}
	}
}

static uint32_t adjustNegativeIndex(int32_t idx, size_t len, char const *functionName) {
	// String functions adjust negative index arguments the same way,
	// such that position -1 is the last character of a string.
	if (idx < 0) {
		idx += len;
	}
	if (idx < 0) {
		warning(WARNING_BUILTIN_ARG, "%s: Index starts at 0\n", functionName);
		idx = 0;
	}
	return static_cast<uint32_t>(idx);
}

static uint32_t adjustNegativePos(int32_t pos, size_t len, char const *functionName) {
	// STRSUB and CHARSUB adjust negative position arguments the same way,
	// such that position -1 is the last character of a string.
	if (pos < 0) {
		pos += len + 1;
	}
	if (pos < 1) {
		warning(WARNING_BUILTIN_ARG, "%s: Position starts at 1\n", functionName);
		pos = 1;
	}
	return static_cast<uint32_t>(pos);
}

static std::string strrpl(std::string_view str, std::string const &old, std::string const &rep) {
	if (old.empty()) {
		warning(WARNING_EMPTY_STRRPL, "STRRPL: Cannot replace an empty string\n");
		return std::string(str);
	}

	std::string rpl;

	while (!str.empty()) {
		auto pos = str.find(old);
		if (pos == str.npos) {
			rpl.append(str);
			break;
		}
		rpl.append(str, 0, pos);
		rpl.append(rep);
		str.remove_prefix(pos + old.size());
	}

	return rpl;
}

static std::string
    strfmt(std::string const &spec, std::vector<Either<uint32_t, std::string>> const &args) {
	std::string str;
	size_t argIndex = 0;

	for (size_t i = 0; spec[i] != '\0'; ++i) {
		int c = spec[i];

		if (c != '%') {
			str += c;
			continue;
		}

		c = spec[++i];

		if (c == '%') {
			str += c;
			continue;
		}

		FormatSpec fmt{};

		while (c != '\0') {
			fmt.useCharacter(c);
			if (fmt.isFinished()) {
				break;
			}
			c = spec[++i];
		}

		if (fmt.isEmpty()) {
			error("STRFMT: Illegal '%%' at end of format string\n");
			str += '%';
			break;
		}

		if (!fmt.isValid()) {
			error("STRFMT: Invalid format spec for argument %zu\n", argIndex + 1);
			str += '%';
		} else if (argIndex >= args.size()) {
			// Will warn after formatting is done.
			str += '%';
		} else if (args[argIndex].holds<uint32_t>()) {
			fmt.appendNumber(str, args[argIndex].get<uint32_t>());
		} else {
			fmt.appendString(str, args[argIndex].get<std::string>());
		}

		argIndex++;
	}

	if (argIndex < args.size()) {
		error("STRFMT: %zu unformatted argument(s)\n", args.size() - argIndex);
	} else if (argIndex > args.size()) {
		error(
		    "STRFMT: Not enough arguments for format spec, got: %zu, need: %zu\n",
		    args.size(),
		    argIndex
		);
	}

	return str;
}

static void compoundAssignment(std::string const &symName, RPNCommand op, int32_t constValue) {
	Expression oldExpr, constExpr, newExpr;
	int32_t newValue;

	oldExpr.makeSymbol(symName);
	constExpr.makeNumber(constValue);
	newExpr.makeBinaryOp(op, std::move(oldExpr), constExpr);
	newValue = newExpr.getConstVal();
	sym_AddVar(symName, newValue);
}

static void failAssert(AssertionType type) {
	switch (type) {
	case ASSERT_FATAL:
		fatalerror("Assertion failed\n");
	case ASSERT_ERROR:
		error("Assertion failed\n");
		break;
	case ASSERT_WARN:
		warning(WARNING_ASSERT, "Assertion failed\n");
		break;
	}
}

static void failAssertMsg(AssertionType type, std::string const &message) {
	switch (type) {
	case ASSERT_FATAL:
		fatalerror("Assertion failed: %s\n", message.c_str());
	case ASSERT_ERROR:
		error("Assertion failed: %s\n", message.c_str());
		break;
	case ASSERT_WARN:
		warning(WARNING_ASSERT, "Assertion failed: %s\n", message.c_str());
		break;
	}
}
